#include "mfinderapp.h"
#include "functions.h"

extern char confdir[];
extern char iconsPath[];
extern char appsPath[];

void mfinderapp::writeDefaults()
{
	FILE *fConf;
	char fPath[MAX_PATH_LEN] = "", buff[MAX_PATH_LEN] = "";
	struct dirent **dir;
	int m;
	
// --- WINDOWS GEOMETRY ---	
	strcpy(fPath, "winsgeom");
	unlink(fPath);
	fConf = fopen(fPath, "a");
	for(int i = 0; i < wGeomDict.count(); i++) 
		fprintf(fConf, "%s:%s\n", wGeomDict.at(i)->key, wGeomDict.at(i)->value);
	fclose(fConf);	

// --- WINDOWS OPEN ---	
	strcpy(fPath, "openwins");
	unlink(fPath);
	fConf = fopen(fPath, "a");
	for(int i = 0; i < wOpenDict.count(); i++) 
		fprintf(fConf, "%s\n", wOpenDict.at(i));
	fclose(fConf);	

// --- DESK OBJECTS ---	
	strcpy(fPath, "deskobjs");
	unlink(fPath);
	fConf = fopen(fPath, "a");
	for(int i = 0; i < deskObjsDict.count(); i++)
		fprintf(fConf, "%s\n", deskObjsDict.at(i));
	fclose(fConf);	

// --- OBJECTS CUSTOM DATA ---
	strcpy(fPath, "objsprops");
	unlink(fPath);
	fConf = fopen(fPath, "a");
	for(int i = 0; i < filesDict.count(); i++) {
		fileMetaData *fmd = filesDict.at(i);
		fprintf(fConf, "%s:%i|%i|%s|%i|%s\n", fmd->filepath, fmd->isopenlocal, 
			fmd->hascustomicn, fmd->custicnname, fmd->hascustomedit, fmd->custedit);
	}
	fclose(fConf);	

// --- required icons --
	strcpy(fPath, "reqicons");
	unlink(fPath);
	fConf = fopen(fPath, "a");
	for(int i = 0; i < icnReqDict.count(); i++) 
		fprintf(fConf, "%s|%s\n", icnReqDict.at(i)->key, icnReqDict.at(i)->value);
	fclose(fConf);	

// --- user icons --
	strcpy(fPath, "usricons");
	unlink(fPath);
	fConf = fopen(fPath, "a");
	for(int i = 0; i < icnDict.count(); i++) 
		fprintf(fConf, "%s|%s\n", icnDict.at(i)->key, icnDict.at(i)->value);
	fclose(fConf);	
}

char *mfinderapp::rectOfWinAtPath(char *wpath) 
{
	char *r = NULL;

	for(int i = 0; i < wGeomDict.count(); i++) {
		if(strcmp(wGeomDict.at(i)->key, wpath) == 0) {
			r = wGeomDict.at(i)->value;
			break;
		}
	}
	return r;
}

void mfinderapp::addDeskObjToDict(DeskObj *d)
{
	removeDeskObjFromDict(d->myFullPath);	
	deskObjsDict.append(new char[2048]);
	sprintf(deskObjsDict.current(), "%s|%s|%s|%i|%i|%i", 
					d->myFullPath, d->myPath, d->myName, d->myType, d->x, d->y);
}

void mfinderapp::removeDeskObjFromDict(char *dsobpath)
{
	char entry[2048] = "";
	char *token, *delims = "|";	
	int count;

	for(int i = 0; i < deskObjsDict.count(); i++) {
		sprintf(entry, "%s", deskObjsDict.at(i));
		token = strtok(entry, delims);
		if(strcmp(token, dsobpath) == 0) {
			deskObjsDict.remove(i);
			break;
		}
	}
}

void mfinderapp::setIcnReqDictEntryForKey(char *key, char *value)
{
	for(int i = 0; i < icnReqDict.count(); i++) {
		if(strcmp(icnReqDict.at(i)->key, key) == 0) {
			icnReqDict.remove(i);
			break;
		}
	}
	icnReqDict.append(new dictEntry());
	dictEntry *de = icnReqDict.current();
	strcpy(de->key, key);
	strcpy(de->value, value);
}

char *mfinderapp::getIcnReqDictEntryForKey(char *key)
{
	char *entry = NULL;

	for(int i = 0; i < icnReqDict.count(); i++) {
		if(strcmp(icnReqDict.at(i)->key, key) == 0) {
			entry = icnReqDict.at(i)->value;
			break;
		}
	}
	return entry;
}

void mfinderapp::setIcnDictEntryForKey(char *key, char *value)
{
	for(int i = 0; i < icnDict.count(); i++) {
		if(strcmp(icnDict.at(i)->key, key) == 0) {
			icnDict.remove(i);
			break;
		}
	}
	icnDict.append(new dictEntry());
	dictEntry *de = icnDict.current();
	strcpy(de->key, key);
	strcpy(de->value, value);
}

char *mfinderapp::getIcnDictEntryForKey(char *key)
{
	char *entry = NULL;

	for(int i = 0; i < icnDict.count(); i++) {
		if(strcmp(icnDict.at(i)->key, key) == 0) {
			entry = icnDict.at(i)->value;
			break;
		}
	}
	return entry;
}

bool mfinderapp::hasCustomIconFileAtPath(char *fpath)
{
	bool hasicn = FALSE;
	
	for(int i = 0; i < filesDict.count(); i++) {
		fileMetaData *fmd = filesDict.at(i);
		if(strcmp(fmd->filepath, fpath) == 0) {
			if(fmd->hascustomicn) {
				hasicn = TRUE;
				break;
			}
		}		
	}
	return hasicn;
}

char *mfinderapp::customIconNameOfFileAtPath(char *fpath)
{
	char *iname = NULL;
	char fname[MAX_NAME_LEN] = "", buff[128] = "";
	char *pref, *suff, *token, *delims = ".";	
	bool found = FALSE;
	
	for(int i = 0; i < filesDict.count(); i++) {
		fileMetaData *fmd = filesDict.at(i);
		if(strcmp(fmd->filepath, fpath) == 0) {
			iname	= fmd->custicnname;
			break;
		}		
	}
	return iname;
}

void mfinderapp::setCustomIconOfFileAtPath(char *fpath, char *icname)
{
	char buff[MAX_PATH_LEN] = "";
	bool found = FALSE;
	
	for(int i = 0; i < filesDict.count(); i++) {
		fileMetaData *fmd = filesDict.at(i);
		if(strcmp(fmd->filepath, fpath) == 0) {
			fmd->hascustomicn = TRUE;
			strcpy(fmd->custicnname, icname);
			found = TRUE;
			break;
		}		
	}
	if(!found) {
		filesDict.append(new fileMetaData());
		fileMetaData *fmd = filesDict.current();
		strcpy(fmd->filepath, fpath);
		fmd->isopenlocal = FALSE;
		fmd->hascustomicn = TRUE;
		strcpy(fmd->custicnname, icname);
		fmd->hascustomedit = FALSE;
		strcpy(fmd->custedit, "-");
	}
	sprintf(buff, "%s/%s", iconsPath, icname);
	loadAndTuneIcon(buff, fpath);	
}

void mfinderapp::unsetCustomIconOfFileAtPath(char *fpath)
{
	for(int i = 0; i < filesDict.count(); i++) {
		fileMetaData *fmd = filesDict.at(i);
		if(strcmp(fmd->filepath, fpath) == 0) {
			fmd->hascustomicn = FALSE;
			strcpy(fmd->custicnname, "-");
			break;
		}		
	}
}

void mfinderapp::setIsOpenLocalOfFileAtPath(char *fpath)
{
	bool found = FALSE;
	
	for(int i = 0; i < filesDict.count(); i++) {
		fileMetaData *fmd = filesDict.at(i);
		if(strcmp(fmd->filepath, fpath) == 0) {
			fmd->isopenlocal = TRUE;
			found = TRUE;
			break;
		}		
	}
	if(!found) {
		filesDict.append(new fileMetaData());
		fileMetaData *fmd = filesDict.current();
		strcpy(fmd->filepath, fpath);
		fmd->isopenlocal = TRUE;
		fmd->hascustomicn = FALSE;
		strcpy(fmd->custicnname, "-");
		fmd->hascustomedit = FALSE;
		strcpy(fmd->custedit, "-");
	}
}

void mfinderapp::unsetIsOpenLocalOfFileAtPath(char *fpath)
{
	for(int i = 0; i < filesDict.count(); i++) {
		fileMetaData *fmd = filesDict.at(i);
		if(strcmp(fmd->filepath, fpath) == 0) {
			fmd->isopenlocal = FALSE;
			break;
		}		
	}
}

bool mfinderapp::isOpenLocalOfFileAtPath(char *fpath)
{
	bool isopen = FALSE;
	
	for(int i = 0; i < filesDict.count(); i++) {
		fileMetaData *fmd = filesDict.at(i);
		if(strcmp(fmd->filepath, fpath) == 0) {
			if(fmd->isopenlocal) {
				isopen = TRUE;
				break;
			}
		}		
	}
	return isopen;
}

