#include "deskobj.h"
#include "mfinderapp.h"
#include "functions.h"

extern bool dropopenapp;
extern char appsPath[];

extern "C" {
	void XShapeCombineMask(Display*, Window, int, int, int, Pixmap, int);
 	#define ShapeBounding                   0 
	#define ShapeSet                        0 
}

DeskObj::DeskObj(QWidget *parent, const char *fullpath, const char *path, 
												const char *name, int type, int x, int y)
        :QWidget(parent, path, WStyle_Customize | WStyle_Tool | WStyle_NoBorder),
      	QDropSite(this)
{
	char *pref, *suff, *token, *delims = ".";
	char fname[MAX_NAME_LEN] = "";
	FILE *fAppData;

	strcpy(myFullPath, fullpath);
	strcpy(myPath, path);
	strcpy(myName, name);
	this->x = x;
	this->y = y;	
	myType = type;
	if(myType == DR || myType == DRLNK)	
		isDir = TRUE;
	else
		isDir = FALSE;
	
	if(myType == FIL)	{
		if(strcmp(myPath, appsPath) == 0) {
			sprintf(fname, "%s", myName);
			token = strtok(fname, delims);
			pref = token;
			while(token != NULL) {
				suff = token;
				token = strtok(NULL, delims);
			}
			if(strcmp(suff, "app") == 0) {
				fAppData = fopen(myFullPath, "r");
				fscanf(fAppData, "%s", myAppName);
				fclose(fAppData);
			}
		}
	}
	
	QColor color = QColor(230, 230, 230);
	QColorGroup cnorm = QColorGroup(black, color, color.light(), color.dark(), 
																color.dark(120), black, color);
	QColorGroup cinv = QColorGroup(white, black, color.light(), color.dark(), 
															color.dark(120), white, black);
	pnorm = QPalette(cnorm, cnorm, cnorm);
	pinv = QPalette(cinv, cinv, cinv);
	font = QFont("Helvetica", 10, 50, TRUE);
	
	havecustomicon = ((mfinderapp *)qApp)->hasCustomIconFileAtPath(myFullPath);
	
	icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
	 														FALSE, UNSELECT, havecustomicon));

	isSelect = FALSE;
	isOpen = FALSE;
	isOpenUnsl = FALSE;
	ismyobj = FALSE;
	fromlistview = FALSE;
	isOpenLocal = FALSE;
	dragdelay = 0;
	xOffset = 0;
	yOffset = 0;

	ismyobj = FALSE;
	myrefobj = NULL;
	myreflistobj = NULL;
	myrefobj = ((mfinderapp *)qApp)->fileObjAtPath(myFullPath);
	if(myrefobj) {
		setMyRefObj(myrefobj);
		myrefobj->setMyDeskObj(this);
	} else {
		myreflistobj = ((mfinderapp *)qApp)->fileListObjAtPath(myFullPath);
		if(myreflistobj) {
			setMyRefObj(myreflistobj);
			myreflistobj->setMyDeskObj(this);
		}
	}

	mask = QBitmap();	
	calcDim();
	popupMenu = createPopup();
	move(x, y);
	show();
	lower();

	if(isDir)
		if(((mfinderapp *)qApp)->winAtPath(myFullPath))
			UnselOpen(TRUE);
}

DeskObj::~DeskObj()
{
	if(fromlistview) {
		if(ismyobj)
			myreflistobj->unsetMyDeskObj();
	} else {
		if(ismyobj)
			myrefobj->unsetMyDeskObj();
	}
}

Popup* DeskObj::createPopup()
{
	Popup *popup;
	
	popup = new Popup();
	popup->insertItem("Remove from Desktop", this, SLOT(deleteFromPopUp()));
   return popup;
}

void DeskObj::setMyRefObj(QWidget *ref)
{
	myrefobj = NULL;
	myreflistobj = NULL;
	if(ref->inherits("FileObj")) {
		myrefobj = (FileObj*)ref;
		fromlistview = FALSE;
		isOpenLocal = FALSE;
		strcpy(myFullPath, myrefobj->myFullPath);
		strcpy(myPath, myrefobj->myPath);
		strcpy(myName, myrefobj->myName);
		myType = myrefobj->myType;
	} else {
		myreflistobj = (FileListObj*)ref;
		fromlistview = TRUE;
		strcpy(myFullPath, myreflistobj->myFullPath);
		strcpy(myPath, myreflistobj->myPath);
		strcpy(myName, myreflistobj->myName);
		myType = myreflistobj->myType;
		isOpenLocal = myreflistobj->isOpenLocal;
	}
	ismyobj = TRUE;
	calcDim();
	update();
}

void DeskObj::unsetMyRefObj()
{
	myrefobj = NULL;
	myreflistobj = NULL;
	ismyobj = FALSE;
}

void DeskObj::updateMyIcon()
{
	Unselect(FALSE);
}

void DeskObj::Select(bool fromlocal)
{
	isSelect = TRUE;
	isOpen = FALSE;
	isOpenUnsl = FALSE;
	icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
	 															FALSE, SELECT, havecustomicon));
	calcDim();
	update();
	if(fromlocal) {
		if(ismyobj) {
			if(fromlistview) 
				myreflistobj->Select(FALSE);
			else
				myrefobj->Select(FALSE);
		}
	}	
}

void DeskObj::Unselect(bool fromlocal)
{
	isSelect = FALSE;
	isOpen = FALSE;
	isOpenUnsl = FALSE;
	icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
																 FALSE, UNSELECT, havecustomicon));
	calcDim();
	update();
	if(fromlocal) {
		if(ismyobj) {
			if(fromlistview)
				myreflistobj->Unselect(FALSE);
			else
				myrefobj->Unselect(FALSE);
		}
	}	
}

void DeskObj::SelOpen(bool fromlocal) 
{
	isOpenUnsl = FALSE;
	isOpen = TRUE;
	isSelect = TRUE;
	icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
																 FALSE, SELOPEN, havecustomicon));
	calcDim();
	update();
	if(fromlocal) {
		if(ismyobj) {
			if(fromlistview) 
				myreflistobj->SelOpen(FALSE);
			else 
				myrefobj->SelOpen(FALSE);
		}
	}		
}

void DeskObj::UnselOpen(bool fromlocal)
{
	isOpenUnsl = TRUE;
	isOpen = TRUE;
	isSelect = FALSE;
	icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType, 
																	FALSE, OPEN, havecustomicon));
	calcDim();
	update();
	if(fromlocal) {
		if(ismyobj) {
			if(fromlistview)
				myreflistobj->UnselOpen(FALSE);
			else
				myrefobj->UnselOpen(FALSE);
		}
	}		
}

void DeskObj::Close(bool fromlocal)
{
	isOpen = FALSE;
	isOpenUnsl = FALSE;
	isSelect = TRUE;
	icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType, 
																FALSE, SELECT, havecustomicon));
	calcDim();
	update();
	if(fromlocal) {
		if(ismyobj) {
			if(fromlistview) 
				myreflistobj->Close(FALSE);
			else 
				myrefobj->Close(FALSE);
		}
	}	
}

void DeskObj::deleteFromPopUp()
{
	((mfinderapp *)qApp)->removeDeskObj(this);
}

void DeskObj::calcDim()
{
	QPainter p;
	QFontMetrics fm = QFontMetrics(font);
	int ascent = fm.ascent();
	int descent = fm.descent();
	wdt = fm.width(myName);
	int w = wdt;
	
	if(icona.width() > w) {
		w = icona.width() + 8;
		textXOffset = (w - wdt) / 2;
		pixmapXOffset = 4;
	} else {
		w += 8;
		textXOffset = 4;
		pixmapXOffset = (w - icona.width()) / 2;
	}
	pixmapYOffset = 2;
	textYOffset = icona.height() + 3 + ascent + 2;
	
	mask.resize(w, icona.height() + 3 + ascent + descent + 4);
	mask.fill(color0);	
	resize(w, icona.height() + 3 + ascent + descent + 4);
	
	p.begin(&mask);
	p.setFont(font);
   p.fillRect(textXOffset-2, textYOffset-10, wdt+4, ascent+descent+4, white);
	p.drawText(textXOffset, textYOffset, myName);
	p.end();
	bitBlt(&mask, pixmapXOffset, pixmapYOffset, icona.mask());
	wdt = w;
	hgt = icona.height() + 3 + ascent + descent + 4;
}

QPixmap DeskObj::prepDragPix()
{
	QPixmap dragpix;
	
	dragpix = QPixmap(this->size());
 	dragpix.fill(color1);
	bitBlt(&dragpix, 0, 0, this);
	dragpix.setMask(dragpix.createHeuristicMask());	
	
	return dragpix;
}

void DeskObj::resizeEvent(QResizeEvent *)
{
	XShapeCombineMask(x11Display(), winId(), ShapeBounding, 0, 0, 
			mask.handle(), ShapeSet);
}

void DeskObj::mousePressEvent(QMouseEvent *e)
{
	if(e->button() == RightButton)
		popupMenu->popup(mapToGlobal(e->pos()));

	if(isOpenUnsl)
		SelOpen(TRUE);
	if(isSelect) {

	} else {
		Select(TRUE);
	}
	if(!ismyobj)
		((mfinderapp *)qApp)->unselectAllObjNotDsObj(this);
}

void DeskObj::mouseDoubleClickEvent(QMouseEvent *e)
{
	((mfinderapp *)qApp)->openAtPath(myFullPath, myName, myType);
}

void DeskObj::mouseMoveEvent(QMouseEvent *e)
{
	QStrList urls;
	QPixmap dragpix;
	QPainter p;

	dragdelay++;
	if(dragdelay >= 2) {
		dragpix = prepDragPix();
		urls.append(myFullPath);
		dropopenapp = isPlainFileAtPath(urls.at(0)) ? TRUE : FALSE;
		urlDragObj = new QUrlDrag(urls, this, "udrag");
		urlDragObj->setPixmap(dragpix, QPoint(0, 0));
		
		((mfinderapp *)qApp)->fileListDragObj = NULL;
		((mfinderapp *)qApp)->fileDragObj = NULL;
		((mfinderapp *)qApp)->deskDragObj = this;
		
		((mfinderapp *)qApp)->isdragcopying = FALSE;
		if(e->state() & ControlButton)
			((mfinderapp *)qApp)->isdragcopying = TRUE;
		((mfinderapp *)qApp)->xdnddone = FALSE;
		((mfinderapp *)qApp)->deskobjmoving = TRUE;	

		urlDragObj->drag();
		
		if(!((mfinderapp *)qApp)->xdnddone) {
			((mfinderapp *)qApp)->offixDrop(this);
		} else {
			if(!((mfinderapp *)qApp)->isdragcopying && ((mfinderapp *)qApp)->dndok) {
				((mfinderapp *)qApp)->removeDeskObjFromDict(myFullPath);
				((mfinderapp *)qApp)->deskobjlist.removeRef(this);
			}
		}
	}
}

void DeskObj::dragEnterEvent(QDragEnterEvent *e)
{
	if(QUrlDrag::canDecode(e)) {
		if(isDir) {
			e->accept();
			if(isOpenUnsl)
				SelOpen(true);
			if(isSelect) {

			} else {
				Select(true);
			}
		}
	}
}

void DeskObj::dragLeaveEvent(QDragLeaveEvent *e)
{
	if(isSelect) {
		if(isOpen)
			UnselOpen(true);
		else
			Unselect(true);
	}
}

void DeskObj::dropEvent(QDropEvent *e)
{
	QStrList droplist;

	if(isDir) {
		e->accept();
		QUrlDrag::decode(e, droplist);
		((mfinderapp *)qApp)->moveOrCopy(droplist, myFullPath, this);
	}
}

void DeskObj::paintEvent(QPaintEvent *)
{
	bitBlt(this, pixmapXOffset, pixmapYOffset, &icona);
  
	QPainter p;
	p.begin(this);
	if(!isSelect) {
		p.setPen(black);   
		p.drawText(textXOffset, textYOffset, myName);
   } else {
   	p.fillRect(textXOffset-2, textYOffset-10, 
   		p.fontMetrics().width(myName)+4, 
   		p.fontMetrics().ascent()+p.fontMetrics().descent()+4, black);
   	p.setPen(white);   
		p.drawText(textXOffset, textYOffset, myName);
	}    
	p.end();
}
