#include "dialogs.h"
#include <qapp.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#include "mfinderapp.h"
#include "functions.h"

extern char confdir[];
extern char iconsPath[];		 	
extern char appsPath[];

Dialog::Dialog(QWidget *parent, const char *name, bool modal, 
	       						WFlags f, bool istitbig, bool istxtfield,
	      				 bool isedfield, bool iscancbutt, const char *contents)
		 :QDialog(parent, name, TRUE, f)
{
	x = QColor(160, 164, 160);
	CGnorm = QColorGroup(black, x, x.light(), x.dark(), x.dark(120), black, white);
	setPalette(QPalette(CGnorm, CGnorm, CGnorm));
	font = QFont("Helvetica", 12);
	fontbold = QFont("Helvetica", 12);
	fontbold.setBold(TRUE);
	fontbig = QFont("Helvetica", 18);

	this->istitbig = istitbig;
	this->istxtfield = istxtfield;
	this->isedfield = isedfield;
	this->iscancbutt = iscancbutt;

	setCaption(" ");
	
	if(istitbig) {
		titlebig = new QLabel(name, this, "title");
		titlebig->setPalette(QPalette(CGnorm, CGnorm, CGnorm));
		titlebig->setGeometry(0, 0, 200, 30);
		titlebig->setFont(fontbig);		
		titlebig->setAlignment(AlignLeft | AlignTop);
	}
	if(istxtfield) {
		textfield = new QLabel(this, "title");
		textfield->setPalette(QPalette(CGnorm, CGnorm, CGnorm));
		textfield->setFont(font);		
		textfield->setAutoResize(TRUE);
		textfield->setAlignment(AlignLeft | AlignTop);
		textfield->setText(contents);
	}	
	if(isedfield) {
		editfield = new QLineEdit(this, "edit");
		editfield->setMaxLength(64);
		editfield->setGeometry(0, 0, 200, 28);
		editfield->setPalette(QPalette(CGnorm, CGnorm, CGnorm));
		editfield->setFont(font);		
	}
	if(iscancbutt) {
		cancel = new QPushButton("Cancel", this);
		cancel->setGeometry(0, 0, 75, 28);
		cancel->setPalette(QPalette(CGnorm, CGnorm, CGnorm));
		cancel->setFont(font);		
		connect(cancel, SIGNAL(clicked()), SLOT(reject()));
	}
	ok = new QPushButton("Ok", this);
	ok->setGeometry(0, 0, 75, 28);
	ok->setPalette(QPalette(CGnorm, CGnorm, CGnorm));
	ok->setFont(font);		
	ok->setDefault(TRUE);
	connect(ok, SIGNAL(clicked()), SLOT(accept()));
}

void Dialog::resizeEvent(QResizeEvent *)
{
	int h, v;
	
	sepor = 20;
	sepv = 20;
	h = 20;
	v = 5;
	if(istitbig) {
		titlebig->move(h, v);
		frstlinv = v + titlebig->height();
		v = frstlinv + sepv;
	} else {
		frstlinv = v;
		v = v + sepv;
	}
	if(istxtfield) {
		textfield->move(h, v);
		seclinv = v + textfield->height() + sepv;
		v = seclinv + sepv;
		if((sepor + textfield->width() + sepor) > 240)
			setFixedSize(sepor + textfield->width() + sepor, v + ok->height());
		else
			setFixedSize(240, v + ok->height());
	}
	if(isedfield) {
		editfield->move(h, v);
		seclinv = v + editfield->height() + sepv;
		v = seclinv + sepv;
		setFixedSize(240, v + ok->height());
		editfield->setFocus();
	}
	if(iscancbutt) {
		cancel->move(width() - cancel->width() - ok->width() - sepor - 10,
							height() - cancel->height() - 10);
	}	
	ok->move(width() - ok->width() - 10, height() - ok->height() - 10);
	
	((mfinderapp *)qApp)->centreDialogOnScreen(this);
}

void Dialog::paintEvent(QPaintEvent *)
{
	QPainter p;
				
	p.begin(this);
	p.setPen(QPen(QColor(80, 80, 80), 1));
	p.drawLine(0, frstlinv, width(), frstlinv);
	p.setPen(QPen(QColor(245, 245, 245), 1));
	p.drawLine(0, frstlinv + 1, width(), frstlinv + 1);

	p.setPen(QPen(QColor(80, 80, 80), 1));
	p.drawLine(0, seclinv, width(), seclinv);
	p.setPen(QPen(QColor(245, 245, 245), 1));
	p.drawLine(0, seclinv + 1, width(), seclinv + 1);

	p.end();
}

DlogOpenAt::DlogOpenAt(QWidget *parent, const char *name)
		 	  :QDialog(parent, name, TRUE)
{
	x = QColor(160, 164, 160);
	CGnorm = QColorGroup(black, x, x.light(), x.dark(), x.dark(120), black, white);
	setPalette(QPalette(CGnorm, CGnorm, CGnorm));
	font = QFont("Helvetica", 12);
	fontbig = QFont("Helvetica", 18);

	setCaption(" ");
	
	titlebig = new QLabel("Open At...", this, name);
	titlebig->setPalette(QPalette(CGnorm, CGnorm, CGnorm));
	titlebig->setGeometry(0, 0, 200, 30);
	titlebig->setFont(fontbig);		
	titlebig->setAlignment(AlignLeft | AlignTop);

	editfield = new QLineEdit(this, "edit");
	editfield->setMaxLength(64);
	editfield->setGeometry(0, 0, 200, 24);
	editfield->setPalette(QPalette(CGnorm, CGnorm, CGnorm));
	editfield->setFont(font);		
	editfield->installEventFilter(this); 	
	editfield->setFocus();

	cancel = new QPushButton("Cancel", this);
	cancel->setGeometry(0, 0, 75, 28);
	cancel->setPalette(QPalette(CGnorm, CGnorm, CGnorm));
	cancel->setFont(font);		
	connect(cancel, SIGNAL(clicked()), SLOT(reject()));
	ok = new QPushButton("Ok", this);
	ok->setGeometry(0, 0, 75, 28);
	ok->setPalette(QPalette(CGnorm, CGnorm, CGnorm));
	ok->setFont(font);		
	ok->setDefault(TRUE);
	connect(ok, SIGNAL(clicked()), SLOT(accept()));
}

void DlogOpenAt::resizeEvent(QResizeEvent *)
{
	setFixedSize(240, 140);
	titlebig->move(20, 5);
	editfield->move(20, 50);
	cancel->move(width() - cancel->width() - ok->width() -20,
							height() - cancel->height() - 10);
	ok->move(width() - ok->width() - 10, height() - ok->height() - 10);
	((mfinderapp *)qApp)->centreDialogOnScreen(this);
}

void DlogOpenAt::completeEditLine()
{
	char basepath[256] = "", namepart[32] = "", completedtxt[256] = "";
	struct stat listStat;
	struct dirent **dir;
	char **dirnames;
	int m = 0, j = 0;	

	if(editfield->text()[0] == '/' && editfield->text()[strlen(editfield->text())-1] != '/') {
		sprintf(basepath, "%s", deleteLastPathComponent((char*)editfield->text()));
		sprintf(namepart, "%s", lastPathComponent((char*)editfield->text()));

		if(!lstat(basepath, &listStat)) {
			if(S_ISDIR(listStat.st_mode)) {
				m = scandir(basepath, &dir, 0, alphasort);
				dirnames = new char*[m];
	
				for(int i = 0; i < m; i++) {
					if(strcmp(dir[i]->d_name, ".") !=0 && strcmp(dir[i]->d_name, "..") !=0) {
						dirnames[j] = dir[i]->d_name;
						j++;
					}
				}
				
				if(strcmp(basepath, "/") == 0)
					sprintf(completedtxt, "%s%s", basepath, 
										bestMatchingStringInArray(dirnames, j, namepart));
				else
					sprintf(completedtxt, "%s/%s", basepath, 
										bestMatchingStringInArray(dirnames, j, namepart));				
				
				editfield->setText(completedtxt);
				
				delete []dirnames;
			}
		}
	}
}

void DlogOpenAt::paintEvent(QPaintEvent *)
{
	QPainter p;
				
	p.begin(this);
	p.setPen(QPen(QColor(80, 80, 80), 1));
	p.drawLine(0, 35, width(), 35);
	p.setPen(QPen(QColor(245, 245, 245), 1));
	p.drawLine(0, 36, width(), 36);

	p.setPen(QPen(QColor(80, 80, 80), 1));
	p.drawLine(0, 88, width(), 88);
	p.setPen(QPen(QColor(245, 245, 245), 1));
	p.drawLine(0, 89, width(), 89);

	p.end();
}

bool DlogOpenAt::eventFilter(QObject *, QEvent *e)
{
	if(e->type() == Event_KeyPress) {
		QKeyEvent *ke = (QKeyEvent *)e;
		if(ke->key() == Key_Tab) {
			completeEditLine();
			return TRUE;
		}
	}
  	return FALSE;
}

DialogCopy::DialogCopy(QWidget *parent, const char *name)
		     :QWidget(parent, name)
{
	x = QColor(160, 164, 160);
	CGnorm = QColorGroup(black, x, x.light(), x.dark(), x.dark(120), black, white);
	setPalette(QPalette(CGnorm, CGnorm, CGnorm));
	font = QFont("Helvetica", 12);
	fontbold = QFont("Helvetica", 12);
	fontbold.setBold(TRUE);
	fontbig = QFont("Helvetica", 18);

	setCaption(" ");
	setFixedSize(220, 100);
	
	titlebig = new QLabel(name, this, "title");
	titlebig->setPalette(QPalette(CGnorm, CGnorm, CGnorm));
	titlebig->setGeometry(20, 5, 300, 29);
	titlebig->setFont(fontbig);		
	titlebig->setAlignment(AlignLeft | AlignTop);
	titlebig->setText("Copy");		
	titlebig->show();
		
	pbar = new QProgressBar(this, "pbar");
	pbar->setGeometry(10, 50, 200, 15);
	pbar->show();
}

void DialogCopy::resizeEvent(QResizeEvent *)
{
	((mfinderapp *)qApp)->centreDialogOnScreen(this);
}

void DialogCopy::paintEvent(QPaintEvent *)
{
	QPainter p;
				
	p.begin(this);
	p.setPen(QPen(QColor(80, 80, 80), 1));
	p.drawLine(0, 34, width(), 34);
	p.setPen(QPen(QColor(245, 245, 245), 1));
	p.drawLine(0, 35, width(), 35);

	p.setPen(QPen(QColor(80, 80, 80), 1));
	p.drawLine(0, 80, width(), 80);
	p.setPen(QPen(QColor(245, 245, 245), 1));
	p.drawLine(0, 81, width(), 81);

	p.end();
}


StrtScrn::StrtScrn(QWidget *parent, const char *name, const char *icnpath)
		   :QWidget(0L, name)
{
	QColor x = QColor(160, 164, 160);
	QColorGroup CGnorm = QColorGroup(black, x, x.light(), x.dark(), x.dark(120), black, white);
	font = QFont("Helvetica", 10);
	fontbold = QFont("Helvetica", 12);
	fontbold.setBold(TRUE);
	fontbig = QFont("Helvetica", 18);
	setCaption(" ");
	setFixedSize(380, 240);

	backpix = QPixmap(icnpath, "XPM");
	setBackgroundPixmap(backpix);
	
	pbar = new QProgressBar(this, "pbar");
	pbar->setGeometry(30, 205, 320, 20);
	pbar->setPalette(QPalette(CGnorm, CGnorm, CGnorm));
	pbar->show();
}

void StrtScrn::resizeEvent(QResizeEvent *)
{
	((mfinderapp *)qApp)->centreDialogOnScreen(this);
}

WMFInfo::WMFInfo(QWidget *parent, const char *name, const char *icnpath)
		  :QWidget(parent, name)
{
	x = QColor(160, 164, 160);
	CGnorm = QColorGroup(black, x, x.light(), x.dark(), x.dark(120), black, white);
	setPalette(QPalette(CGnorm, CGnorm, CGnorm));
	font = QFont("Helvetica", 10);
	fontbold = QFont("Helvetica", 10);
	fontbold.setBold(TRUE);
	fontbig = QFont("Helvetica", 18);
	setCaption(" ");
	setFixedSize(380, 240);
	
	backpix = QPixmap(icnpath, "XPM");
	setBackgroundPixmap(backpix);
	
	statusfield = new QLabel(this, "status");
	statusfield->setPalette(QPalette(CGnorm, CGnorm, CGnorm));
	statusfield->setGeometry(0, 0, 320, 20);
	statusfield->setFont(font);		
	statusfield->setAlignment(AlignCenter);
	statusfield->setText("version 0.5 - 10 Feb 1999");
	statusfield->move(30, 160);

	explfield = new QLabel(this, "expl");
	explfield->setPalette(QPalette(CGnorm, CGnorm, CGnorm));
	explfield->setGeometry(0, 0, 320, 40);
	explfield->setFont(font);		
	explfield->setAlignment(AlignCenter);
	explfield->setText("by Enrico Sersale\n \
	enrico@imago.ro - http://www.imago.ro/wmfinder");
	explfield->move(30, 180);
}

void WMFInfo::resizeEvent(QResizeEvent *)
{
	((mfinderapp *)qApp)->centreDialogOnScreen(this);
}

HelpWin::HelpWin(QWidget *parent, const char *name)
		  :QWidget(parent, name)
{
	FILE *fHelp;
	char fHelpPath[256];
	struct stat st;

	sprintf(fHelpPath, "%s/GNUstep/Library/xwfinder/wmfhelp", getenv("HOME"));
	if (!stat(fHelpPath, &st)) { // stat() returns 0 on success
		help_buf = new char[st.st_size+1];
	  	fHelp = fopen(fHelpPath, "r");
	  	fread(help_buf, st.st_size, 1, fHelp);
	  	fclose(fHelp);
	  	help_buf[st.st_size+1] = '\0';
	}
	font = QFont("Helvetica", 10);
	fontbold = QFont("Helvetica", 10);
	setCaption("Help");

	setFixedSize(500, 300);
	mlefield = new QMultiLineEdit(this, "status");
	mlefield->setReadOnly(TRUE);
	mlefield->setFont(font);		
	mlefield->setMinimumHeight(300);
	mlefield->setMinimumWidth(500);
	mlefield->setText(help_buf);
}

void HelpWin::resizeEvent(QResizeEvent *)
{
	((mfinderapp *)qApp)->centreDialogOnScreen(this);
}

void HelpWin::paintEvent(QPaintEvent *)
{
	QPainter p;
	p.begin(this);
	p.end();
}

void HelpWin::close()
{
	hide();
	delete [] help_buf;
 	QWidget::close();
}


ChooseAnIcnDlg::ChooseAnIcnDlg(QWidget *parent, const char *name,
						char *iconPath, char *icname, char *errmsg, bool iscancbutt)
		         :QDialog(parent, name, TRUE)
{
	struct dirent **namelist;
	int m;

	x = QColor(160, 164, 160);
	CGnorm = QColorGroup(black, x, x.light(), x.dark(), x.dark(120), black, white);
	setPalette(QPalette(CGnorm, CGnorm, CGnorm));
	font = QFont("Helvetica", 10);
	fontbig = QFont("Helvetica", 18);
	setCaption(" ");
	setFixedSize(400, 290); 

	strcpy(icnPath, iconPath);

	titlebig = new QLabel("Choose an Icon", this, "big");
	titlebig->setPalette(QPalette(CGnorm, CGnorm, CGnorm));
	titlebig->setGeometry(20, 5, 200, 30);
	titlebig->setFont(fontbig);		
	titlebig->setAlignment(AlignLeft | AlignTop);

	messglab = new QLabel(errmsg, this, "msg");
	messglab->setPalette(QPalette(CGnorm, CGnorm, CGnorm));
	messglab->setGeometry(20, 50, 360, 80);
	messglab->setFont(font);		
	messglab->setAlignment(AlignLeft | AlignTop);
	
	icnList = new QListBox(this, "Bu");
	icnList->setLineWidth(2);
	icnList->setPalette(QPalette(CGnorm, CGnorm, CGnorm));
	icnList->setFont(font);
	icnList->setBackgroundMode(PaletteBase);
	icnList->setGeometry(250, 140, 130, 90);
	connect(icnList, SIGNAL(highlighted(const char *)), 
						this, SLOT(setNameIcn(const char *)));
	connect(icnList, SIGNAL(highlighted(const char *)), 
						this, SLOT(changeIcn(const char *)));

 	m = scandir(iconPath, &namelist, 0, alphasort);
	for(int i = 0; i < m; i++)
		if(strcmp(namelist[i]->d_name, ".") != 0 
					&& strcmp(namelist[i]->d_name, "..") != 0)
			icnList->insertItem(namelist[i]->d_name);

	namIcnEd = new QLineEdit(this, "");
	namIcnEd->setGeometry(20, 195, 158, 24);
	namIcnEd->setPalette(QPalette(CGnorm, CGnorm, CGnorm));
	namIcnEd->setFont(font);
	namIcnEd->setText(icnList->text(0));

	sprintf(icnName, "%s/%s", iconPath, icnList->text(0));
	Icona = QPixmap(icnName);
	IcnLab = new QLabel(this);
	IcnLab->setGeometry(190, 170, 55, 55);
	IcnLab->setPalette(QPalette(CGnorm, CGnorm, CGnorm));
	IcnLab->setFont(font);
	IcnLab->setPixmap(Icona);

	if(iscancbutt) {
		butCancel = new QPushButton("Cancel", this);
		butCancel->setGeometry(210, 250, 80, 30);
		butCancel->setPalette(QPalette(CGnorm, CGnorm, CGnorm));
		butCancel->setFont(font);
		connect(butCancel, SIGNAL(clicked()), SLOT(reject()));
	}	
	butOk = new QPushButton("OK", this);
	butOk->setGeometry(300, 250, 80,30);
	butOk->setPalette(QPalette(CGnorm, CGnorm, CGnorm));
	butOk->setFont(font);
	butOk->setDefault(TRUE);
	connect(butOk, SIGNAL(clicked()), SLOT(accept()));	
}

void ChooseAnIcnDlg::setNameIcn(const char *icname)
{
	namIcnEd->setText(icname);
}

void ChooseAnIcnDlg::changeIcn(const char *icname)
{
	char buff[512] = "";
	
	sprintf(buff, "%s/%s", icnPath, icname);
   Icona = QPixmap(buff);
	IcnLab->setPixmap(Icona);
	IcnLab->update();
}

void ChooseAnIcnDlg::resizeEvent(QResizeEvent *)
{
	((mfinderapp *)qApp)->centreDialogOnScreen(this);
}

void ChooseAnIcnDlg::paintEvent(QPaintEvent *)
{
	QPainter p;
				
	p.begin(this);
	p.setPen(QPen(QColor(80, 80, 80), 1));
	p.drawLine(0, 35, width(), 35);
	p.setPen(QPen(QColor(245, 245, 245), 1));
	p.drawLine(0, 36, width(), 36);

	p.setPen(QPen(QColor(80, 80, 80), 1));
	p.drawLine(0, 240, width(), 240);
	p.setPen(QPen(QColor(245, 245, 245), 1));
	p.drawLine(0, 241, width(), 241);

	p.end();
}

