#include "filelistobj.h"
#include "mfinderapp.h"
#include "functions.h"

const char *down_xpm[] = {
"15 8 6 1",
" 	c None",
".	c #000000",
"+	c #9595BE",
"@	c #9595BD",
"#	c #9696BD",
"$	c #9696BE",
"...............",
" .+@##$$@$$@$. ",
"  .@##$$@$$@.  ",
"   .##$$@$$.   ",
"    .#$$@$.    ",
"     .$$@.     ",
"      .$.      ",
"       .       "};

const char *dx_xpm[] = {
"8 15 6 1",
" 	c None",
".	c #000000",
"+	c #9696BE",
"@	c #9595BD",
"#	c #9696BD",
"$	c #9595BE",
".       ",
"..      ",
".+.     ",
".@@.    ",
".+++.   ",
".++++.  ",
".@@@@@. ",
".++++++.",
".+++++. ",
".####.  ",
".###.   ",
".@@.    ",
".$.     ",
"..      ",
".       "};

extern bool iview;
extern bool dfirst;
extern char appsPath[];
extern bool dropopenapp;

ArrowLabel::ArrowLabel(QWidget *parent, const char *name, bool isdir, 
					bool isopen, const char **arrovdesc, const char **openarrovdesc)
        		:QLabel(parent, name)
{
	this->isopen = isopen;
	this->isdir = isdir;

	if(isdir) {
		arrowpix = QPixmap(arrovdesc);
		openarrowpix = QPixmap(openarrovdesc);
		if(isopen)
 			setPixmap(openarrowpix);
		else
 			setPixmap(arrowpix);
	}
}

ArrowLabel::~ArrowLabel()
{

}

void ArrowLabel::Close()
{
	isopen = FALSE;
 	setPixmap(arrowpix);
	update();
}

void ArrowLabel::mousePressEvent(QMouseEvent *e)
{
	isopen = (!isopen) ? TRUE : FALSE;

	if(isdir) {
		if(isopen) {
 			setPixmap(openarrowpix);
			emit arrowopen();
			update();
		} else {
			emit arrowclose();
		}
	}
}


MyLabel::MyLabel(QWidget *parent, const char *name)
        		:QLabel(parent, name),
				 QDropSite(this)
{
	mustdrawselrect = FALSE;
	r1o = QRect(0,0,0,0);
	r2o = QRect(0,0,0,0);
	r1v = QRect(0,0,0,0);
	r2v = QRect(0,0,0,0);
	myflistobj = (FileListObj*)parent;
}

void MyLabel::drawSelRect(QRect selrect, bool mustdraw)
{
	QRect oldrect;
	
	if(mustdraw) {
		mustdrawselrect = TRUE;
		
		oldrect = r1o;
		r1o = QRect(selrect.topLeft(), selrect.topRight());
		if(oldrect.topLeft().y() > 1)
			update(oldrect);
		if(r1o.topLeft().y() > 1)
			update(r1o);
		
		oldrect = r2o;
		r2o = QRect(selrect.bottomLeft(), selrect.bottomRight());
		if(oldrect.topLeft().y() < height() - 1)
			update(oldrect);
		if(r2o.topLeft().y() < height() - 1)
			update(r2o);

		oldrect = r1v;
		r1v = QRect(selrect.topLeft(), selrect.bottomLeft());
		if(oldrect.topLeft().x() > 1)
			update(oldrect);
		if(r1v.topLeft().x() > 1)
			update(r1v);

		oldrect = r2v;
		r2v = QRect(selrect.topRight(), selrect.bottomRight());
		if(oldrect.topLeft().x() < width() - 1)
			update(oldrect);
		if(r2v.topLeft().x() < width() - 1)
			update(r2v);
		
	} else {
		if(mustdrawselrect) {
			mustdrawselrect = FALSE;
			update(r1o);
			update(r2o);
			update(r1v);
			update(r2v);
		}
	}
}

void MyLabel::mousePressEvent(QMouseEvent *e)
{
	QMouseEvent ev = QMouseEvent(e->type(), mapToParent(e->pos()), e->button(), e->state());
	((mfinderapp *)qApp)->sendEvent(myflistobj, &ev);
}

void MyLabel::mouseReleaseEvent(QMouseEvent *e)
{
	((mfinderapp *)qApp)->sendEvent(myflistobj, e);
}

void MyLabel::mouseMoveEvent(QMouseEvent *e)
{
	QMouseEvent ev = QMouseEvent(e->type(), mapToParent(e->pos()), e->button(), e->state());
	((mfinderapp *)qApp)->sendEvent(myflistobj, &ev);
}

void MyLabel::dragEnterEvent(QDragEnterEvent *e)
{
	if(QUrlDrag::canDecode(e)) {
		e->accept();
		QDragEnterEvent ev = QDragEnterEvent(myflistobj->mapToParent(mapToParent(e->pos())));
		((mfinderapp *)qApp)->sendEvent(myflistobj->myobjsframe, &ev);
	}
}

void MyLabel::dragLeaveEvent(QDragLeaveEvent *e)
{
	((mfinderapp *)qApp)->sendEvent(myflistobj, e);
}

void MyLabel::dropEvent(QDropEvent *e)
{
	myflistobj->myobjsframe->isDragTarget = FALSE;
	myflistobj->myobjsframe->update();
	((mfinderapp *)qApp)->sendEvent(myflistobj->myobjsframe, e);
}

void MyLabel::paintEvent(QPaintEvent *e)
{
	QPainter p;
	
	QLabel::paintEvent(e);
	p.begin(this);
	if(mustdrawselrect) {
		if(r1o.topLeft().y() > 1)
			p.drawRect(r1o);
		if(r2o.topLeft().y() < height() - 1)
			p.drawRect(r2o);
		if(r1v.topLeft().x() > 1)
			p.drawRect(r1v);
		if(r2v.topLeft().x() < width() - 1)
			p.drawRect(r2v);
	}
	p.end();
}


FileListObj::FileListObj(QWidget *parent, const char *path, const char *name, 
					int type, int movx, bool isOpenFromLocal, FileListObj *parentobj)
        		:QWidget(parent, path),
      		 QDropSite(this)
{
	char *pref, *suff, *token, *delims = ".";
	char fname[MAX_NAME_LEN] = "";
	FILE *fAppData;

	QColor color = QColor(230, 230, 230);
	QColorGroup cnorm = QColorGroup(black, color, color.light(), color.dark(), 
																color.dark(120), black, color);
	QColorGroup cinv = QColorGroup(white, black, color.light(), color.dark(), 
															color.dark(120), white, black);
	pnorm = QPalette(cnorm, cnorm, cnorm);
	pinv = QPalette(cinv, cinv, cinv);
	QFont font = QFont("Helvetica", 10);

	strcpy(myPath, path);
	strcpy(myName, name);
	if(strcmp(myPath, "/") != 0)
		sprintf(myFullPath, "%s/%s", myPath, myName);
	else
		sprintf(myFullPath, "%s%s", myPath, myName);
	myType = type;	
	this->movx = movx;
	this->isOpenFromLocal = isOpenFromLocal;
	if(isOpenFromLocal)
		myparentobj = parentobj;
	else
		myparentobj = NULL;
	sep = 25;
	leftspace = 1;
	hght = hspace = 20;
	myobjsframe = (ObjsListFrame *)parent;
	resize(myobjsframe->width()-10, hght);
	
	list.setAutoDelete(TRUE);
	isSelect = FALSE;
	isOpen = FALSE;
	isOpenUnsl = FALSE;
	isOpenLocal = FALSE;
	mustdrawselrect = FALSE;
	candrag = TRUE;
	havecustomicon = ((mfinderapp *)qApp)->hasCustomIconFileAtPath(myFullPath);
	popupMenu = NULL;
	dragdelay = 0;
	r1o = QRect(0,0,0,0);
	r2o = QRect(0,0,0,0);
	r1v = QRect(0,0,0,0);
	r2v = QRect(0,0,0,0);
	
	if(myType == DR || myType == DRLNK) {
		isDir = TRUE;
		arrowlabel = new ArrowLabel(this, "arrow", isDir, isOpenLocal, dx_xpm, down_xpm);
		arrowlabel->setPalette(pnorm);
		arrowlabel->resize(20, hght);
		connect(arrowlabel, SIGNAL(arrowopen()), this, SLOT(OpenLocal()));
		connect(arrowlabel, SIGNAL(arrowclose()), this, SLOT(CloseLocal()));
	} else {
		isDir = FALSE;
		arrowlabel = new ArrowLabel(this, "arrow", isDir, isOpenLocal);
		arrowlabel->setPalette(pnorm);
		arrowlabel->resize(20, hght);
	}
	if(myType == FILLNK || myType == DRLNK || myType == FILEXELNK)	
		readlink(myFullPath, myLinkTo, MAX_PATH_LEN);

	if(myType == FIL)	{
		if(strcmp(myPath, appsPath) == 0) {
			sprintf(fname, "%s", myName);
			token = strtok(fname, delims);
			pref = token;
			while(token != NULL) {
				suff = token;
				token = strtok(NULL, delims);
			}
		}
	}
	
	label = new QLabel(this, myName);
	label->setAutoResize(TRUE);
	label->setPalette(pnorm);
	label->setFont(font);
	label->setText(myName);
	label->installEventFilter(this); 	
	labdim = new MyLabel(this, "dim");
	labdim->setFixedSize(70, hght);
	labdim->setPalette(pnorm);
	labdim->setFont(font);	
	labusr = new MyLabel(this, "usr");
	labusr->setFixedSize(50, hght);
	labusr->setPalette(pnorm);
	labusr->setFont(font);
	labperm = new MyLabel(this, "perm");
	labperm->setFixedSize(70, hght);
	labperm->setPalette(pnorm);
	labperm->setFont(font);
	labdate = new MyLabel(this, "date");
	labdate->setFixedSize(90, hght);
	labdate->setPalette(pnorm);
	labdate->setFont(font);
	
	updateLabelsContents();
	
	icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
	 														  TRUE, UNSELECT, havecustomicon));

	if(isDir)
		if(((mfinderapp *)qApp)->winAtPath(myFullPath))
			UnselOpen(true);
	
	DeskObj *d = ((mfinderapp *)qApp)->deskObjectAtPath(myFullPath);
	if(d) {
		setMyDeskObj(d);
		d->setMyRefObj(this);
	} else {
		ismydeskobj = FALSE;
	}
			
	show();
	label->show();
	
	if(isDir) {
		if(((mfinderapp *)qApp)->isOpenLocalOfFileAtPath(myFullPath)) {
			arrowlabel->isopen = TRUE;
 			arrowlabel->setPixmap(arrowlabel->openarrowpix);
			arrowlabel->update();		
			OpenLocal();
		}
	}
}

FileListObj::~FileListObj()
{
	if(ismydeskobj)
		mydeskobj->unsetMyRefObj();

	label->~QLabel();
	labdim->~MyLabel();
	labusr->~MyLabel();
	labperm->~MyLabel();
	labdate->~MyLabel();
	arrowlabel->~ArrowLabel();
}

void FileListObj::updateLabelsContents()
{
	struct dirent **namelist;
	struct stat listStat;
	struct passwd *usr;
	char user[32] = "";
	struct tm *t;
	mode_t oldperm = 0L, newperm = 0L;
	char buff[64] = "";
	
	lstat(myFullPath, &listStat);

	if(myType == DR || myType == DRLNK) {
		int m = scandir(myFullPath, &namelist, 0, alphasort);
		sprintf(buff, "%i elements", m - 2);
		labdim->setText(buff);
	} else if(myType == FIL || myType == FILEXE) {
		sprintf(buff, "%.0li", listStat.st_size);
		labdim->setText(buff);
	} else {
		labdim->setText(" ");
	}

	if((usr = getpwuid(listStat.st_uid)) == NULL)
		sprintf(user, "%i", listStat.st_uid);
	else
		sprintf(user, "%s", usr->pw_name);

	labusr->setText(user);

	sprintf(buff, " ");
  	if(myType == FILLNK || myType == FILEXELNK || myType == DRLNK)  
		buff[0] = 'l';
  	else if (S_ISDIR(listStat.st_mode))  
		buff[0] = 'd';
  	else 
		buff[0] = '-';
  	buff[1] = listStat.st_mode & S_IRUSR ? 'r' : '-'; 
  	buff[2] = listStat.st_mode & S_IWUSR ? 'w' : '-'; 
  	buff[3] = listStat.st_mode & S_IXUSR ? 'x' : '-'; 
  	buff[4] = listStat.st_mode & S_IRGRP ? 'r' : '-'; 
  	buff[5] = listStat.st_mode & S_IWGRP ? 'w' : '-'; 
  	buff[6] = listStat.st_mode & S_IXGRP ? 'x' : '-'; 
  	buff[7] = listStat.st_mode & S_IROTH ? 'r' : '-'; 
  	buff[8] = listStat.st_mode & S_IWOTH ? 'w' : '-'; 
  	buff[9] = listStat.st_mode & S_IXOTH ? 'x' : '-'; 
  	buff[10] = '\0';
	labperm->setText(buff);
	
	t = localtime(&listStat.st_mtime);
	sprintf(buff, "%02i:%02i %02i.%02i.%04i", 
										t->tm_hour, t->tm_min, t->tm_mday, 
												t->tm_mon + 1, t->tm_year + 1900);
	labdate->setText(buff);
}

void FileListObj::setMyDeskObj(DeskObj *dobj)
{
	mydeskobj = dobj;
	ismydeskobj = TRUE;
}

void FileListObj::unsetMyDeskObj()
{
	mydeskobj = NULL;
	ismydeskobj = FALSE;
}

Popup* FileListObj::createPopup()
{
	Popup *popup;
	
	popup = new Popup();
	popup->insertItem("Open", this, SLOT(openFromPopUp()));
	popup->insertItem("Rename", this, SLOT(renameFromPopUp()));
	popup->insertItem("Info", this, SLOT(infoFromPopUp()));
	if(myType != DR && myType != DRLNK) {
		popup->insertItem("Edit", this, SLOT(editFromPopUp()));
		popup->insertItem("Open With...", this, SLOT(openWithFromPopUp()));
	}
	popup->insertItem("Duplicate", this, SLOT(duplicateFromPopUp()));
	popup->insertItem("Delete", this, SLOT(deleteFromPopUp()));
   return popup;
}

void FileListObj::InfoObj()
{
	((mfinderapp *)qApp)->getFileAttributesAtPath(myFullPath);
}

void FileListObj::updateMyData()
{
	struct stat listStat;
	char confdir[MAX_PATH_LEN] = "";
	char *pref, *suff, *token, *delims = ".";
	char fname[MAX_NAME_LEN] = "";
	FILE *fAppData;

	sprintf(confdir, "%s/GNUstep/Library/xwfinder", getenv("HOME"));
	lstat(myFullPath, &listStat);

	if(S_ISREG(listStat.st_mode)) {
		if(listStat.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH))
			myType = FILEXE;
		else
			myType = FIL;
	}
	if(S_ISDIR(listStat.st_mode)) 
		myType = DR;
	if(S_ISLNK(listStat.st_mode)) {
		readlink(myFullPath, myLinkTo, MAX_PATH_LEN);
		if(chdir(myFullPath) != 0) {
			if(listStat.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH)) {
				myType = FILEXELNK;
			} else {
				myType = FILLNK;
			}
		} else {
			chdir(confdir);
			myType = DRLNK;
		}
	}		
	if(S_ISCHR(listStat.st_mode)) 
		myType = CHRDV;		
	if(S_ISBLK(listStat.st_mode)) 
		myType = BLCKDV;	
	if(S_ISFIFO(listStat.st_mode)) 
		myType = FIF;		
	if(S_ISSOCK(listStat.st_mode)) 
		myType = SCKT;		
		
	if(isOpen)
		UnselOpen(TRUE);
	else
		Unselect(TRUE);
				
	if(myType == FIL)	{
		if(strcmp(myPath, appsPath) == 0) {
			sprintf(fname, "%s", myName);
			token = strtok(fname, delims);
			pref = token;
			while(token != NULL) {
				suff = token;
				token = strtok(NULL, delims);
			}
		}
	}
	
	DeskObj *d = ((mfinderapp *)qApp)->deskObjectAtPath(myFullPath);
	if(d) {
		setMyDeskObj(d);
		d->setMyRefObj(this);
	} else {
		ismydeskobj = FALSE;
	}
	
}

void FileListObj::updateMyIcon()
{
	Unselect(TRUE);
}

void FileListObj::Select(bool fromlocal)
{
	isSelect = TRUE;
	isOpen = FALSE;
	isOpenUnsl = FALSE;
	icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
																 TRUE, SELECT, havecustomicon));
	label->setPalette(pinv);
	update();
	myobjsframe->unselectOtherFileObjects(myFullPath, TRUE);
	if(fromlocal) {
		if(ismydeskobj) {
			mydeskobj->Select(FALSE);
		}
	}						
}

void FileListObj::Unselect(bool fromlocal)
{
	isSelect = FALSE;
	isOpen = FALSE;
	isOpenUnsl = FALSE;
	icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
															 TRUE, UNSELECT, havecustomicon));
	label->setPalette(pnorm);
	update();
	if(fromlocal) {
		if(ismydeskobj) {
			mydeskobj->Unselect(FALSE);
		}
	}							
}

void FileListObj::SelOpen(bool fromlocal)
{
	isOpenUnsl = FALSE;
	isOpen = TRUE;
	isSelect = TRUE;
	icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
															 TRUE, SELOPEN, havecustomicon));
	label->setPalette(pinv);
	update();
	myobjsframe->unselectOtherFileObjects(myFullPath, TRUE);
	if(fromlocal) {
		if(ismydeskobj) {
			mydeskobj->SelOpen(FALSE);
		}
	}								
}

void FileListObj::UnselOpen(bool fromlocal)
{
	isOpenUnsl = TRUE;
	isOpen = TRUE;
	isSelect = FALSE;
	icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
																 TRUE, OPEN, havecustomicon));
	label->setPalette(pnorm);
	update();
	if(fromlocal) {
		if(ismydeskobj) {
			mydeskobj->UnselOpen(FALSE);
		}
	}								
}

FileListObj* FileListObj::discendentFileObjectAtPath(char *apath)
{
	int i, count;
	FileListObj *fileobj = NULL, *tpmobj = NULL;
	
	count = list.count();
	for(i = 0; i < count; i++) {
		tpmobj = list.at(i);
		if(strcmp(tpmobj->myFullPath, apath) == 0) {
			fileobj = tpmobj;
			return fileobj;
		} else {
			if(tpmobj->isOpenLocal)
				fileobj = tpmobj->discendentFileObjectAtPath(apath);
		}
	}
	return fileobj;
}

void FileListObj::unselectDiscendentFileObjects(char *selobjpath)
{
	int i, count;
	FileListObj *fileobj;

	count = list.count();
	for(i = 0; i < count; i++) {
		fileobj = list.at(i);
		if(strcmp(fileobj->myFullPath, selobjpath) != 0) {
			if(fileobj->isSelect && !fileobj->isOpen)
				fileobj->Unselect(true);
			if(fileobj->isSelect && fileobj->isOpen)
				fileobj->UnselOpen(true);
		}
		if(fileobj->isOpenLocal)
			fileobj->unselectDiscendentFileObjects(selobjpath);
	}
}

void FileListObj::Open(QMouseEvent *e)
{
	if(isOpenFromLocal) {
		if(e->state() & ControlButton)
			((mfinderapp *)qApp)->openSelectedObjectsOfFileListObject(myparentobj, TRUE);
		else
			((mfinderapp *)qApp)->openSelectedObjectsOfFileListObject(myparentobj, FALSE);
	} else {
		if(e->state() & ControlButton)
			((mfinderapp *)qApp)->openSelectedObjectsOfWin(myobjsframe->myfwin, TRUE);
		else
			((mfinderapp *)qApp)->openSelectedObjectsOfWin(myobjsframe->myfwin, FALSE);
	}
}

void FileListObj::OpenLocal()
{
	struct dirent **namelist;
	struct stat listStat;
	char filepath[MAX_PATH_LEN];
	char confdir[MAX_PATH_LEN];
	int m = 0, a = 0, type = 0;
	bool fileok;
	MFinderWin *wintoclose;
	
	isOpenLocal = TRUE;
	if(ismydeskobj)
		mydeskobj->isOpenLocal = TRUE;

	((mfinderapp *)qApp)->setIsOpenLocalOfFileAtPath(myFullPath);

	iview = FALSE;
	sprintf(confdir, "%s/GNUstep/Library/xwfinder", getenv("HOME"));
  	m = scandir(myFullPath, &namelist, 0, alphasort);
	for(a = 0; a <= m - 1; a++) {
		fileok = FALSE;
		if(strcmp(namelist[a]->d_name, ".") != 0 
								&& strcmp(namelist[a]->d_name, "..") != 0)
			fileok = TRUE;
		if(!myobjsframe->dotfiles)	
			if(namelist[a]->d_name[0] == '.')
				fileok = FALSE;

		if(fileok) {
			sprintf(filepath, "%s/%s", myFullPath, namelist[a]->d_name);

   		lstat(filepath, &listStat);
			if(S_ISREG(listStat.st_mode)) {
				if(listStat.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH))
					type = FILEXE;
				else
					type = FIL;
			}
			if(S_ISDIR(listStat.st_mode)) 
				type = DR;
			if(S_ISLNK(listStat.st_mode)) {
				if(chdir(filepath) != 0) {
					if(listStat.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH)) {
						type = FILEXELNK;
					} else {
						type = FILLNK;
					}
				} else {
					chdir(confdir);
					type = DRLNK;
				}
			}		
			if(S_ISCHR(listStat.st_mode)) 
				type = CHRDV;		
			if(S_ISBLK(listStat.st_mode)) 
				type = BLCKDV;	
			if(S_ISFIFO(listStat.st_mode)) 
				type = FIF;		
			if(S_ISSOCK(listStat.st_mode)) 
				type = SCKT;		

			list.inSort(new FileListObj(myobjsframe, myFullPath, 
									namelist[a]->d_name, type, (movx + 1), TRUE, this));
			myobjsframe->allobjslist.append(list.current());
		}
	}
	QResizeEvent e = QResizeEvent(myobjsframe->size(), myobjsframe->size());
	((mfinderapp *)qApp)->sendEvent(myobjsframe, &e);

	wintoclose = NULL;	
	wintoclose = ((mfinderapp *)qApp)->winAtPath(myFullPath);
	if(wintoclose)
		wintoclose->close();
}

void FileListObj::Close(bool fromlocal)
{
	isOpen = FALSE;
	isOpenUnsl = FALSE;
	isSelect = TRUE;
	icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
																 TRUE, SELECT, havecustomicon));
	label->setPalette(pinv);
	update();
	myobjsframe->unselectOtherFileObjects(myFullPath, TRUE);
	if(fromlocal) {
		if(ismydeskobj) {
			mydeskobj->Close(FALSE);
		}
	}									
}

void FileListObj::CloseLocal()
{
	FileListObj *fileobj;
	int i, count;
	
	((mfinderapp *)qApp)->unsetIsOpenLocalOfFileAtPath(myFullPath);
	
	isOpenLocal = FALSE;
	if(ismydeskobj)
		mydeskobj->isOpenLocal = FALSE;
	count = list.count();
	for(i = 0; i < count; i++) {
		fileobj = list.at(i);
		if(fileobj->isOpenLocal)
			fileobj->CloseLocal();
		myobjsframe->allobjslist.removeRef(fileobj);
		list.removeRef(fileobj);
		count--;
		i--;
	}
	arrowlabel->Close();
	QResizeEvent e = QResizeEvent(myobjsframe->size(), myobjsframe->size());
	((mfinderapp *)qApp)->sendEvent(myobjsframe, &e);
}

void FileListObj::openFromPopUp()
{
	((mfinderapp *)qApp)->openSelectedObjectsOfWin(myobjsframe->myfwin, FALSE);		
}

void FileListObj::renameFromPopUp()
{
	((mfinderapp *)qApp)->renameFileObjects(myobjsframe->myfwin);
}

void FileListObj::infoFromPopUp()
{
	((mfinderapp *)qApp)->InfoObj(myobjsframe->myfwin);
}

void FileListObj::editFromPopUp()
{
	((mfinderapp *)qApp)->Edit(myobjsframe->myfwin);
}

void FileListObj::openWithFromPopUp()
{
	((mfinderapp *)qApp)->OpenWith(myobjsframe->myfwin);
}

void FileListObj::duplicateFromPopUp()
{
	((mfinderapp *)qApp)->duplicateFileObjects(myobjsframe->myfwin);
}

void FileListObj::deleteFromPopUp()
{
	((mfinderapp *)qApp)->removeFileObjects(myobjsframe->myfwin);
}

void FileListObj::updateContents()
{
	struct dirent **namelist;
	int m, count;
	bool fileok, found;
	FileListObj *fileobj;	
	QList<char> newlist; 
	QList<char> difflistp;
	QList<char> difflistm;
	
	newlist.setAutoDelete(TRUE);
	difflistp.setAutoDelete(TRUE);
	difflistm.setAutoDelete(TRUE);
	
  	m = scandir(myFullPath, &namelist, 0, alphasort);
	for(int i = 0; i <= m - 1; i++) {
		fileok = FALSE;
		if(strcmp(namelist[i]->d_name, ".") != 0 
								&& strcmp(namelist[i]->d_name, "..") != 0)
			fileok = TRUE;
		if(!myobjsframe->dotfiles)	
			if(namelist[i]->d_name[0] == '.')
				fileok = FALSE;

		if(fileok) {
			newlist.append(new char[strlen(namelist[i]->d_name)+1]);
			strcpy(newlist.current(), namelist[i]->d_name);
		}
	}
	
	for(int i = 0; i < newlist.count(); i++) {
		found = FALSE;
		for(int j = 0; j < list.count(); j++)
			if(strcmp(newlist.at(i), list.at(j)->myName) == 0)
				found = TRUE;
		if(!found) {
			difflistp.append(new char[strlen(newlist.at(i))+1]);
			strcpy(difflistp.current(), newlist.at(i));
		}
	}
	
	for(int i = 0; i < list.count(); i++) {
		found = FALSE;
		for(int j = 0; j < newlist.count(); j++)
			if(strcmp(list.at(i)->myName, newlist.at(j)) == 0)
				found = TRUE;
		if(!found) {
			difflistm.append(new char[strlen(list.at(i)->myName)+1]);
			strcpy(difflistm.current(), list.at(i)->myName);
		}
	}
	
	if(difflistp.count() > 0)
		for(int i = 0; i < difflistp.count(); i++)
			addAnObject(difflistp.at(i));
	
	if(difflistm.count() > 0) {
		count = list.count();
		for(int i = 0; i < count; i++) {
			fileobj = list.at(i);
			for(int j = 0; j < difflistm.count(); j++) {
				if(strcmp(fileobj->myName, difflistm.at(j)) == 0) {
					myobjsframe->allobjslist.removeRef(fileobj);
					list.removeRef(fileobj);
					count--;
					i--;
				}
			}
		}
	}
	
	newlist.clear();	
	difflistp.clear();	
	difflistm.clear();
}

void FileListObj::removeAnObject(FileListObj *obj)
{
	myobjsframe->allobjslist.removeRef(obj);
	list.removeRef(obj);
}

void FileListObj::addAnObject(char *nameobj)
{
	char filepath[MAX_PATH_LEN];
	char confdir[MAX_PATH_LEN];
	struct stat listStat;
	int type = 0;

	iview = FALSE;
	
	sprintf(confdir, "%s/GNUstep/Library/xwfinder", getenv("HOME"));
	sprintf(filepath, "%s/%s", myFullPath, nameobj);

   lstat(filepath, &listStat);
	if(S_ISREG(listStat.st_mode)) {
		if(listStat.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH))
			type = FILEXE;
		else
			type = FIL;
	}
	if(S_ISDIR(listStat.st_mode)) 
		type = DR;
	if(S_ISLNK(listStat.st_mode)) {
		if(chdir(filepath) != 0) {
			if(listStat.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH)) {
				type = FILEXELNK;
			} else {
				type = FILLNK;
			}
		} else {
			chdir(confdir);
			type = DRLNK;
		}
	}		
	if(S_ISCHR(listStat.st_mode)) 
		type = CHRDV;		
	if(S_ISBLK(listStat.st_mode)) 
		type = BLCKDV;	
	if(S_ISFIFO(listStat.st_mode)) 
		type = FIF;		
	if(S_ISSOCK(listStat.st_mode)) 
		type = SCKT;		

	list.inSort(new FileListObj(myobjsframe, myFullPath, 
											nameobj, type, (movx + 1), TRUE, this));
	myobjsframe->allobjslist.append(list.current());
}

void FileListObj::changeNormalDirFirstSort()
{
	FileListObj *fileobj;
	QList<FileListObj> tmplist; 	

	iview = FALSE;

	dirfirst = myobjsframe->dirfirst;
	dfirst = dirfirst;
	tmplist.clear();
	for(int i = 0; i < list.count(); i++) {
		fileobj = list.at(i);
		tmplist.inSort(fileobj);
		if(fileobj->isOpenLocal)
			fileobj->changeNormalDirFirstSort();		
	}
	list.setAutoDelete(FALSE);
	list = tmplist;
	list.setAutoDelete(TRUE);
}

void FileListObj::mousePressEvent(QMouseEvent *e)
{
	if(e->pos().x() < 100 + leftspace + arrowlabel->width() + (sep * movx)) {
		candrag = TRUE;
		if(e->button() == RightButton) {
			myobjsframe->isShiftClick = FALSE;
			if(!popupMenu)
     			popupMenu = createPopup();
			popupMenu->popup(mapToGlobal(e->pos()));
		}

		if(e->state() & ShiftButton) {
			myobjsframe->isShiftClick = TRUE;
			if(isSelect) {
				if(isOpen) UnselOpen(true);
				else Unselect(true);
				return;
			} else {
				if(isOpen) SelOpen(true);
				else Select(true);
			}
		} else {
			myobjsframe->isShiftClick = FALSE;
			if(isOpenUnsl)
				SelOpen(true);
			if(isSelect) {

			} else {
				Select(true);
			}
		}
	} else {
		candrag = FALSE;
		QMouseEvent ev = QMouseEvent(e->type(), mapToParent(e->pos()), e->button(), e->state());
		((mfinderapp *)qApp)->sendEvent(myobjsframe, &ev);
	}
}

void FileListObj::mouseReleaseEvent(QMouseEvent *e)
{
	candrag = TRUE;
	((mfinderapp *)qApp)->sendEvent(myobjsframe, e);
}

void FileListObj::resizeEvent(QResizeEvent *e)
{
	resize(myobjsframe->width()-10, hght);

	arrowlabel->move(leftspace, 0);
	label->move(leftspace + (sep * movx) + arrowlabel->width() 
												+ icona.width() + 10, 0);
	labdim->move(myobjsframe->labelsshift, 0);
	labusr->move(myobjsframe->labelsshift + labdim->width(), 0);
	labperm->move(myobjsframe->labelsshift + labdim->width() 
																	+ labusr->width(), 0);
	labdate->move(myobjsframe->labelsshift + labdim->width()
											+ labusr->width() + labperm->width(), 0);
}

void FileListObj::shiftLabels()
{
	resize(myobjsframe->width(), hght);

	labdim->move(myobjsframe->labelsshift, 0);
	labusr->move(myobjsframe->labelsshift + labdim->width(), 0);
	labperm->move(myobjsframe->labelsshift + labdim->width() 
																	+ labusr->width(), 0);
	labdate->move(myobjsframe->labelsshift + labdim->width()
											+ labusr->width() + labperm->width(), 0);
}

void FileListObj::mouseDoubleClickEvent(QMouseEvent *e)
{
	Open(e);
}

void FileListObj::mouseMoveEvent(QMouseEvent *e)
{
	QStrList urls;
	QPixmap dragpix;

	if(e->pos().x() < 100 + leftspace + arrowlabel->width() + (sep * movx) && candrag) {
		dragdelay++;
		if(dragdelay >= 2) {
			urls = myobjsframe->makeUrlList();
			if(urls.count() == 1)
				dragpix = prepDragPix();
			else
				dragpix = ((mfinderapp *)qApp)->dragIconForFiles(urls.count());			
			myobjsframe->isShiftClick = FALSE;
			myobjsframe->unselectAllFileObjects();
			dropopenapp = isPlainFileAtPath(urls.at(0)) ? TRUE : FALSE;
			urlDragObj = new QUrlDrag(urls, this, "udrag");
			urlDragObj->setPixmap(dragpix, QPoint(0, 0));
			((mfinderapp *)qApp)->fileDragObj = NULL;
			((mfinderapp *)qApp)->deskDragObj = NULL;
			((mfinderapp *)qApp)->fileListDragObj = this;
			((mfinderapp *)qApp)->isdragcopying = FALSE;
			if(e->state() & ControlButton)
				((mfinderapp *)qApp)->isdragcopying = TRUE;
			((mfinderapp *)qApp)->xdnddone = FALSE;
			
			urlDragObj->drag();

			if(!((mfinderapp *)qApp)->xdnddone) 
				((mfinderapp *)qApp)->offixDrop(this);
		}
	} else {
		QMouseEvent ev = QMouseEvent(e->type(), mapToParent(e->pos()), e->button(), e->state());
		((mfinderapp *)qApp)->sendEvent(myobjsframe, &ev);
	}
}

void FileListObj::dragEnterEvent(QDragEnterEvent *e)
{
	if(e->pos().x() < 100 + leftspace + arrowlabel->width() + (sep * movx)) {
		if(QUrlDrag::canDecode(e)) {
			if(isDir) {
				e->accept();
				myobjsframe->isShiftClick = FALSE;
				if(isOpenUnsl)
					SelOpen(true);
				if(isSelect) {

				} else {
					Select(true);
				}
				update();
			}  
		}
	} else {
		((mfinderapp *)qApp)->sendEvent(myobjsframe, e);
	}
}

void FileListObj::dragLeaveEvent(QDragLeaveEvent *e)
{
	if(isSelect) {
		if(isOpen)
			UnselOpen(true);
		else
			Unselect(true);
	}
	myobjsframe->isDragTarget = FALSE;
	myobjsframe->update();
}

void FileListObj::dropEvent(QDropEvent *e)
{
	QStrList droplist;

	if(e->pos().x() < 100 + leftspace + arrowlabel->width() + (sep * movx)) {
		if(isDir) {
			e->accept();
			QUrlDrag::decode(e, droplist);
			((mfinderapp *)qApp)->moveOrCopy(droplist, myFullPath, this);
		}
	} else {
		myobjsframe->isDragTarget = FALSE;
		myobjsframe->update();
		((mfinderapp *)qApp)->sendEvent(myobjsframe, e);
	}			
}

QPixmap FileListObj::prepDragPix()
{
	QPixmap dragpix;
	int labelw, labelh, icnw, icnh;

	icnw = icona.width() + 2;
	icnh = icona.height() + 2;
	labelw = label->width() + 2;
	labelh = label->height() + 2;
	dragpix = QPixmap(icnw + 5 + labelw, icnh);
 	dragpix.fill(color1);
	bitBlt(&dragpix, 1, 1, &icona);
	bitBlt(&dragpix, icnw + 5, 1, label);
	dragpix.setMask(dragpix.createHeuristicMask());	

	return dragpix;
}

void FileListObj::drawSelRect(QRect selrect, bool mustdraw)
{
	QRect oldrect, labelrect, r;
	
	if(mustdraw) {
		mustdrawselrect = TRUE;
		
		oldrect = r1o;
		r1o = QRect(selrect.topLeft(), selrect.topRight());
		if(oldrect.topLeft().y() > 1)
			update(oldrect);
		if(r1o.topLeft().y() > 1)
			update(r1o);

		oldrect = r2o;
		r2o = QRect(selrect.bottomLeft(), selrect.bottomRight());
		if(oldrect.topLeft().y() < height() - 1)
			update(oldrect);
		if(r2o.topLeft().y() < height() - 1) 
			update(r2o);
		
		oldrect = r1v;
		r1v = QRect(selrect.topLeft(), selrect.bottomLeft());
		if(oldrect.topLeft().x() > 1)
			update(oldrect);
		if(r1v.topLeft().x() > 1)
			update(r1v);

		oldrect = r2v;
		r2v = QRect(selrect.topRight(), selrect.bottomRight());
		if(oldrect.topLeft().x() < width() - 1)
			update(oldrect);
		if(r2v.topLeft().x() < width() - 1) 
			update(r2v);
		
		r = QRect(labdim->mapToParent(labdim->rect().topLeft()), 
							 labdim->mapToParent(labdim->rect().bottomRight()));
		if(selrect.intersects(r)) {
	 		labelrect = QRect(labdim->mapFromParent(selrect.intersect(r).topLeft()), 
										labdim->mapFromParent(selrect.intersect(r).bottomRight()));
			labdim->drawSelRect(labelrect, TRUE);
		} else {
			labdim->drawSelRect(QRect(0,0,0,0), FALSE);
		}
		
		r = QRect(labusr->mapToParent(labusr->rect().topLeft()), 
							 labusr->mapToParent(labusr->rect().bottomRight()));
		if(selrect.intersects(r)) {
	 		labelrect = QRect(labusr->mapFromParent(selrect.intersect(r).topLeft()), 
										labusr->mapFromParent(selrect.intersect(r).bottomRight()));
			labusr->drawSelRect(labelrect, TRUE);
		} else {
			labusr->drawSelRect(QRect(0,0,0,0), FALSE);
		}

		r = QRect(labperm->mapToParent(labperm->rect().topLeft()), 
							 labperm->mapToParent(labperm->rect().bottomRight()));
		if(selrect.intersects(r)) {
	 		labelrect = QRect(labperm->mapFromParent(selrect.intersect(r).topLeft()), 
										labperm->mapFromParent(selrect.intersect(r).bottomRight()));
			labperm->drawSelRect(labelrect, TRUE);
		} else {
			labperm->drawSelRect(QRect(0,0,0,0), FALSE);
		}

		r = QRect(labdate->mapToParent(labdate->rect().topLeft()), 
							 labdate->mapToParent(labdate->rect().bottomRight()));
		if(selrect.intersects(r)) {
	 		labelrect = QRect(labdate->mapFromParent(selrect.intersect(r).topLeft()), 
										labdate->mapFromParent(selrect.intersect(r).bottomRight()));
			labdate->drawSelRect(labelrect, TRUE);
		} else {
			labdate->drawSelRect(QRect(0,0,0,0), FALSE);
		}
		
	} else {
		if(mustdrawselrect) {
			mustdrawselrect = FALSE;
			update(r1o);
			update(r2o);
			update(r1v);
			update(r2v);
			
			labdim->drawSelRect(QRect(0,0,0,0), FALSE);
			labusr->drawSelRect(QRect(0,0,0,0), FALSE);
			labperm->drawSelRect(QRect(0,0,0,0), FALSE);
			labdate->drawSelRect(QRect(0,0,0,0), FALSE);
		}
	}
}

void FileListObj::paintEvent(QPaintEvent *)
{
	QPainter p;
	
	p.begin(this);
	p.drawPixmap(leftspace + arrowlabel->width() + (sep * movx), 0, icona);
	if(mustdrawselrect) {
		if(r1o.topLeft().y() > 1)
			p.drawRect(r1o);
		if(r2o.topLeft().y() < height() - 1)
			p.drawRect(r2o);
		if(r1v.topLeft().x() > 1)
			p.drawRect(r1v);
		if(r2v.topLeft().x() < width() - 1)
			p.drawRect(r2v);
	}
	p.end();
}

bool FileListObj::eventFilter(QObject *, QEvent *e)
{
	if(e->type() == Event_MouseButtonPress 
			|| e->type() == Event_MouseButtonDblClick) {
		this->event(e);
		return TRUE;
	}
  	return FALSE;
}

