#ifndef FILELISTOBJ_H
#define FILELISTOBJ_H

#include <qwidget.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <qcolor.h>
#include <qpalette.h> 
#include <qfontmet.h> 
#include <qfont.h> 
#include <qlabel.h> 
#include <qdragobject.h>
#include <qdropsite.h>
#include <qstrlist.h> 
#include <qlist.h> 
#include <pwd.h>
#include "deskobj.h"
#include "dialogs.h"
#include "common.h"
#include "popup.h"

class ObjsListFrame;

class ArrowLabel : public QLabel
{
	Q_OBJECT
public:
	ArrowLabel(QWidget *parent=0, const char *name=0, bool isdir=FALSE,
		bool isopen=FALSE, const char **arrovdesc=0, const char **openarrovdesc=0);

	~ArrowLabel();
 	
	bool isopen;
	bool isdir;
	QPixmap arrowpix;
	QPixmap openarrowpix;

	void Close();
	
signals:
	void arrowopen();
	void arrowclose();
		
private:
	void mousePressEvent(QMouseEvent *);
};

class MyLabel : public QLabel, QDropSite
{
	Q_OBJECT
public:
	MyLabel(QWidget *parent=0, const char *name=0);

	void drawSelRect(QRect, bool);		

	FileListObj *myflistobj;
	bool mustdrawselrect;
	QRect r1o, r2o, r1v, r2v;
			
private:
	void mousePressEvent(QMouseEvent *);
	void mouseReleaseEvent(QMouseEvent *);	
	void mouseMoveEvent(QMouseEvent *);
	void dragEnterEvent(QDragEnterEvent *);
	void dragLeaveEvent(QDragLeaveEvent *);
	void dropEvent(QDropEvent *);	
	void paintEvent(QPaintEvent *);	
};

class FileListObj : public QWidget, QDropSite
{
	Q_OBJECT
public:
	FileListObj(QWidget *parent=0, const char *path=0, const char *name=0, int type=-1, 
				int movx=0, bool isOpenFromLocal=FALSE, FileListObj *parentobj=0);
	~FileListObj();

	QList<FileListObj> list; 

	char myName[MAX_NAME_LEN];
	char myPath[MAX_PATH_LEN];
	char myFullPath[MAX_PATH_LEN];
	char myLinkTo[MAX_PATH_LEN];
	char myAppName[256];	
	int myType;
		
	int movx, sep, leftspace, hght, hspace;
	
	ObjsListFrame *myobjsframe;
	FileListObj *myparentobj;
	FileListObj *objtokeep;
	DeskObj *mydeskobj;
	bool isDir;

	bool isSelect;
	bool isOpen;
	bool isOpenUnsl;
	bool isOpenLocal;
	bool isOpenFromLocal;
	bool ismydeskobj;
	bool dirfirst;
	bool mustdrawselrect;
	bool candrag;
	bool havecustomicon;

	QPixmap icona;
	QLabel *label;
	ArrowLabel *arrowlabel; 
	MyLabel *labdim;
	MyLabel *labusr;
	MyLabel *labperm;
	MyLabel *labdate;
	
	QPalette pnorm, pinv;
	QFont font;
	Popup *popupMenu;
	Dialog *dialog;
	QRect SelRect, r1o, r2o, r1v, r2v;
	QUrlDrag *urlDragObj;

	void Select(bool);
	void Unselect(bool);
	void SelOpen(bool);
	void UnselOpen(bool);
	void Open(QMouseEvent *);
	void Close(bool);
	
	FileListObj *discendentFileObjectAtPath(char *);
	void unselectDiscendentFileObjects(char *);
	void addAnObject(char *);
	void removeAnObject(FileListObj *);
	void updateContents();	
	void updateLabelsContents();
	void shiftLabels();
	void changeNormalDirFirstSort();
	void drawSelRect(QRect, bool);		
	void setMyDeskObj(DeskObj*);
	void unsetMyDeskObj();
	void updateMyData();
	void updateMyIcon();
	void InfoObj();
	QPixmap prepDragPix();
	Popup* createPopup();

signals:

public slots:
	void OpenLocal();
	void CloseLocal();
	void openFromPopUp();
	void renameFromPopUp();
	void infoFromPopUp();
	void editFromPopUp();
	void openWithFromPopUp();
	void duplicateFromPopUp();
	void deleteFromPopUp();	

protected:
	void mousePressEvent(QMouseEvent *);
	void mouseReleaseEvent(QMouseEvent *);	
	void mouseDoubleClickEvent(QMouseEvent *);
	void mouseMoveEvent(QMouseEvent *);
	void resizeEvent(QResizeEvent *);
	void paintEvent(QPaintEvent *);	
	void dragEnterEvent(QDragEnterEvent *);
	void dragLeaveEvent(QDragLeaveEvent *);
	void dropEvent(QDropEvent *);	

	bool eventFilter(QObject *, QEvent *);
	
private:
	int dragdelay;

};

#endif // FILELISTOBJ_H
