#include "fileobj.h"
#include "mfinderapp.h"
#include "functions.h"

char cutname[256] = "";
extern char appsPath[];
extern bool dropopenapp;

FileObj::FileObj(QWidget *parent, const char *path, 
										const char *name, int type, int viewtype)
        :QWidget(parent, path),
      	QDropSite(this)
{
	char *pref, *suff, *token, *delims = ".";
	char fname[MAX_NAME_LEN] = "";
	FILE *fAppData;
	
	strcpy(myPath, path);
	strcpy(myName, name);
	if(strcmp(myPath, "/") != 0)
		sprintf(myFullPath, "%s/%s", myPath, myName);
	else
		sprintf(myFullPath, "%s%s", myPath, myName);
	myType = type;
	this->viewtype = viewtype;
	if(myType == DR || myType == DRLNK)	
		isDir = TRUE;
	else
		isDir = FALSE;
	if(myType == FILLNK || myType == DRLNK || myType == FILEXELNK)	
		readlink(myFullPath, myLinkTo, MAX_PATH_LEN);

	if(myType == FIL)	{
		if(strcmp(myPath, appsPath) == 0) {
			sprintf(fname, "%s", myName);
			token = strtok(fname, delims);
			pref = token;
			while(token != NULL) {
				suff = token;
				token = strtok(NULL, delims);
			}
		}
	}
		
	myobjsframe = (ObjsFrame *)parent;
	
	QColor color = QColor(230, 230, 230);
	QColorGroup cnorm = QColorGroup(black, color, color.light(), color.dark(), 
																color.dark(120), black, color);
	QColorGroup cinv = QColorGroup(white, black, color.light(), color.dark(), 
															color.dark(120), white, black);
	pnorm = QPalette(cnorm, cnorm, cnorm);
	pinv = QPalette(cinv, cinv, cinv);
	font = QFont("Helvetica", 10);

	havecustomicon = ((mfinderapp *)qApp)->hasCustomIconFileAtPath(myFullPath);

	if(viewtype == BYICONS) 
		icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
															 FALSE, UNSELECT, havecustomicon));
	else
		icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
		 														TRUE, UNSELECT, havecustomicon));

	resize(icona.width(), icona.height());

	label = new QLabel(myobjsframe, myName);
	label->setAutoResize(TRUE);
	label->setPalette(pnorm);
	label->setFont(font);
	label->installEventFilter(this); 
	if(viewtype == BYICONS) 
		label->setText(cutFileLabelText(myName));
	else
		label->setText(myName);
	
	isSelect = FALSE;
	isOpen = FALSE;
	isOpenUnsl = FALSE;
	isOpenLocal = FALSE;
	mustdrawselrect = FALSE;
	popupMenu = NULL;
	dragdelay = 0;
	r1o = QRect(0,0,0,0);
	r2o = QRect(0,0,0,0);
	r1v = QRect(0,0,0,0);
	r2v = QRect(0,0,0,0);
	
	if(isDir)
		if(((mfinderapp *)qApp)->winAtPath(myFullPath))
			UnselOpen(true);

	DeskObj *d = ((mfinderapp *)qApp)->deskObjectAtPath(myFullPath);
	if(d) {
		setMyDeskObj(d);
		d->setMyRefObj(this);
	} else {
		ismydeskobj = FALSE;
	}
	
	show();
	label->show();
}

FileObj::~FileObj()
{
	if(ismydeskobj)
		mydeskobj->unsetMyRefObj();
	label->~QLabel();
}

Popup* FileObj::createPopup()
{
	Popup *popup;
	
	popup = new Popup();
	popup->insertItem("Open", this, SLOT(openFromPopUp()));
	popup->insertItem("Rename", this, SLOT(renameFromPopUp()));
	popup->insertItem("Info", this, SLOT(infoFromPopUp()));
	if(myType != DR && myType != DRLNK) {
		popup->insertItem("Edit", this, SLOT(editFromPopUp()));
		popup->insertItem("Open With...", this, SLOT(openWithFromPopUp()));
	}
	popup->insertItem("Duplicate", this, SLOT(duplicateFromPopUp()));
	popup->insertItem("Delete", this, SLOT(deleteFromPopUp()));
   return popup;
}

void FileObj::InfoObj()
{
	((mfinderapp *)qApp)->getFileAttributesAtPath(myFullPath);
}

void FileObj::updateMyData()
{
	struct stat listStat;
	char confdir[MAX_PATH_LEN] = "";
	char *pref, *suff, *token, *delims = ".";
	char fname[MAX_NAME_LEN] = "";
	FILE *fAppData;

	sprintf(confdir, "%s/GNUstep/Library/xwfinder", getenv("HOME"));
	lstat(myFullPath, &listStat);

	if(S_ISREG(listStat.st_mode)) {
		if(listStat.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH))
			myType = FILEXE;
		else
			myType = FIL;
	}
	if(S_ISDIR(listStat.st_mode)) 
		myType = DR;
	if(S_ISLNK(listStat.st_mode)) {
		readlink(myFullPath, myLinkTo, MAX_PATH_LEN);
		if(chdir(myFullPath) != 0) {
			if(listStat.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH)) {
				myType = FILEXELNK;
			} else {
				myType = FILLNK;
			}
		} else {
			chdir(confdir);
			myType = DRLNK;
		}
	}		
	if(S_ISCHR(listStat.st_mode)) 
		myType = CHRDV;		
	if(S_ISBLK(listStat.st_mode)) 
		myType = BLCKDV;	
	if(S_ISFIFO(listStat.st_mode)) 
		myType = FIF;		
	if(S_ISSOCK(listStat.st_mode)) 
		myType = SCKT;		
		
	if(isOpen)
		UnselOpen(TRUE);
	else
		Unselect(TRUE);
		
	if(myType == FIL)	{
		if(strcmp(myPath, appsPath) == 0) {
			sprintf(fname, "%s", myName);
			token = strtok(fname, delims);
			pref = token;
			while(token != NULL) {
				suff = token;
				token = strtok(NULL, delims);
			}
		}
	}
	DeskObj *d = ((mfinderapp *)qApp)->deskObjectAtPath(myFullPath);
	if(d) {
		setMyDeskObj(d);
		d->setMyRefObj(this);
	} else {
		ismydeskobj = FALSE;
	}
}

void FileObj::updateMyIcon()
{
	Unselect(TRUE);
}

void FileObj::setMyDeskObj(DeskObj *dobj)
{
	mydeskobj = NULL;
	mydeskobj = dobj;
	ismydeskobj = TRUE;
}

void FileObj::unsetMyDeskObj()
{
	mydeskobj = NULL;
	ismydeskobj = FALSE;
}

void FileObj::Select(bool fromlocal)
{
	isSelect = TRUE;
	isOpen = FALSE;
	isOpenUnsl = FALSE;
	if(viewtype == BYICONS) {
		icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
																 FALSE, SELECT, havecustomicon));
		label->setText(myName);
		label->raise();
		adjustLabelPosition();
	} else {
		icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
																	 TRUE, SELECT, havecustomicon));
	}
	label->setPalette(pinv);
	update();
	myobjsframe->unselectOtherFileObjects(myName);
	if(fromlocal) {
		if(ismydeskobj) {
			mydeskobj->Select(FALSE);
		}
	}	
}

void FileObj::Unselect(bool fromlocal)
{
	isSelect = FALSE;
	isOpen = FALSE;
	isOpenUnsl = FALSE;
	if(viewtype == BYICONS) {
		icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
																 FALSE, UNSELECT, havecustomicon));
		label->setText(cutFileLabelText(myName));
		adjustLabelPosition();
	} else {
		icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
																 TRUE, UNSELECT, havecustomicon));
	}
	label->setPalette(pnorm);
	update();
	if(fromlocal) {
		if(ismydeskobj) {
			mydeskobj->Unselect(FALSE);
		}
	}		
}

void FileObj::SelOpen(bool fromlocal) 
{
	isOpenUnsl = FALSE;
	isOpen = TRUE;
	isSelect = TRUE;
	if(viewtype == BYICONS) {
		icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
																 FALSE, SELOPEN, havecustomicon));
		label->setText(myName);
		adjustLabelPosition();
	} else {
		icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
																 TRUE, SELOPEN, havecustomicon));
	}
	label->setPalette(pinv);
	label->raise();
	update();
	myobjsframe->unselectOtherFileObjects(myName);
	if(fromlocal) {
		if(ismydeskobj) {
			mydeskobj->SelOpen(FALSE);
		}
	}			
}

void FileObj::UnselOpen(bool fromlocal)
{
	isOpenUnsl = TRUE;
	isOpen = TRUE;
	isSelect = FALSE;
	if(viewtype == BYICONS) {
		icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
																	 FALSE, OPEN, havecustomicon));
		label->setText(cutFileLabelText(myName));
		adjustLabelPosition();
	} else {
		icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
																	 TRUE, OPEN, havecustomicon));
	}
	label->setPalette(pnorm);
	update();
	if(fromlocal) {
		if(ismydeskobj) {
			mydeskobj->UnselOpen(FALSE);
		}
	}				
}

void FileObj::Open(QMouseEvent *e)
{
	if(e->state() & ControlButton)
		((mfinderapp *)qApp)->openSelectedObjectsOfWin(myobjsframe->myfwin, TRUE);
	else
		((mfinderapp *)qApp)->openSelectedObjectsOfWin(myobjsframe->myfwin, FALSE);
}

void FileObj::Close(bool fromlocal)
{
	isOpen = FALSE;
	isOpenUnsl = FALSE;
	isSelect = TRUE;
	if(viewtype == BYICONS) {
		icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
																 FALSE, SELECT, havecustomicon));
		label->setText(myName);
		label->raise();
		adjustLabelPosition();
	} else {
		icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
																	 TRUE, SELECT, havecustomicon));
	}
	label->setPalette(pinv);
	update();
	myobjsframe->unselectOtherFileObjects(myName);
	if(fromlocal) {
		if(ismydeskobj) {
			mydeskobj->Close(FALSE);
		}
	}					
}

void FileObj::openFromPopUp()
{
	((mfinderapp *)qApp)->openSelectedObjectsOfWin(myobjsframe->myfwin, FALSE);		
}

void FileObj::renameFromPopUp()
{
	((mfinderapp *)qApp)->renameFileObjects(myobjsframe->myfwin);
}

void FileObj::infoFromPopUp()
{
	((mfinderapp *)qApp)->InfoObj(myobjsframe->myfwin);
}

void FileObj::editFromPopUp()
{
	((mfinderapp *)qApp)->Edit(myobjsframe->myfwin);
}

void FileObj::openWithFromPopUp()
{
	((mfinderapp *)qApp)->OpenWith(myobjsframe->myfwin);
}

void FileObj::duplicateFromPopUp()
{
	((mfinderapp *)qApp)->duplicateFileObjects(myobjsframe->myfwin);
}

void FileObj::deleteFromPopUp()
{
	((mfinderapp *)qApp)->removeFileObjects(myobjsframe->myfwin);
}

void FileObj::adjustLabelPosition()
{
	if(viewtype == BYICONS) {
		if(label->width() < width())
			label->move(x() + ((width() - label->width()) / 2), y() + 50);
		else
			label->move(x() - ((label->width() - width()) / 2), y() + 50);
	} else {
		label->move(x() + width() + 5, y());
	}
}

char *FileObj::cutFileLabelText(char *labelname)
{
	int w = 0, cw = 0, i = 0;
	int maxlabelw = 60;

	sprintf(cutname, "");
	QFontMetrics fm = QFontMetrics(font);
	QFontMetrics fmm = QFontMetrics(font);
	w = fm.boundingRect(labelname, strlen(labelname)).width();
	if(w > maxlabelw) {
		i = 0;
		while(cw <= maxlabelw - 3) {
			cutname[i] = labelname[i];
			cutname[i+1] = '\0';
			cw = fmm.boundingRect(cutname, strlen(cutname)).width();
			i++;
		}
		cutname[i] = '\0';
		if(strcmp(labelname, cutname) != 0) {
			if((strlen(labelname) - strlen(cutname)) < 3) {
				strncpy(cutname, labelname, 256);
			} else {
				cutname[i] = '.';
				cutname[i + 1] = '.';
				cutname[i + 2] = '.';
				cutname[i + 3] = '\0';
			}
		} 
	} else {
		strncpy(cutname, labelname, 256);
	}
	
	return cutname;
}

void FileObj::moveEvent(QMoveEvent *e)
{
	adjustLabelPosition();
}

void FileObj::mousePressEvent(QMouseEvent *e)
{
	dragdelay = 0;
	if(e->button() == RightButton) {
		myobjsframe->isShiftClick = FALSE;
		if(!popupMenu)
     		popupMenu = createPopup();
		popupMenu->popup(mapToGlobal(e->pos()));
	}

	if(e->state() & ShiftButton) {
		myobjsframe->isShiftClick = TRUE;
		if(isSelect) {
			if(isOpen) UnselOpen(true);
			else Unselect(true);
			return;
		} else {
			if(isOpen) SelOpen(true);
			else Select(true);
		}
	} else {
		myobjsframe->isShiftClick = FALSE;
		if(isOpenUnsl)
			SelOpen(true);
		if(isSelect) {

		} else {
			Select(true);
		}
	}
}

void FileObj::mouseDoubleClickEvent(QMouseEvent *e)
{
	dragdelay = 0;
	Open(e);
}

void FileObj::mouseMoveEvent(QMouseEvent *e)
{
	QStrList urls;
	QPixmap dragpix;

	dragdelay++;
	if(dragdelay >= 2) {
		urls = myobjsframe->makeUrlList();
		if(urls.count() == 1)
			dragpix = prepDragPix();
		else
			dragpix = ((mfinderapp *)qApp)->dragIconForFiles(urls.count());			
		myobjsframe->isShiftClick = FALSE;
		myobjsframe->unselectAllFileObjects();
		dropopenapp = isPlainFileAtPath(urls.at(0)) ? TRUE : FALSE;
		urlDragObj = new QUrlDrag(urls, this, "udrag");
		urlDragObj->setPixmap(dragpix, QPoint(0, 0));
		((mfinderapp *)qApp)->fileListDragObj = NULL;
		((mfinderapp *)qApp)->deskDragObj = NULL;
		((mfinderapp *)qApp)->fileDragObj = this;
		((mfinderapp *)qApp)->isdragcopying = FALSE;
		if(e->state() & ControlButton)
			((mfinderapp *)qApp)->isdragcopying = TRUE;
		((mfinderapp *)qApp)->xdnddone = FALSE;
		
		urlDragObj->drag();
		
		if(!((mfinderapp *)qApp)->xdnddone) 
			((mfinderapp *)qApp)->offixDrop(this);
	}
}

void FileObj::dragEnterEvent(QDragEnterEvent *e)
{
	if(QUrlDrag::canDecode(e)) {
		if(isDir) {
			e->accept();
			myobjsframe->isShiftClick = FALSE;
			if(isOpenUnsl)
				SelOpen(true);
			if(isSelect) {

			} else {
				Select(true);
			}
			update();
		}
	}
}

void FileObj::dragLeaveEvent(QDragLeaveEvent *e)
{
	if(isSelect) {
		if(isOpen)
			UnselOpen(true);
		else
			Unselect(true);
	}
}

void FileObj::dropEvent(QDropEvent *e)
{
	QStrList droplist;

	if(isDir) {
		e->accept();
		QUrlDrag::decode(e, droplist);
		((mfinderapp *)qApp)->moveOrCopy(droplist, myFullPath, this);
	}
}

QPixmap FileObj::prepDragPix()
{
	QPixmap dragpix;
	int labelw, labelh, icnw, icnh;

	icnw = icona.width() + 2;
	icnh = icona.height() + 2;
	labelw = label->width() + 2;
	labelh = label->height() + 2;
	if(viewtype == BYICONS) {
		if(labelw < icnw) {
			dragpix = QPixmap(icnw, 50 + labelh);
 			dragpix.fill(color1);
			bitBlt(&dragpix, 1, 1, &icona);
			bitBlt(&dragpix, (icnw - labelw) / 2, 50, label);
		} else {
			dragpix = QPixmap(labelw, 50 + labelh);
 			dragpix.fill(color1);
			bitBlt(&dragpix, (labelw - icnw) / 2, 1, &icona);
			bitBlt(&dragpix, 1, 50, label);
		}
	} else {
		dragpix = QPixmap(icnw + 5 + labelw, icnh);
 		dragpix.fill(color1);
		bitBlt(&dragpix, 1, 1, &icona);
		bitBlt(&dragpix, icnw + 5, 1, label);
	}
	dragpix.setMask(dragpix.createHeuristicMask());	

	return dragpix;
}

void FileObj::drawSelRect(QRect selrect, bool mustdraw)
{
	QRect oldrect;
	
	if(mustdraw) {
		mustdrawselrect = TRUE;
		
		oldrect = r1o;
		r1o = QRect(selrect.topLeft(), selrect.topRight());
		if(oldrect.topLeft().y() > 1)
			update(oldrect);
		if(r1o.topLeft().y() > 1)
			update(r1o);
		
		oldrect = r2o;
		r2o = QRect(selrect.bottomLeft(), selrect.bottomRight());
		if(oldrect.topLeft().y() < height() - 1)
			update(oldrect);
		if(r2o.topLeft().y() < height() - 1)
			update(r2o);

		oldrect = r1v;
		r1v = QRect(selrect.topLeft(), selrect.bottomLeft());
		if(oldrect.topLeft().x() > 1)
			update(oldrect);
		if(r1v.topLeft().x() > 1)
			update(r1v);

		oldrect = r2v;
		r2v = QRect(selrect.topRight(), selrect.bottomRight());
		if(oldrect.topLeft().x() < width() - 1)
			update(oldrect);
		if(r2v.topLeft().x() < width() - 1)
			update(r2v);
		
	} else {
		if(mustdrawselrect) {
			mustdrawselrect = FALSE;
			update(r1o);
			update(r2o);
			update(r1v);
			update(r2v);
		}
	}
}

void FileObj::paintEvent(QPaintEvent *)
{
	QPainter p;
	
	p.begin(this);
	p.drawPixmap(0, 0, icona);
	if(mustdrawselrect) {
		if(r1o.topLeft().y() > 1)
			p.drawRect(r1o);
		if(r2o.topLeft().y() < height() - 1)
			p.drawRect(r2o);
		if(r1v.topLeft().x() > 1)
			p.drawRect(r1v);
		if(r2v.topLeft().x() < width() - 1)
			p.drawRect(r2v);
	}
	p.end();
}

bool FileObj::eventFilter(QObject *, QEvent *e)
{
   if(e->type() == Event_MouseButtonPress 
			|| e->type() == Event_MouseButtonDblClick) {
		this->event(e);
		return TRUE;
  }
  return FALSE;
}



