#include "findframe.h"
#include "mfinderapp.h"

FindFrame::FindFrame(QWidget *parent, const char *name)
			  	  : QWidget(parent, name)
{
	QColor x = QColor(230, 230, 230);
	QColorGroup CGnorm = QColorGroup(black, x, x.light(), x.dark(), x.dark(120), black, x);
	setPalette(QPalette(CGnorm, CGnorm, CGnorm));
	sprintf(confdir, "%s/GNUstep/Library/xwfinder", getenv("HOME"));
	mywin = (FindWin *)parent;
	list.setAutoDelete(TRUE);
	isShiftClick = FALSE;
	VObjs = 4;
	show();
}

FindFrame::~FindFrame()
{
	FindObj *obj;

	for(int i = 0; i < list.count(); i++) {
		obj = list.at(0);
		list.removeRef(obj);
	}
}

void FindFrame::addAnObject(char *objpath)
{
	struct stat listStat;
	int type;
	
   lstat(objpath, &listStat);
	if(S_ISREG(listStat.st_mode)) {
		if(listStat.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH))
			type = FILEXE;
		else
			type = FIL;
	}
	if(S_ISDIR(listStat.st_mode)) 
		type = DR;
	if(S_ISLNK(listStat.st_mode)) {
		if(chdir(objpath) != 0) {
			if(listStat.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH)) {
				type = FILEXELNK;
			} else {
				type = FILLNK;
			}
		} else {
			chdir(confdir);
			type = DRLNK;
		}
	}		
	if(S_ISCHR(listStat.st_mode)) 
		type = CHRDV;		
	if(S_ISBLK(listStat.st_mode)) 
		type = BLCKDV;	
	if(S_ISFIFO(listStat.st_mode)) 
		type = FIF;		
	if(S_ISSOCK(listStat.st_mode)) 
		type = SCKT;		

	list.append(new FindObj(this, objpath, type));
}

void FindFrame::selectAllObjects()
{
	int i, count;
	FindObj *obj;

	isShiftClick = TRUE;
	count = list.count();
	for(i = 0; i < count; i++) {
		obj = list.at(i);
		obj->Select();
	}
	isShiftClick = FALSE;
}

void FindFrame::unselectOtherObjects(FindObj *anobj)
{
	int i, count;
	FindObj *currobj;

	if(!isShiftClick) {
		count = list.count();
		for(i = 0; i < count; i++) {
			currobj = list.at(i);
			if(currobj != anobj) {
				if(currobj->isSelect)
					currobj->Unselect();
			}
		}
	}
}

void FindFrame::openSelectedObjects()
{
//	((mfinderapp *)qApp)->openSelectedObjectsOfWin(myfwin, FALSE);		
}

void FindFrame::resizeEvent(QResizeEvent *e)
{
	FindObj *currobj;
	
	VObjs = 4;
	for(int i = 0; i < list.count(); i++)  {
		currobj = list.at(i);
		currobj->move(0, VObjs);
		VObjs = VObjs + currobj->hspace;
	}
	emit resizeDone(VObjs);
}

void FindFrame::mousePressEvent(QMouseEvent *e)
{
	if(e->button() & RightButton) {
	//	globalPopUp->popup(mapToGlobal(e->pos()));
		isShiftClick = FALSE;
	}
	if(e->button() & LeftButton) {
		if(e->state() & ShiftButton)
			isShiftClick = TRUE;
		else 
			isShiftClick = FALSE;
	}
}

void FindFrame::paintEvent(QPaintEvent *)
{
	QPainter p;
	
//	p.begin(this);

//	p.end();
}
