#include "findobj.h"
#include "mfinderapp.h"
#include "functions.h"
#include "findframe.h"

FindObj::FindObj(QWidget *parent, const char *path, int type)
        		:QWidget(parent, path)
{
	QColor color = QColor(230, 230, 230);
	QColorGroup cnorm = QColorGroup(black, color, color.light(), color.dark(), 
																color.dark(120), black, color);
	QColorGroup cinv = QColorGroup(white, black, color.light(), color.dark(), 
															color.dark(120), white, black);
	pnorm = QPalette(cnorm, cnorm, cnorm);
	pinv = QPalette(cinv, cinv, cinv);
	QFont font = QFont("Helvetica", 10);

	strcpy(myFullPath, path);
	strcpy(myName, lastPathComponent(myFullPath));
	myType = type;
	
	leftspace = 10;
	hght = hspace = 20;
	myfindframe = (FindFrame *)parent;
	resize(myfindframe->width(), hght);

	isSelect = FALSE;
	havecustomicon = ((mfinderapp *)qApp)->hasCustomIconFileAtPath(myFullPath);

	label = new QLabel(this, myFullPath);
	label->setAutoResize(TRUE);
	label->setPalette(pnorm);
	label->setFont(font);
	label->setText(myFullPath);
	label->installEventFilter(this); 

	icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
	 														  TRUE, UNSELECT, havecustomicon));
	show();
	label->show();
}

FindObj::~FindObj()
{
	label->~QLabel();
	
// Aggiungere la distruzione dell'icona !!
}

void FindObj::Select()
{
	isSelect = TRUE;
	icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
																 TRUE, SELECT, havecustomicon));
	label->setPalette(pinv);
	update();
	myfindframe->unselectOtherObjects(this);
}

void FindObj::Unselect()
{
	isSelect = FALSE;
	icona = QPixmap(((mfinderapp *)qApp)->iconForFile(myFullPath, myName, myType,
															 TRUE, UNSELECT, havecustomicon));
	label->setPalette(pnorm);
	update();
}

void FindObj::mousePressEvent(QMouseEvent *e)
{
	if(e->state() & ShiftButton) {
		myfindframe->isShiftClick = TRUE;
		if(isSelect)
			Unselect();
		else 
			Select();
	} else {
		myfindframe->isShiftClick = FALSE;
		if(!isSelect)
			Select();
	}
}

void FindObj::resizeEvent(QResizeEvent *e)
{
	resize(myfindframe->width(), hght);
	label->move(leftspace + icona.width() + 5, 0);
}

void FindObj::mouseDoubleClickEvent(QMouseEvent *e)
{
	((mfinderapp *)qApp)->DirOpenFromFindFileWin(myFullPath);
}

void FindObj::paintEvent(QPaintEvent *)
{
	QPainter p;
	
	p.begin(this);
	p.drawPixmap(leftspace, 0, icona);
	p.end();
}

bool FindObj::eventFilter(QObject *, QEvent *e)
{
	if(e->type() == Event_MouseButtonPress 
			|| e->type() == Event_MouseButtonDblClick) {
		this->event(e);
		return TRUE;
	}
  	return FALSE;
}
