#include "functions.h"
#include "common.h"
#include "fileobj.h"
#include "filelistobj.h"

extern char confdir[];
extern char iconsPath[];	
extern char appsPath[];
	 	
char p[MAX_PATH_LEN] = "";

bool dfirst;
bool iview;

typedef struct {
	int uid, gid;
	char *name;
  	char home[MAX_PATH_LEN];
  	char shell[MAX_PATH_LEN];
  	char hostname[128];
  	mode_t umask;
} UserInfo;	

UserInfo user;

int QGList::compareItems(GCI item1, GCI item2)
{
	if(iview) {
		FileObj *f1 = (FileObj *)item1;
		FileObj *f2 = (FileObj *)item2;
		
		if(dfirst) {
			if(f1->isDir && f2->isDir)
				return strcmp(f1->myName, f2->myName);
			if(f1->isDir && !f2->isDir)
				return -1;
			if(!f1->isDir && f2->isDir)
				return 1;
			if(!f1->isDir && !f2->isDir)
				return strcmp(f1->myName, f2->myName);
		} else {
			return strcmp(f1->myName, f2->myName);
		}
	} else {
		FileListObj *f1 = (FileListObj *)item1;
		FileListObj *f2 = (FileListObj *)item2;
		
		if(dfirst) {
			if(f1->isDir && f2->isDir)
				return strcmp(f1->myName, f2->myName);
			if(f1->isDir && !f2->isDir)
				return -1;
			if(!f1->isDir && f2->isDir)
				return 1;
			if(!f1->isDir && !f2->isDir)
				return strcmp(f1->myName, f2->myName);
		} else {
			return strcmp(f1->myName, f2->myName);
		}
	}
}

char *deleteLastPathComponent(char *apath)
{
	int i, pos;

	sprintf(p, "");
	pos = 1;	
	for(i = 1; i < strlen(apath); i++) 
		if(apath[i] == '/')
			pos = i;

	strncpy(p, apath, pos);
	p[pos] = '\0';

	return p;
}

char *lastPathComponent(char *apath)
{
	char buff[MAX_PATH_LEN] = "";
	char *token, *delims = "/";

	sprintf(p, "");

	if(strcmp(apath, "/") == 0)
		return apath;
	strncpy(buff, apath, MAX_PATH_LEN);
	token = strtok(buff, delims);
	sprintf(p, "%s", token);
	while(1) {
		token = strtok(NULL, delims);
		if(token != NULL)
			sprintf(p, "%s", token);
		else
			break;
	}
	return p;
}

bool isPathContainedInPath(char *p1, char *p2)
{
	char buff[MAX_PATH_LEN] = "";
	bool iscontained = FALSE;
	
	sprintf(buff, "%s", p1);
	while(strcmp(buff, "/") != 0) {
		if(strcmp(buff, p2) == 0) {
			iscontained = TRUE;
			break;
		}
		sprintf(buff, "%s", deleteLastPathComponent(buff));
	}
	return iscontained;
}

char *bestMatchingStringInArray(char **argv, int argc, char *str)
{
	int i, count = 0, pos = 0;

	sprintf(p, "");
	for(i = 0; i < argc; i++) { 
		if(strncmp(argv[i], str, strlen(str)) == 0) {
			count++;
			pos = i;
		}
	}
	if(count > 1)
		sprintf(p, "%s", str);
	else
		sprintf(p, "%s", argv[pos]);
	
	return p;
}

char *bestMatchingPath(char *subpath, char *namepart)
{
	struct stat listStat;
	struct dirent **dir;
	int m = 0, pos = 0;

	sprintf(p, "");
	if(!lstat(subpath, &listStat)) {
		m = scandir(subpath, &dir, 0, alphasort);
		for(int i = 0; i < m; i++) {
			if(strcmp(dir[i]->d_name, ".") !=0 && strcmp(dir[i]->d_name, "..") !=0) {
				if(strncmp(dir[i]->d_name, namepart, strlen(namepart)) == 0) {
					pos = i;
					break;
				}
			}
		}
		if(pos) {
			if(strcmp(subpath, "/") == 0)
				sprintf(p, "%s%s", subpath, dir[pos]->d_name);
			else
				sprintf(p, "%s/%s", subpath, dir[pos]->d_name);
		}				
	}
	return p;
}

char *subtractPathComponentToPath(char *apath, char *pathtoken)
{
	int i = 0, j = 0;

	sprintf(p, "");
	if(strcmp(apath, pathtoken) == 0)
		return p;
	while(pathtoken[i] == apath[i])
		i++;
	
	while(apath[i] != '\0') {
		p[j] = apath[i];
		i++;
		j++;
	}
	p[j] = '\0';
	
	return p;
}

char *divideStringInLines(char *fname, int pnts, char *txt, char sep, int maxwidth)
{
	char buff[MAX_PATH_LEN] = "", cutpath[MAX_PATH_LEN] = "";
	QFont font;
	int w = 0, lw = 0, i = 0, j = 0, pos = 0, add = 0;

	sprintf(p, "");	
	font = QFont(fname, pnts);
	QFontMetrics fm = QFontMetrics(font);
	w = fm.width(txt, strlen(txt));
	if(w > maxwidth) {
		i = 0;
		while(i < strlen(txt)) {
			j = 0;
			lw = 0;
			sprintf(buff, "");
			while(lw < maxwidth) {
				if(i == strlen(txt)) {
					pos = i;
					break;
				}
				buff[j] = txt[i];
				buff[j+1] = '\0';
				lw = fm.width(buff, strlen(buff));
				if(buff[j] == sep) 
					pos = j;
				i++;
				j++;
			}
			strncpy(cutpath, buff, pos+1);
			cutpath[pos+1] = '\0';	
			sprintf(p, "%s\n%s", p, cutpath);
			add++;
			i = strlen(p)-add;
		}
	} else {
		strncpy(p, txt, MAX_PATH_LEN);
	}
	return p;
}

bool isStringInString(char *str1, char *str2)
{
	char *s = NULL;
	
	s = strstr(str1, str2);
	if(s)
		return TRUE;

	return FALSE;
}

bool isPlainFileAtPath(char *apath)
{
	struct stat listStat;

	lstat(apath, &listStat);
	if(S_ISREG(listStat.st_mode)) {
		if(listStat.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH))
			return FALSE;
		else
			return TRUE;
	}
	return FALSE;
}

void loadAndTuneIcon(char *ipath, char *key)
{
	QImage icn, smicn, pixmask;
	QPixmap icona, smicona;
	QBitmap altmask;
	char buff[128] = "";
	bool domask;

	icn = QImage(ipath, "XPM");
	icn = icn.convertDepth(32);
	domask = FALSE;
	for(int h = 0; h < icn.width(); h++) {
		QRgb pix = icn.pixel(h, 0);
		if(qRed(pix) > 5 && qGreen(pix) > 5 && qBlue(pix) > 5)
			domask = TRUE;
	}
	icn = icn.convertDepth(8);
	smicn = icn.smoothScale(16, 16);
	icona = QPixmap();
	smicona = QPixmap();
	icona.convertFromImage(icn);
	smicona.convertFromImage(smicn);
	if(domask) {
		icona.setMask(icona.createHeuristicMask());
		smicona.setMask(smicona.createHeuristicMask());
	} else {
		altmask = QBitmap();
		altmask.resize(icona.width(), icona.height());
		altmask.fill(color1);	
		icona.setMask(altmask);
		altmask.resize(smicona.width(), smicona.height());
		altmask.fill(color1);	
		smicona.setMask(altmask);
	}
	sprintf(buff, "%sUNSLBG", key);	
	QPixmapCache::insert(buff, new QPixmap(icona));
	sprintf(buff, "%sUNSLSM", key);	
	QPixmapCache::insert(buff, new QPixmap(smicona));

	icn = icn.convertDepth(32);
	pixmask = icn.createHeuristicMask();
	for(int h = 0; h < icn.width(); h++) {
		for(int j = 0; j < icn.height(); j++) {
			QRgb pix = icn.pixel(h, j);
			if(qRed(pixmask.pixel(h, j)) == 0) {
				int red = qRed(pix);
				int green = qGreen(pix);		
				int blue = qBlue(pix);
				if(red > 0) red = (int)(red/2);
				if(green > 0) green = (int)(green/2);
				if(blue > 0) blue = (int)(blue/2);
				icn.setPixel(h, j, qRgb(red, green, blue));
			}
		}
	}
	icn = icn.convertDepth(8);
	smicn = icn.smoothScale(16, 16);
	icona.convertFromImage(icn);
	smicona.convertFromImage(smicn);
	if(domask) {
		icona.setMask(icona.createHeuristicMask());
		smicona.setMask(smicona.createHeuristicMask());
	}
	sprintf(buff, "%sSELBG", key);	
	QPixmapCache::insert(buff, new QPixmap(icona));
	sprintf(buff, "%sSELSM", key);	
	QPixmapCache::insert(buff, new QPixmap(smicona));
}

void loadAndTuneFolderIcon(char *ipath, char *key)
{
	QImage icn, smicn, icncopy, pixmask;
	QPixmap icona, smicona;
	char buff[128] = "";
	
	icn = QImage(ipath, "XPM");
	smicn = icn.smoothScale(16, 16);
	icona = QPixmap();
	smicona = QPixmap();	
	icona.convertFromImage(icn);
	smicona.convertFromImage(smicn);
	icona.setMask(icona.createHeuristicMask());
	smicona.setMask(smicona.createHeuristicMask());
	
	sprintf(buff, "%sUNSLBG", key);	
	QPixmapCache::insert(buff, new QPixmap(icona));
	sprintf(buff, "%sUNSLSM", key);	
	QPixmapCache::insert(buff, new QPixmap(smicona));
	
	icncopy = icn.copy();
	icncopy = icncopy.convertDepth(32);
	pixmask = icona.createHeuristicMask();
	for(int h = 0; h < icncopy.width(); h++) {
		for(int j = 0; j < icncopy.height(); j++) {
			QRgb pix = icncopy.pixel(h, j);
			if(qRed(pixmask.pixel(h, j)) == 0) {
				int red = qRed(pix);
				int green = qGreen(pix);		
				int blue = qBlue(pix);
				if(red > 0) red = (int)(red/2);
				if(green > 0) green = (int)(green/2);
				if(blue > 0) blue = (int)(blue/2);
				icncopy.setPixel(h, j, qRgb(red, green, blue));
			}
		}
	}
	icncopy = icncopy.convertDepth(8);
	smicn = icncopy.smoothScale(16, 16);
	icona.convertFromImage(icncopy);
	smicona.convertFromImage(smicn);
	icona.setMask(icona.createHeuristicMask());
	smicona.setMask(smicona.createHeuristicMask());
		
	sprintf(buff, "%sSELBG", key);	
	QPixmapCache::insert(buff, new QPixmap(icona));
	sprintf(buff, "%sSELSM", key);	
	QPixmapCache::insert(buff, new QPixmap(smicona));

	icncopy = icn.copy();
	icncopy = icncopy.convertDepth(32);
	for(int h = 0; h < icncopy.width(); h++) {
		for(int j = 0; j < icncopy.height(); j++) {
			QRgb pix = icncopy.pixel(h, j);
			if(qRed(pixmask.pixel(h, j)) == 0) {
				icncopy.setPixel(h, j, qRgb(150, 150, 190));
			}
		}
	}	
	icncopy = icncopy.convertDepth(8);
	smicn = icncopy.smoothScale(16, 16);
	icona.convertFromImage(icncopy);
	smicona.convertFromImage(smicn);
	icona.setMask(icona.createHeuristicMask());
	smicona.setMask(smicona.createHeuristicMask());
		
	sprintf(buff, "%sOPNBG", key);	
	QPixmapCache::insert(buff, new QPixmap(icona));
	sprintf(buff, "%sOPNSM", key);	
	QPixmapCache::insert(buff, new QPixmap(smicona));

	icncopy = icn.copy();
	icncopy = icncopy.convertDepth(32);
	for(int h = 0; h < icncopy.width(); h++) {
		for(int j = 0; j < icncopy.height(); j++) {
			QRgb pix = icncopy.pixel(h, j);
			if(qRed(pixmask.pixel(h, j)) == 0) {
				icncopy.setPixel(h, j, qRgb(80, 80, 110));
			}
		}
	}	
	icncopy = icncopy.convertDepth(8);
	smicn = icncopy.smoothScale(16, 16);
	icona.convertFromImage(icncopy);
	smicona.convertFromImage(smicn);
	icona.setMask(icona.createHeuristicMask());
	smicona.setMask(smicona.createHeuristicMask());
		
	sprintf(buff, "%sSOPNBG", key);	
	QPixmapCache::insert(buff, new QPixmap(icona));
	sprintf(buff, "%sSOPNSM", key);	
	QPixmapCache::insert(buff, new QPixmap(smicona));	
}


int compare(const void *a, const void *b)
{
	return strcmp(*(char **)a, *(char **)b);
}

int create(char *path, mode_t mode)
{
 	int file = open(path, O_WRONLY|O_CREAT|O_EXCL, mode);
 	if (file == -1 || close(file)) return -1;
 	return 0;
}

int rdel(char *path, bool isdir)
{
	struct dirent **namelist;
	struct stat listStat;
	char *path2;
	int m, a, i, res;
  
	if(stat(path, &listStat) != 0)
		return 0;  					// se non esiste piu' non da errore...
  
  	path2 = (char*)malloc(512 * sizeof(char));
  
	res = 0;
	if(!isdir) {
		free(path2);
		return res = unlink(path);
	}
	m = scandir(path, &namelist, 0, alphasort);
	if(m == 2) {
		free(path2);
		return res = rmdir(path);
	}
	for(a = 0; a < m; a++) {
		if(strcmp(namelist[a]->d_name, ".") != 0 && strcmp(namelist[a]->d_name, "..") != 0) {
			sprintf(path2, "%s/%s", path, namelist[a]->d_name);
			i = lstat(path2, &listStat);
			if(!S_ISDIR(listStat.st_mode)) {
				res = unlink(path2);
			} else {
				res = rdel(path2, TRUE);
				res = rmdir(path2);
			} 
		}
	}
	free(path2);
	return res = rmdir(path);
}

int copyfile(char *oldpath, char *newpath) 
{
	struct stat stats;
	int src = -1, dest = -1, n;
	char *buf;

  	buf = (char*)malloc(BUFSIZ * sizeof(char));

	if((src = open(oldpath, O_RDONLY)) == -1 || stat(oldpath, &stats)) {
		free(buf);
		return -1;
	}
	if((dest = creat(newpath, stats.st_mode)) == -1) {
		free(buf);
		return -1;
	}
  	while((n = read(src, buf, BUFSIZ)) != 0) {
    	if(n == -1 || write(dest, buf, n) != n) {
    		free(buf);
			return -1;
		}
	}

  	if(close(src)) {
    	src = -1;
    	free(buf);
		return -1;
  	} else {
  		free(buf);
    	return close(dest);
   }
}

int copydir(char *oldpath, char *newpath) 
{
	struct dirent **namelist;
	struct stat listStat;
	char *oldpath2;
	char *newpath2;
	int m, a, ls, res;
  	user.umask = umask(0);
  	umask(user.umask);
  	user.umask = 0777777 ^ user.umask;
  
  	oldpath2 = (char*)malloc(512 * sizeof(char));
  	newpath2 = (char*)malloc(512 * sizeof(char));

	res = 0;
	res = mkdir(newpath, user.umask & 0777);
	if(res != 0) {
		free(oldpath2);
		free(newpath2);
		return res;
	}
	m = scandir(oldpath, &namelist, 0, alphasort);
	for(a = 0; a < m; a++) {
		if(strcmp(namelist[a]->d_name, ".") != 0 && strcmp(namelist[a]->d_name, "..") != 0) {
			if(strcmp(oldpath, "/") == 0) {
				sprintf(oldpath2, "%s%s", oldpath, namelist[a]->d_name);
			} else {
				sprintf(oldpath2, "%s/%s", oldpath, namelist[a]->d_name);
			}
			if(strcmp(newpath, "/") == 0) {
				sprintf(newpath2, "%s%s", newpath, namelist[a]->d_name);
			} else {
				sprintf(newpath2, "%s/%s", newpath, namelist[a]->d_name);
			}

			ls = lstat(oldpath2, &listStat);

			if(S_ISREG(listStat.st_mode))
				res = copyfile(oldpath2, newpath2);
			else if(S_ISDIR(listStat.st_mode))
				res = copydir(oldpath2, newpath2);
			else if(S_ISFIFO(listStat.st_mode))
	  			res = mkfifo(newpath2, user.umask & 0666);
			else if(S_ISBLK(listStat.st_mode) || S_ISCHR(listStat.st_mode) || S_ISSOCK(listStat.st_mode))
	  			res = mknod(newpath2, user.umask & 0666, listStat.st_rdev);
			else if(S_ISLNK(listStat.st_mode)) {
	  			char lnk[MAXPATHLEN+1];
	  			int l = readlink(oldpath2, lnk, MAXPATHLEN);
	  			if(l<0) res = 1;
	  			else {
	      		lnk[l] = '\0';
	      		res = symlink(lnk, newpath2);
	  			}
      	}
		}
	}
	free(oldpath2);
	free(newpath2);
	return res;
}

int catchXError(Display *dpy, XErrorEvent *error)
{
	printf("XError!\n");

	return 0;
}
