#include "infoframe.h"
#include "mfinderapp.h"

InfoFrame::InfoFrame(QWidget *parent, const char *name)
			  : QFrame(parent, name)
{
	QColor x = QColor(160, 164, 160);
	QColorGroup CGnorm = QColorGroup(black, x, x.light(), x.dark(), x.dark(120), black, x);
	setPalette(QPalette(CGnorm, CGnorm, CGnorm));
	setFrameStyle(QFrame::Panel | QFrame::Raised);
	font = QFont("Helvetica", 10);
 	strncpy(path, name, MAX_PATH_LEN);
	myfwin = (MFinderWin *)parent;
	labInf = new QLabel(this, "labInf");
	labInf->installEventFilter(this); 
	labInf->setPalette(QPalette(CGnorm, CGnorm, CGnorm));
	labInf->setFont(font);
	labInf->setAutoResize(TRUE);
	closemywin = FALSE;
	makePopUp();
}

void InfoFrame::updateData()
{
	struct dirent **namelist;
	int m;
	char buf[256] = "";
 	long bsize, ksize, free, free_blocks;
	struct statfs statf;
	
	m = scandir(path, &namelist, 0, alphasort);
	sprintf(buf, "%i items", m - 2);

 	if(!statfs(path, &statf)) {
  		bsize = (long)statf.f_bsize / 1024L;
  		if(!bsize)
  			ksize = 1024L / (long)statf.f_bsize;

 		free_blocks = statf.f_bfree;
 		if(bsize)
 			free = free_blocks * bsize;
 		else
 			free = free_blocks / ksize;

 		sprintf(buf, "%s  -  %ld KBytes available", buf, free);
 	}

	labInf->setText(buf);
	labInf->update();
}

void InfoFrame::makePopUp()
{
	char buff[MAX_PATH_LEN] = "";
	char apath[MAX_PATH_LEN] = "";
	QList<char> pathslist; 
	char *token, *delims = "/";
	int count = 0;

	pathslist.setAutoDelete(TRUE);
	pathslist.append(new char[8]);
	strcpy(pathslist.current(), "/");
	strncpy(buff, path, MAX_PATH_LEN);
	token = strtok(buff, delims);
	if(token != NULL) {
		pathslist.append(new char[strlen(token)+1]);
		strcpy(pathslist.current(), token);
	}
	while(token != NULL) {
		token = strtok(NULL, delims);
		if(token != NULL) {
			pathslist.append(new char[strlen(token)+1]);
			strcpy(pathslist.current(), token);
		}
	}

	popupMenu = new Popup();
	connect(popupMenu, SIGNAL(activated(int)), this, SLOT(OpenOtherWins(int)));
	for(int i = pathslist.count()-1; i >= 0; i--) 
		popupMenu->insertItem(pathslist.at(i));
	
	pathslist.clear();
}

void InfoFrame::OpenOtherWins(int menuid)
{
	char winpath[MAX_PATH_LEN] = "";
	
	for(int i = popupMenu->count()-1; i >= menuid; i--) {
		if(i == popupMenu->count()-1 || i == menuid)
			sprintf(winpath, "%s%s", winpath, popupMenu->text(i));
		else
			sprintf(winpath, "%s%s/", winpath, popupMenu->text(i));
	}
	if(strcmp(winpath, path) != 0) {
		((mfinderapp *)qApp)->DirOpen(winpath);
		if(closemywin)
			myfwin->close();
	} 	
}

void InfoFrame::resizeEvent(QResizeEvent *)
{
	labInf->move(21, 3);
}

void InfoFrame::mousePressEvent(QMouseEvent *e)
{
	if(e->state() & ControlButton)
		closemywin = TRUE;
	else
		closemywin = FALSE;

	if(strcmp(path, "/") != 0)
		popupMenu->popup(mapToGlobal(e->pos()));
}

bool InfoFrame::eventFilter(QObject *, QEvent *e)
{
   if(e->type() == Event_MouseButtonPress) {
		this->event(e);
		return TRUE;
  }
  return FALSE;
}
