#include "mfinderapp.h"
#include "functions.h"

#define MAX_PROPERTY_SIZE 32*1024

#define ASKTOCONNECT 0
#define NOTIFICATIONREQ 1
#define SERVERREPLY 2
#define OPENAPP 3
#define OPENFILE 4
#define OPENFILEWITHAPP 5
#define STARTXTERM 6
#define FILEOPS 7

const char reqIcnsProvv[] =
"plainfile|Files.xpm\n"
"plainfilelink|FilesLink.xpm\n"
"execfile|Appl.xpm\n"
"execfilelink|ApplLink.xpm\n"
"directory|Folder.xpm\n"
"directorylink|FolderLink.xpm\n";

const char icnsProvv[] =
".*|HAL-Eye.xpm\n"
"Makefile*|RayTracedGear.xpm\n"
"makefile*|RayTracedGear.xpm\n"
"Imakefile*|RayTracedGear.xpm\n"
"README*|Info.xpm\n"
"INSTALL*|Info.xpm\n"
"AUTHORS*|Info.xpm\n"
"NEWS*|Info.xpm\n"
"COPYING*|Info.xpm\n"
"*c|MagDotC.xpm\n"
"*cc|MagDotCC.xpm\n"
"*cpp|MagDotCC.xpm\n"
"*cxx|MagDotCC.xpm\n"
"*C|MagDotCC.xpm\n"
"*cp|MagDotCC.xpm\n"
"*m|FileDotM.xpm\n"
"*h|MagDotH.xpm\n"
"*o|FileDotO.xpm\n"
"*html|FileURL.xpm\n"
"*htm|FileURL.xpm\n"
"*gif|ImageGIF.xpm\n"
"*GIF|ImageGIF.xpm\n"
"*tiff|ImageTIFF.xpm\n"
"*tif|ImageTIFF.xpm\n"
"*TIFF|ImageTIFF.xpm\n"
"*TIF|ImageTIFF.xpm\n"
"*jpeg|ImageJPG.xpm\n"
"*jpg|ImageJPG.xpm\n"
"*JPEG|ImageJPG.xpm\n"
"*JPG|ImageJPG.xpm\n"
"*xpm|ImageXPM.xpm\n"
"*xbm|ImageXBM.xpm\n"
"*ps|FilePS.xpm\n"
"*gz|FileTgz.xpm\n"
"*z|FileZ.xpm\n"
"*zip|FileZip.xpm\n"
"*tar|FileTar.xpm\n";

char confdir[MAX_PATH_LEN];
char iconsPath[MAX_PATH_LEN];		 	
char appsPath[MAX_PATH_LEN];		 	

bool dropopenapp;

mfinderapp::mfinderapp(int &argc, char** argv, const char *cln, 
		      									 const unsigned char* icondata)
   		  :QApplication(argc, argv)
{
	struct stat st;
	struct dirent **dir;
	int m;	
	FILE *fConf;
	char fPath[MAX_PATH_LEN] = "";
	char buff[MAX_PATH_LEN] = "", buf2[256] = "";
	char icnsrcpath[MAX_PATH_LEN] = "", icndestpath[MAX_PATH_LEN] = "";
	char *delims = "|", *delims2 = ":";
	char *token, *tok1, *tok2;
	char line[1024] = "";	
	int count = 0;
	int numlines, bpos;
	int len = 0;
	QPixmap pix;
	QPainter p;
	bool firsttime = FALSE, isdiricon = FALSE;
	WId win;
   XWMHints *hints;
   XClassHint classhint;
	const char version[] = "xwfinder-0.1";
	
	dspl = desktop()->x11Display();
	appScreen = DefaultScreen(dspl);
	appRootWin = RootWindow(dspl, appScreen);	
	
	DndProtocol = XInternAtom(dspl, "DndProtocol", FALSE);
	DndSelection = XInternAtom(dspl, "DndSelection", FALSE);

	XSetErrorHandler((XErrorHandler)catchXError);

	setStyle(WindowsStyle);
	setWinStyleHighlightColor(QColor(80, 80, 80));
	font = QFont("Helvetica", 10);
	setFont(font);
	QPixmapCache::setCacheLimit(2048);
	isdragcopying = FALSE;
	deskobjmoving = FALSE;
	xdnddone = TRUE;	
	isfinding = FALSE;
	xappnum = 0;
	
	winlist.setAutoDelete(TRUE);
	deskobjlist.setAutoDelete(TRUE);
	wGeomDict.setAutoDelete(TRUE);
	wOpenDict.setAutoDelete(TRUE);
	deskObjsDict.setAutoDelete(TRUE);
	icnReqDict.setAutoDelete(TRUE);
	icnDict.setAutoDelete(TRUE);
	filesDict.setAutoDelete(TRUE);
		
	user.uid = getuid();
  	user.gid = getgid();
  	user.name = getenv("USER");
	strcpy(user.home, getenv("HOME"));
	sprintf(confdir, "%s/GNUstep/Library/xwfinder", user.home);
	strcpy(user.shell, "/bin/sh");
	if(gethostname(user.hostname, 128 - 1) == EINVAL)
		strcpy(user.hostname, "localhost");
  	user.umask = umask(0);
  	umask(user.umask);
  	user.umask = 0777777 ^ user.umask;
	sprintf(iconsPath, "%s/Pixmaps", confdir);
	sprintf(appsPath, "%s/Apps", confdir);

	timer = new QTimer(0L);
	connect(timer, SIGNAL(timeout()), SLOT(processPendingData()));

// -------------------------
// --- THE FIRST TIME... ---
// -------------------------	
	firsttime = FALSE;
	sprintf(fPath, "%s/GNUstep", getenv("HOME"));
	if(stat(fPath, &st)) {
		firsttime = TRUE;
		if(mkdir(fPath, user.umask & 0777)) {
			sprintf(buff, "Error in creating \"%s\"", fPath);
			fileopsErr("Error", buff);
			exit(1);
		}
	}
	sprintf(fPath, "%s/GNUstep/Library", getenv("HOME"));
	if(stat(fPath, &st)) {
		firsttime = TRUE;
		if(mkdir(fPath, user.umask & 0777)) {
			sprintf(buff, "Error in creating \"%s\"", fPath);
			fileopsErr("Error", buff);
			exit(1);
		}
	}
	sprintf(fPath, "%s/GNUstep/Library/xwfinder", getenv("HOME"));
	if(stat(fPath, &st)) {
		firsttime = TRUE;
		if(mkdir(fPath, user.umask & 0777)) {
			sprintf(buff, "Error in creating \"%s\"", fPath);
			fileopsErr("Error", buff);
			exit(1);
		}
	}
	sprintf(fPath, "%s/GNUstep/Library/xwfinder/Pixmaps", getenv("HOME"));
	if(stat(fPath, &st)) {
		firsttime = TRUE;
		if(mkdir(fPath, user.umask & 0777)) {
			sprintf(buff, "Error in creating \"%s\"", fPath);
			fileopsErr("Error", buff);
			exit(1);
		}
	}

	sprintf(fPath, "%s/GNUstep/Library/xwfinder/version", getenv("HOME"));
	if(stat(fPath, &st)) {
		firsttime = TRUE;
	} else {
		fConf = fopen(fPath, "r");
		fscanf(fConf, "%s", buff);
		fclose(fConf);
		if(strcmp(version, buff) != 0)
			firsttime = TRUE;
	}
	
	chdir(confdir);
	
	if(firsttime) {
		strscrnft = new StrtScrn(0L, "ss", "/usr/local/share/xwfinder/Pixmaps/StrScr.xpm");
		sprintf(strscrnft->expl1, "copying icons in your user icons directory...");
		strscrnft->show();

		sprintf(fPath, "/usr/local/share/xwfinder/Pixmaps");
		m = scandir(fPath, &dir, 0, alphasort);
		for(int i = 0; i < m; i++) {
			if(strcmp(dir[i]->d_name, ".") !=0 && strcmp(dir[i]->d_name, "..") !=0) {
				sprintf(icnsrcpath, "%s/%s", fPath, dir[i]->d_name);
				sprintf(icndestpath, "%s/%s", iconsPath, dir[i]->d_name);
				if(copyfile(icnsrcpath, icndestpath)) { 
					fileopsErr("Error", "Error in copying your user icons!");
					exit(1);
				}
			}
			strscrnft->update();
			sprintf(strscrnft->expl2, "%s", dir[i]->d_name);
			p.begin(strscrnft);
			p.setFont(font);
			p.drawText(35, 172, strscrnft->expl1);
			p.drawText(35, 192, strscrnft->expl2);
			p.end();
		}

		sprintf(fPath, "%s/GNUstep/Library/xwfinder/version", getenv("HOME"));
		fConf = fopen(fPath, "w");
		fprintf(fConf, "%s", version);
		fclose(fConf);

		strscrnft->close();
	}
	
// ---------------------
// --- USER DEFAULTS ---
// ---------------------	
	numlines = 0;
	strcpy(fPath, "winsgeom");
	if(stat(fPath, &st)) {
		fConf = fopen(fPath, "w");
		fprintf(fConf, "%s:150|30|400|300|0|1|0", getenv("HOME"));
		fclose(fConf);
	}
	fConf = fopen(fPath, "r");
	while(fgets(line, 1024, fConf) != NULL)
		numlines++;
	fclose(fConf);		
	
	strcpy(fPath, "objsprops");
	if(stat(fPath, &st)) {
		fConf = fopen(fPath, "w");
		fprintf(fConf, "/:0|0|-|0|-\n");
		fclose(fConf);
	}
	fConf = fopen(fPath, "r");
	while(fgets(line, 1024, fConf) != NULL)
		numlines++;
	fclose(fConf);		
	
	strcpy(fPath, "reqicons");
	if(stat(fPath, &st)) {
		fConf = fopen(fPath, "w");
		fprintf(fConf, "%s", reqIcnsProvv);
		fclose(fConf);
	}
	fConf = fopen(fPath, "r");
	while(fgets(line, 1024, fConf) != NULL)
		numlines++;
	fclose(fConf);		

	strcpy(fPath, "usricons");
	if(stat(fPath, &st)) {
		fConf = fopen(fPath, "w");
		fprintf(fConf, "%s", icnsProvv);
		fclose(fConf);
	}
	fConf = fopen(fPath, "r");
	while(fgets(line, 1024, fConf) != NULL)
		numlines++;
	fclose(fConf);		

	sprintf(buff, "%s/StrScr.xpm", iconsPath);
	strscrn = new StrtScrn(0L, "ss", buff);
	strscrn->show();

	win = strscrn->winId();
	
   appClassName = (char*)(cln == NULL ? argv[0] : cln);
	
	if(icondata != NULL) {
  		icon = new QPixmap();
      icon->loadFromData(icondata, strlen((const char*)icondata), "XPM");
      if(icon->isNull()) {
	 		delete icon;
	 		icon = NULL;
      } else {
			strscrn->setIcon(*icon);
		}
   } else { 
		icon = NULL; 
	}

   classhint.res_name = (char*)name();
   classhint.res_class = appClassName;
   XSetClassHint(dspl, win, &classhint);
		
   hints = XGetWMHints(dspl, win);
   hints->window_group = win;
   hints->flags |= WindowGroupHint; 
   XSetWMHints(dspl, win, hints);
   XSetCommand(dspl, win, argv, argc);
	
	strscrn->pbar->setTotalSteps(numlines);
	strscrn->pbar->setProgress(0);
	bpos = 0;

// --- WINDOWS GEOMETRY ---
	sprintf(strscrn->expl2, "reading windows geometry...");
	p.begin(strscrn);
	p.setFont(font);
	p.drawText(35, 192, strscrn->expl2);
	p.end();
	strcpy(fPath, "winsgeom");
	fConf = fopen(fPath, "r");
	while(fgets(line, 1024, fConf) != NULL) {
		int n = cstrlen(line);
		if(line[n-1] == '\n')
			line[n-1] = 0;
		tok1 = strtok(line, delims2);
		tok2 = strtok(NULL, delims2);
		if(!stat(tok1, &st)) {
			wGeomDict.append(new dictEntry());
			dictEntry *de = wGeomDict.current();
			strcpy(de->key, tok1);
			strcpy(de->value, tok2);
		}
		bpos++;
		strscrn->pbar->setProgress(bpos);
	}
	fclose(fConf);		
	strscrn->update();

// --- OBJECTS CUSTOM DATA ---
	sprintf(strscrn->expl2, "reading objects meta-data...");
	p.begin(strscrn);
	p.setFont(font);
	p.drawText(35, 192, strscrn->expl2);
	p.end();
	strcpy(fPath, "objsprops");
	fConf = fopen(fPath, "r");
	while(fgets(line, 1024, fConf) != NULL) {
		int n = cstrlen(line);
		if(line[n-1] == '\n')
			line[n-1] = 0;
		tok1 = strtok(line, delims2);
		tok2 = strtok(NULL, delims2);
		if(!stat(tok1, &st)) {
			filesDict.append(new fileMetaData());
			fileMetaData *fmd = filesDict.current();
			strcpy(fmd->filepath, tok1);
			sprintf(buff, "%s", tok2);
			token = strtok(buff, delims);
			fmd->isopenlocal = atoi(token);
			token = strtok(NULL, delims);
			fmd->hascustomicn = atoi(token);
			token = strtok(NULL, delims);
			strcpy(fmd->custicnname, token);
			token = strtok(NULL, delims);
			fmd->hascustomedit = atoi(token);
			token = strtok(NULL, delims);
			strcpy(fmd->custedit, token);
		}
		bpos++;
		strscrn->pbar->setProgress(bpos);
	}
	fclose(fConf);		
	strscrn->update();

// --- ICONS ---
// --- required icons --
	strcpy(fPath, "reqicons");
	fConf = fopen(fPath, "r");
	while(fgets(line, 1024, fConf) != NULL) {
		int n = cstrlen(line);
		if(line[n-1] == '\n')
			line[n-1] = 0;
		tok1 = strtok(line, delims);
		tok2 = strtok(NULL, delims);
		icnReqDict.append(new dictEntry());
		dictEntry *de = icnReqDict.current();
		strcpy(de->key, tok1);
		strcpy(de->value, tok2);
	}
	fclose(fConf);		

	sprintf(strscrn->expl1, "reading icons...");
	count = icnReqDict.count();
	for(int i = 0; i < count; i++) {
		dictEntry *de = icnReqDict.at(i);
		if(strcmp(de->key, "directory") == 0 
								|| strcmp(de->key, "directorylink") == 0)
			isdiricon = TRUE;
		else
			isdiricon = FALSE;
		sprintf(buff, "%s/%s", iconsPath, de->value);
		if(stat(buff, &st)) {
			sprintf(ERR, "The required icon for the file type \"%s\"\n"
                     "is missing in your courrent icons path:\n\"%s\"\n"
                     "You must choose it now from this list !\n",
                     de->key, iconsPath);
			chanicndlg = new ChooseAnIcnDlg(0L, 0L, iconsPath, buf2, ERR, FALSE);
         if(chanicndlg->exec()) {
				sprintf(buff, "%s/%s", iconsPath, chanicndlg->namIcnEd->text());
				sprintf(de->value, "%s", chanicndlg->namIcnEd->text());
			}
		} 
		if(isdiricon)
			loadAndTuneFolderIcon(buff, de->key);
		else
			loadAndTuneIcon(buff, de->key);
			
		strscrn->update();
		sprintf(strscrn->expl2, "%s", de->value);
		p.begin(strscrn);
		p.setFont(font);
		p.drawText(35, 172, strscrn->expl1);
		p.drawText(35, 192, strscrn->expl2);
		p.end();
		bpos++;
		strscrn->pbar->setProgress(bpos);
	}
	
// --- user icons --
	strcpy(fPath, "usricons");
	fConf = fopen(fPath, "r");
	while(fgets(line, 1024, fConf) != NULL) {
		int n = cstrlen(line);
		if(line[n-1] == '\n')
			line[n-1] = 0;
		tok1 = strtok(line, delims);
		tok2 = strtok(NULL, delims);
		icnDict.append(new dictEntry());
		dictEntry *de = icnDict.current();
		strcpy(de->key, tok1);
		strcpy(de->value, tok2);
	}
	fclose(fConf);		

	count = icnDict.count();
	for(int i = 0; i < count; i++) {
		dictEntry *de = icnDict.at(i);
		sprintf(buff, "%s/%s", iconsPath, de->value);
		if(stat(buff, &st)) {
			sprintf(ERR, "The icon for the file type: \"%s\"\n"
                     "is missing in your courrent icons path:\n\"%s\"\n"
                     "You can choose it from this list now or,\n"
                     "later, from the Special->Configure->Icons Menu\n",
                     de->key, iconsPath);
			chanicndlg = new ChooseAnIcnDlg(0L, 0L, iconsPath, buf2, ERR, TRUE);
         if(chanicndlg->exec()) {
				sprintf(buff, "%s/%s", iconsPath, chanicndlg->namIcnEd->text());
				sprintf(de->value, "%s", chanicndlg->namIcnEd->text());
			}
		} 
		loadAndTuneIcon(buff, de->key);
		
		strscrn->update();
		sprintf(strscrn->expl2, "%s", de->value);
		p.begin(strscrn);
		p.setFont(font);
		p.drawText(35, 172, strscrn->expl1);
		p.drawText(35, 192, strscrn->expl2);
		p.end();
		bpos++;
		strscrn->pbar->setProgress(bpos);
	}
		strscrn->pbar->setProgress(numlines);
		strscrn->update();

// --- custom icons --
	count = filesDict.count();
	for(int i = 0; i < count; i++) {
		fileMetaData *fmd = filesDict.at(i);
		if(fmd->hascustomicn) {
			sprintf(buff, "%s/%s", iconsPath, fmd->custicnname);
			if(stat(buff, &st)) {
				sprintf(ERR, "The custom icon for the file:\n \"%s\"\n"
                     	"is missing in your courrent icons path:\n\"%s\"\n"
                     	"You must choose it from this list now!\n",
                     	fmd->filepath, iconsPath);
				chanicndlg = new ChooseAnIcnDlg(0L, 0L, iconsPath, buf2, ERR, TRUE);
         	if(chanicndlg->exec()) {
					sprintf(buff, "%s/%s", iconsPath, chanicndlg->namIcnEd->text());
					sprintf(fmd->custicnname, "%s", chanicndlg->namIcnEd->text());
				}
			}
			loadAndTuneIcon(buff, fmd->filepath);
			
			strscrn->update();
			sprintf(strscrn->expl2, "%s", fmd->custicnname);
			p.begin(strscrn);
			p.setFont(font);
			p.drawText(35, 172, strscrn->expl1);
			p.drawText(35, 192, strscrn->expl2);
			p.end();
		}
	}

	strscrn->hide();

// ------------- INITIALIZING --------------	
// ----- DESK OBJECTS -------
	strcpy(fPath, "deskobjs");
	if(!stat(fPath, &st)) {
		fConf = fopen(fPath, "r");
		while(fgets(line, 1024, fConf) != NULL) {
			int n = cstrlen(line);
			if(line[n-1] == '\n')
				line[n-1] = 0;
			sprintf(buff, "%s", line);	
			tok1 = strtok(buff, delims);
			if(!stat(tok1, &st))
				makeDeskObjFromDictEntry(line);
		}
	}
	
// ----- WINDOWS -------
	strcpy(fPath, "openwins");
	if(stat(fPath, &st)) {
		fConf = fopen(fPath, "w");
		fprintf(fConf, "%s\n", getenv("HOME"));
		fclose(fConf);
	}
	fConf = fopen(fPath, "r");
	while(fgets(line, 1024, fConf) != NULL) {
		int n = cstrlen(line);
		if(line[n-1] == '\n')
			line[n-1] = 0;
		if(!stat(line, &st)) {
			wOpenDict.append(new char[MAX_PATH_LEN]);
			strcpy(wOpenDict.current(), line);
		}
	}
	fclose(fConf);		
	for(int i = 0; i < wOpenDict.count(); i++)
		DirOpen(wOpenDict.at(i));

	timer->start(3000, FALSE);
	
// ---------------- libXWorkspace -----------------	
	startXWSConnection(strscrn->winId());
	verifyXWSConnection();
	
}

void mfinderapp::DirOpen(char *path)
{
	MFinderWin *FWin;
	Window win;
	XWindowAttributes attribs;
	int i, count;
	bool isopen;
	QRect rect;
	char rectbuf[64] = "";
	char *rwin;
	char *delims = "|";
	char *token;
	int x, y, w, h;
	bool dirfirst, dotfiles;
	int viewtype;
	
	count = winlist.count();
	isopen = FALSE;
	for(i = 0; i < count; i++) {
		FWin = winlist.at(i);
		if(strcmp(FWin->myPath, path) == 0) {
			if(XGetWindowAttributes(dspl, FWin->winId(), &attribs)) {
				if(attribs.map_state == IsUnmapped)
		 	   	XMapWindow(dspl, FWin->winId());
			}
			FWin->raise();
			FWin->setActiveWindow();
			isopen = TRUE;
			selectFileObjectFatherOfWindow(path, FALSE);
			break;
		}
	}
	if(!isopen) {
		rwin = rectOfWinAtPath(path);
		if(rwin) {
			sprintf(rectbuf, "%s", rwin);
			token = strtok(rectbuf, delims);
			x = atoi(token);
			token = strtok(NULL, delims);
			y = atoi(token);
			token = strtok(NULL, delims);
			w = atoi(token);
			token = strtok(NULL, delims);
			h = atoi(token);
			token = strtok(NULL, delims);
			dirfirst = atoi(token);
			token = strtok(NULL, delims);
			dotfiles = atoi(token);
			token = strtok(NULL, delims);
			viewtype = atoi(token);		
			rect = QRect(x, y, w, h);
		} else {
			rect = QRect(150, 30, 400, 300);
			dirfirst = 0;
			dotfiles = 1;
			viewtype = 0;
		}
		winlist.append(new MFinderWin(0L, path, &rect, dirfirst, dotfiles, viewtype));
		selectFileObjectFatherOfWindow(path, FALSE);
	}
}

void mfinderapp::DirOpenFromFindFileWin(char *findfilepath)
{
	MFinderWin *FWin = NULL;
	FileListObj *filelistobj = NULL, *listobjleaf = NULL;
	int i, count;
	bool isopen;
	QRect rect;
	char *rwin;
	char rectbuf[64] = "";
	char path[MAX_PATH_LEN] = "";
	char *delims = "|";
	char *token;
	int x, y, w, h;
	bool dirfirst, dotfiles;
	int viewtype;

	sprintf(path, "%s", deleteLastPathComponent(findfilepath));
	count = winlist.count();
	isopen = FALSE;
	for(i = 0; i < count; i++) {
		FWin = winlist.at(i);
		if(strcmp(FWin->myPath, path) == 0) {
			FWin->raise();
			FWin->setActiveWindow();
			isopen = TRUE;
			FWin->scrollToSeeObjectAtPath(findfilepath);
			break;
		} else {
			if(FWin->viewtype == BYNAME) {
				for(int j = 0; j < FWin->objslistframe->list.count(); j++) {
					filelistobj = FWin->objslistframe->list.at(j);
					if(filelistobj->isDir) {
						if(filelistobj->isOpenLocal) {
							if(strcmp(filelistobj->myFullPath, path) == 0) {
								FWin->raise();
								FWin->setActiveWindow();
								isopen = TRUE;
								FWin->scrollToSeeObjectAtPath(findfilepath);	
								break;
							} else {
								listobjleaf = filelistobj->discendentFileObjectAtPath(path);
								if(listobjleaf != NULL) {
									if(listobjleaf->isOpenLocal) {
										FWin->raise();
										FWin->setActiveWindow();
										isopen = TRUE;
										FWin->scrollToSeeObjectAtPath(findfilepath);	
										break;
									}
								}
							}
						}
					}
				}
			}
		}
	}
	if(!isopen) {
		rwin = rectOfWinAtPath(path);
		if(rwin) {
			sprintf(rectbuf, "%s", rwin);
			token = strtok(rectbuf, delims);
			x = atoi(token);
			token = strtok(NULL, delims);
			y = atoi(token);
			token = strtok(NULL, delims);
			w = atoi(token);
			token = strtok(NULL, delims);
			h = atoi(token);
			token = strtok(NULL, delims);
			dirfirst = atoi(token);
			token = strtok(NULL, delims);
			dotfiles = atoi(token);
			token = strtok(NULL, delims);
			viewtype = atoi(token);		
			rect = QRect(x, y, w, h);
		} else {
			rect = QRect(150, 30, 400, 300);
			dirfirst = 0;
			dotfiles = 1;
			viewtype = 0;
		}
		winlist.append(new MFinderWin(0L, path, &rect, dirfirst, dotfiles, viewtype));
		winlist.current()->scrollToSeeObjectAtPath(findfilepath);	
	} 
}

void mfinderapp::openDirAt()
{
	DlogOpenAt *dopenat;
	char openpath[MAX_PATH_LEN] = "";
	struct stat listStat;
	bool iserr = FALSE;
		
	dopenat = new DlogOpenAt(0L, "dopenat");
	if(dopenat->exec()) {
		sprintf(openpath, "%s", dopenat->editfield->text());
		if(openpath[strlen(openpath)-1] == '/')
			openpath[strlen(openpath)-1] = '\0';
		if(lstat(openpath, &listStat)) {
			iserr = TRUE;
		} else {
			if(!S_ISDIR(listStat.st_mode)) 
				iserr = TRUE;
		}
		if(iserr) {
			sprintf(ERR, "\"%s\" doesn't exist\nor is not a directory!\n", openpath);
			fileopsErr("Error", ERR);
			return;
		} else {
			DirOpen(openpath);
		}
	}
}

void mfinderapp::DirClose(char *winpath)
{
	MFinderWin *wintoclose = NULL;
	char rectbuf[32] = "";

	wintoclose = winAtPath(winpath);
	if((winlist.count() -1) == 0) {
		sprintf(ERR, "This is your last window!\nYou can iconify it\nor exit WMFinder!");
		dialog = new Dialog(0L, "Iconify or Quit", TRUE, 0, TRUE, TRUE, FALSE, TRUE, ERR);
		dialog->cancel->setText("Quit");
		dialog->ok->setText("Iconify");
		if(dialog->exec()) 
			wintoclose->iconify();
		else
			quit();
	} else {
		selectFileObjectFatherOfWindow(wintoclose->myPath, TRUE);

		sprintf(rectbuf, "%i|%i|%i|%i|%i|%i|%i", 
					wintoclose->x(), wintoclose->y(), wintoclose->width(), wintoclose->height(), 
					wintoclose->dirfirst, wintoclose->dotfiles, wintoclose->viewtype);

		for(int j = 0; j < wGeomDict.count(); j++)
			if(strcmp(wGeomDict.at(j)->key, wintoclose->myPath) == 0)
				wGeomDict.remove(j);

		wGeomDict.append(new dictEntry());
		dictEntry *de = wGeomDict.current();
		strcpy(de->key, wintoclose->myPath);
		strcpy(de->value, rectbuf);

		winlist.removeRef(wintoclose);
	}
}

void mfinderapp::openSelectedObjectsOfWin(MFinderWin *awin, bool wasclosed)
{
	int i, count;
	FileObj *fileobj;
	FileListObj *filelistobj;
	char ffullpath[MAX_PATH_LEN] = "";
	bool isplainfile;	

	setOverrideCursor(waitCursor);
	if(awin->viewtype == BYNAME) {
		count = awin->objslistframe->list.count();
		awin->objslistframe->isShiftClick = TRUE;
	} else {	
		count = awin->objsframe->list.count();
		awin->objsframe->isShiftClick = TRUE;
	}
	for(i = 0; i < count; i++) {
		isplainfile = FALSE;
		if(awin->viewtype == BYNAME) {
			filelistobj = awin->objslistframe->list.at(i);
			if(filelistobj->isSelect) {
				if(filelistobj->myType == DR || filelistobj->myType == DRLNK) {
					DirOpen(filelistobj->myFullPath);
					if(filelistobj->isOpenLocal)
						filelistobj->CloseLocal();
				}
				if(filelistobj->myType == FIL || filelistobj->myType == FILLNK) {
					isplainfile = TRUE;
					strcpy(ffullpath, filelistobj->myFullPath);
				}
			}
		} else {
			fileobj = awin->objsframe->list.at(i);
			if(fileobj->isSelect) {
				if(fileobj->myType == DR || fileobj->myType == DRLNK) {
					DirOpen(fileobj->myFullPath);
				}
				if(fileobj->myType == FIL || fileobj->myType == FILLNK) {
					isplainfile = TRUE;
					strcpy(ffullpath, fileobj->myFullPath);
				}
			}		
		}		
		if(isplainfile) {
			sendQueryToXWServer(OPENFILE, ffullpath);
		}
	}
	if(awin->viewtype == BYNAME) 
		awin->objslistframe->isShiftClick = FALSE;
	else	
		awin->objsframe->isShiftClick = FALSE;
	if(wasclosed)
		awin->close();
	restoreOverrideCursor();
}

void mfinderapp::openAtPath(char *path, char *name, int type)
{
	char ffullpath[MAX_PATH_LEN] = "";

	setOverrideCursor(waitCursor);
	if(type == DR || type == DRLNK) {
		DirOpen(path);
		MFinderWin *win = winAtPath(deleteLastPathComponent(path));
		if(win) {
			if(win->viewtype == BYNAME) {
				FileListObj *flobj = win->objslistframe->fileObjAtPath(path);
				if(flobj) {
					if(flobj->isOpenLocal)
						flobj->CloseLocal();
				}
			}
		}
	}
	if(type == FIL || type == FILLNK) {
		strcpy(ffullpath, path);
		sendQueryToXWServer(OPENFILE, ffullpath);
	}
	restoreOverrideCursor();
}

void mfinderapp::openSelectedObjectsOfFileListObject(FileListObj *anobject, bool wasclosed)
{
	int i, count;
	FileListObj *filelistobj;
	char ffullpath[MAX_PATH_LEN] = "";

	setOverrideCursor(waitCursor);
	anobject->myobjsframe->isShiftClick = TRUE;
	count = anobject->list.count();
	for(i = 0; i < count; i++) {
		filelistobj = anobject->list.at(i);
		if(filelistobj->isSelect) {
			if(filelistobj->myType == DR || filelistobj->myType == DRLNK) {
				DirOpen(filelistobj->myFullPath);
				if(filelistobj->isOpenLocal)
					filelistobj->CloseLocal();
			}
			if(filelistobj->myType == FIL || filelistobj->myType == FILLNK) {
				strcpy(ffullpath, filelistobj->myFullPath);
				sendQueryToXWServer(OPENFILE, ffullpath);
			}
		}	
	}		
	anobject->myobjsframe->isShiftClick = FALSE;
	if(wasclosed)
		anobject->myobjsframe->myfwin->close();
	restoreOverrideCursor();
}

void mfinderapp::unselectFileObjectsOfOtherWins(char *winpath)
{
	MFinderWin *FWin;

	for(int i = 0; i < winlist.count(); i++) {
		FWin = winlist.at(i);
		if(strcmp(FWin->myPath, winpath) != 0) {
			if(FWin->viewtype == BYNAME)
				FWin->objslistframe->unselectAllFileObjects();
			else
				FWin->objsframe->unselectAllFileObjects();
		}
	}
	for(int i = 0; i < deskobjlist.count(); i++) {
		DeskObj *dobj = deskobjlist.at(i);
		if(!dobj->ismyobj)
			if(dobj->isSelect)
				dobj->Unselect(FALSE);
	}
}

void mfinderapp::unselectAllObjNotDsObj(DeskObj *keepdsob)
{
	MFinderWin *FWin;

	for(int i = 0; i < winlist.count(); i++) {
		FWin = winlist.at(i);
		if(FWin->viewtype == BYNAME)
			FWin->objslistframe->unselectAllFileObjects();
		else
			FWin->objsframe->unselectAllFileObjects();
	}
	for(int i = 0; i < deskobjlist.count(); i++) {
		DeskObj *dobj = deskobjlist.at(i);
		if(dobj != keepdsob) {
			if(dobj->isSelect && !dobj->isOpen)
				dobj->Unselect(FALSE);
			if(dobj->isSelect && dobj->isOpen)
				dobj->UnselOpen(FALSE);
		}
	}
}

void mfinderapp::selectFileObjectFatherOfWindow(char *namewin, bool wasClosed)
{
	FileObj *fileobj;
	FileListObj *filelistobj, *listobjleaf;
	MFinderWin *FWin;
	char dirpath[MAX_PATH_LEN] = "";
	int i, j, count, localcount;
	bool found = FALSE;
	
	if(strcmp(namewin, "/") != 0) {
		sprintf(dirpath, "%s", deleteLastPathComponent(namewin));
		count = winlist.count();
		for(i = 0; i < count; i++) {
			FWin = winlist.at(i);
			if(strcmp(FWin->myPath, dirpath) == 0) {
				if(FWin->viewtype == BYNAME) {
					filelistobj = FWin->objslistframe->fileObjAtPath(namewin);
					if(filelistobj != NULL) {
						if(wasClosed)
							filelistobj->Close(true);
						else
							filelistobj->SelOpen(true);
						found = TRUE;
						break;
					}	
				} else {
					fileobj = FWin->objsframe->fileObjAtPath(namewin);
					if(fileobj != NULL) {
						if(wasClosed)
							fileobj->Close(true);
						else
							fileobj->SelOpen(true);
						found = TRUE;
						break;
					}
				}
			}
		}
		if(!found) {
			for(i = 0; i < count; i++) {
				if(found)
					break;
				FWin = winlist.at(i);	
				if(FWin->viewtype == BYNAME) {
					localcount = FWin->objslistframe->list.count();
					for(j = 0; j < localcount; j++) {
						filelistobj = FWin->objslistframe->list.at(j);
						if(filelistobj->isOpenLocal) {
							listobjleaf = filelistobj->discendentFileObjectAtPath(namewin);
							if(listobjleaf != NULL) {
								if(wasClosed)
									listobjleaf->Close(true);
								else
									listobjleaf->SelOpen(true);
								found = TRUE;
								break;
							}
						}
					}
				}
			}	
		}
		if(!found)
			unselectFileObjectsOfOtherWins(dirpath);			
	} else {
		unselectFileObjectsOfOtherWins("");			
	}
}

QPixmap mfinderapp::iconForFile(char *filepath, char *filename, 
														int type, bool issmall, 
															int state, bool iscustom)
{
	QPixmap pix;
	char fname[MAX_NAME_LEN] = "";
	char buff[32] = "";
	char buff2[MAX_PATH_LEN] = "";
	char fstate[8] = "";
	char fissmall[4] = "";
	char *pref, *suff, *token, *delims = ".";

	if(issmall)
		strcpy(fissmall, "SM");
	else
		strcpy(fissmall, "BG");
	
	switch(state) {
		case UNSELECT:
			strcpy(fstate, "UNSL");
			break;
		case SELECT:
			strcpy(fstate, "SEL");
			break;
		case OPEN:
			strcpy(fstate, "OPN");
			break;
		case SELOPEN:
			strcpy(fstate, "SOPN");
			break;
		default:
			strcpy(fstate, "UNSL");
			break;
	}

	if(iscustom) {
		sprintf(buff2, "%s%s%s", filepath, fstate, fissmall);
		if(QPixmapCache::find(buff2, pix))
        	return pix;
	}
	
	sprintf(fname, "%s", filename);
	token = strtok(fname, delims);
	pref = token;
	while(token != NULL) {
		suff = token;
		token = strtok(NULL, delims);
	}
	switch(type) {
		case FIL:
			if(filename[0] == '.') {
				sprintf(buff, ".*%s%s", fstate, fissmall);
				if(QPixmapCache::find(buff, pix))
        			return pix;
			}
			sprintf(buff, "%s*%s%s", pref, fstate, fissmall);
			if(QPixmapCache::find(buff, pix))
        		return pix;
			sprintf(buff, "*%s%s%s", suff, fstate, fissmall);
			if(QPixmapCache::find(buff, pix))
        		return pix;
			sprintf(buff, "plainfile%s%s", fstate, fissmall);
			QPixmapCache::find(buff, pix);
        		return pix;
			
		case FILEXE:
			sprintf(buff, "%s*%s%s", pref, fstate, fissmall);
			if(QPixmapCache::find(buff, pix))
        		return pix;
			sprintf(buff, "execfile%s%s", fstate, fissmall);
			QPixmapCache::find(buff, pix);
        		return pix;

		case FILLNK:
			sprintf(buff, "plainfilelink%s%s", fstate, fissmall);
			QPixmapCache::find(buff, pix);
        		return pix;
			
		case FILEXELNK:
			sprintf(buff, "execfilelink%s%s", fstate, fissmall);
			QPixmapCache::find(buff, pix);
        		return pix;
			
		case DR:
			sprintf(buff, "directory%s%s", fstate, fissmall);
			QPixmapCache::find(buff, pix);
        		return pix;		
			
		case DRLNK:
			sprintf(buff, "directorylink%s%s", fstate, fissmall);
			QPixmapCache::find(buff, pix);
        		return pix;		
			
		default:
			sprintf(buff, "plainfile%s%s", fstate, fissmall);
			QPixmapCache::find(buff, pix);
        		return pix;
	}
}

QPixmap mfinderapp::dragIconForFiles(int nfiles)
{
	QPixmap pix, dragpix;
	char pixpath[MAX_PATH_LEN] = "", labeltxt[24] = "";
	int labelw, labelh, icnw, icnh, asnt;
	QPainter p;

	sprintf(pixpath, "%s/Pixmaps/multiple.xpm", confdir);
	pix = QPixmap(pixpath, "XPM");
	sprintf(labeltxt, "%i elements", nfiles);
	QFontMetrics fm = QFontMetrics(font);
	asnt = fm.ascent();
	icnw = pix.width() + 2;
	icnh = pix.height() + 2;
	labelw = fm.width(labeltxt) + 2;
	labelh = fm.height() + 2;
	if(labelw < icnw) {
		dragpix = QPixmap(icnw, 50 + labelh);
 		dragpix.fill(color1);
		p.begin(&dragpix);
		p.drawPixmap(1, 1, pix);
   	p.fillRect((icnw - labelw) / 2, 50, labelw, labelh, black);		
   	p.setPen(white); 
		p.drawText((icnw - labelw) / 2 + 1, 51 + asnt, labeltxt);
		p.end();
	} else {
		dragpix = QPixmap(labelw, 50 + labelh);
 		dragpix.fill(color1);
		p.begin(&dragpix);
		p.drawPixmap((labelw - icnw) / 2, 1, pix);
   	p.fillRect(1, 50, labelw, labelh, black);		
   	p.setPen(white); 
		p.drawText(2, 51 + asnt, labeltxt);
		p.end();
	}
	dragpix.setMask(dragpix.createHeuristicMask());	
	return dragpix;
}

void mfinderapp::configureBaseIcons()
{
	baseIconWin = new ConfBaseIcnWin(0L, "biwin", iconsPath);
	
	sprintf(baseIconWin->icnFilePath, "%s", getIcnReqDictEntryForKey("plainfile"));
	sprintf(baseIconWin->icnFileLnkPath, "%s", getIcnReqDictEntryForKey("plainfilelink"));
	sprintf(baseIconWin->icnFileExePath, "%s", getIcnReqDictEntryForKey("execfile"));
	sprintf(baseIconWin->icnFileExeLnkPath, "%s", getIcnReqDictEntryForKey("execfilelink"));
	sprintf(baseIconWin->icnFoldPath, "%s", getIcnReqDictEntryForKey("directory"));
	sprintf(baseIconWin->icnFoldLnkPath, "%s", getIcnReqDictEntryForKey("directorylink"));

	baseIconWin->show();
}

void mfinderapp::makeNewReqIconList()
{
	char buff[MAX_PATH_LEN] = "";
	MFinderWin *FWin;
	
	setOverrideCursor(waitCursor);

	setIcnReqDictEntryForKey("plainfile", baseIconWin->icnFilePath);
	sprintf(buff, "%s/%s", iconsPath, baseIconWin->icnFilePath);
	loadAndTuneIcon(buff, "plainfile");

	setIcnReqDictEntryForKey("plainfilelink", baseIconWin->icnFileLnkPath);
	sprintf(buff, "%s/%s", iconsPath, baseIconWin->icnFileLnkPath);
	loadAndTuneIcon(buff, "plainfilelink");	

	setIcnReqDictEntryForKey("execfile", baseIconWin->icnFileExePath);
	sprintf(buff, "%s/%s", iconsPath, baseIconWin->icnFileExePath);
	loadAndTuneIcon(buff, "execfile");	

	setIcnReqDictEntryForKey("execfilelink", baseIconWin->icnFileExeLnkPath);
	sprintf(buff, "%s/%s", iconsPath, baseIconWin->icnFileExeLnkPath);
	loadAndTuneIcon(buff, "execfilelink");	

	setIcnReqDictEntryForKey("directory", baseIconWin->icnFoldPath);
	sprintf(buff, "%s/%s", iconsPath, baseIconWin->icnFoldPath);
	loadAndTuneFolderIcon(buff, "directory");	

	setIcnReqDictEntryForKey("directorylink", baseIconWin->icnFoldLnkPath);
	sprintf(buff, "%s/%s", iconsPath, baseIconWin->icnFoldLnkPath);
	loadAndTuneFolderIcon(buff, "directorylink");	

	baseIconWin->close();
	writeDefaults();
	for(int i = 0; i < winlist.count(); i++) {
		FWin = winlist.at(i);
		if(FWin->viewtype == BYNAME) 
			for(int j = 0; j < FWin->objslistframe->allobjslist.count(); j++) 
				FWin->objslistframe->allobjslist.at(j)->updateMyIcon();
		else 	
			for(int j = 0; j < FWin->objsframe->list.count(); j++)
				FWin->objsframe->list.at(j)->updateMyIcon();
	}
	restoreOverrideCursor();
}

void mfinderapp::configureFilesIcons()
{
	QList<char> prefsufflist, nameslist;
		
	for(int i = 0; i < icnDict.count(); i++) {
		dictEntry *de = icnDict.at(i);
		prefsufflist.append(new char[strlen(de->key)+1]);
		strcpy(prefsufflist.current(), de->key);
		nameslist.append(new char[strlen(de->value)+1]);
		strcpy(nameslist.current(), de->value);
	}
	filesIconWin = new ConfFilesIcnWin(0L, "fiwin", iconsPath, prefsufflist, nameslist);
	filesIconWin->show();
}

void mfinderapp::makeNewIconList()
{
	char buff[MAX_PATH_LEN] = "";
	MFinderWin *FWin;

	setOverrideCursor(waitCursor);

	for(int i = 0; i < filesIconWin->keyslist.count(); i++) {
		setIcnDictEntryForKey(filesIconWin->keyslist.at(i), filesIconWin->icnnamelist.at(i));
		sprintf(buff, "%s/%s", iconsPath, filesIconWin->icnnamelist.at(i));
		loadAndTuneIcon(buff, filesIconWin->keyslist.at(i));		
	}
	filesIconWin->close();
	writeDefaults();
	for(int i = 0; i < winlist.count(); i++) {
		FWin = winlist.at(i);
		if(FWin->viewtype == BYNAME) 
			for(int j = 0; j < FWin->objslistframe->allobjslist.count(); j++) 
				FWin->objslistframe->allobjslist.at(j)->updateMyIcon();
		else 	
			for(int j = 0; j < FWin->objsframe->list.count(); j++)
				FWin->objsframe->list.at(j)->updateMyIcon();
	}
	restoreOverrideCursor();
}

void mfinderapp::showInfo()
{
	char buff[MAX_PATH_LEN] = "";

	sprintf(buff, "%s/StrScr.xpm", iconsPath);
	wmfinfo = new WMFInfo(0L, "ss", buff);
	wmfinfo->show();
}

MFinderWin* mfinderapp::winAtPath(char *apath)
{
	int i, count;
	MFinderWin *FWin = NULL;
	
	count = winlist.count();
	for(i = 0; i < count; i++) {
		if(strcmp(winlist.at(i)->myPath, apath) == 0) {
			FWin = winlist.at(i);
			break;
		}
	}
	return FWin;
}

void mfinderapp::setFocusWin(MFinderWin *newfocuswin)
{
	focusWin = newfocuswin;
}

MFinderWin* mfinderapp::currentFocusWin()
{
	return focusWin;
}

FileObj* mfinderapp::fileObjAtPath(char *apath)
{
	MFinderWin *FWin;
	FileObj *fileobj = NULL;

	for(int i = 0; i < winlist.count(); i++) {
		FWin = winlist.at(i);
		if(FWin->viewtype != BYNAME) {
			fileobj = FWin->objsframe->fileObjAtPath(apath);
			if(fileobj) 
				return fileobj;
		}
	}
	return fileobj;
}

FileListObj* mfinderapp::fileListObjAtPath(char *apath)
{
	MFinderWin *FWin;
	FileListObj *fileobj = NULL;

	for(int i = 0; i < winlist.count(); i++) {
		FWin = winlist.at(i);
		if(FWin->viewtype == BYNAME) {
			fileobj = FWin->objslistframe->fileObjInTreeAtPath(apath);
			if(fileobj) 
				return fileobj;
		}
	}
	return fileobj;
}

DeskObj* mfinderapp::deskObjectAtPath(char *apath)
{
	int i, count;
	DeskObj *deskobj = NULL;

	count = deskobjlist.count();
	for(i = 0; i < count; i++) {
		if(strcmp(deskobjlist.at(i)->myFullPath, apath) == 0) {
			deskobj = deskobjlist.at(i);
			break;
		}
	}
	return deskobj;
}

void mfinderapp::centreDialogOnScreen(QWidget *d)
{
	d->move((desktop()->width() - d->width()) / 2, 
					  		(desktop()->height() - d->height()) / 2);
}

void mfinderapp::processPendingData()
{
	verifyXWSConnection();
}

bool mfinderapp::x11EventFilter(XEvent *e)
{
	if(e->type == ClientMessage) {
		if(strcmp(XGetAtomName(dspl, e->xclient.message_type), "XWServerReply") == 0) {
			parseServerReply(e);
			return TRUE;
		}
	}
	return QApplication::x11EventFilter(e);
}

void mfinderapp::quit()
{
	MFinderWin *FWin;
	char rectbuf[128] = "";
	int i, count, wcount;
	
	wOpenDict.clear();
	
	for(i = 0; i < winlist.count(); i++) {
		FWin = winlist.at(i);

		for(int j = 0; j < wGeomDict.count(); j++)
			if(strcmp(wGeomDict.at(j)->key, FWin->myPath) == 0)
				wGeomDict.remove(j);
		
		sprintf(rectbuf, "%i|%i|%i|%i|%i|%i|%i", 
						FWin->x(), FWin->y(), FWin->width(), FWin->height(), 
								FWin->dirfirst, FWin->dotfiles, FWin->viewtype);
						
		wGeomDict.append(new dictEntry());
		dictEntry *de = wGeomDict.current();
		strcpy(de->key, FWin->myPath);
		strcpy(de->value, rectbuf);

		wOpenDict.append(new char[MAX_PATH_LEN]);
		strcpy(wOpenDict.current(), FWin->myPath);
	}
	
	writeDefaults();
	winlist.clear();
	qApp->quit();	
}
