#ifndef ___mfinderapp___
#define ___mfinderapp___

#include "common.h"
#include "dialogs.h"
#include "mfinderwin.h"
#include "fileobj.h"
#include "filelistobj.h"
#include "deskobj.h"
#include "objsframe.h"
#include "objslistframe.h"
#include "configure.h"
#include "findwin.h"

#include <qapp.h>
#include <qsignal.h>
#include <qdict.h>
#include <qlist.h> 
#include <qdatastream.h> 
#include <qfile.h>
#include <qpixmapcache.h> 
#include <qprogressbar.h> 
#include <qlabel.h>
#include <qbuttongroup.h>
#include <qradiobutton.h> 
#include <qtimer.h> 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>
#include <errno.h>
#include <sys/stat.h>
#include <dirent.h>
#include <sys/param.h>
#include <ctype.h>
#include <math.h>
#include <signal.h>
#include <fcntl.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/wait.h>
#include <X11/X.h>
#include <X11/Intrinsic.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
	
typedef struct {
	int uid, gid;
	char *name;
  	char home[MAX_PATH_LEN];
  	char shell[MAX_PATH_LEN];
  	char hostname[128];
  	mode_t umask;
} UserInfo;	
	
struct fileMetaData {
	char filepath[MAX_PATH_LEN];
	bool isopenlocal;
	bool hascustomicn;
	char custicnname[MAX_NAME_LEN];
	bool hascustomedit;
	char custedit[128];
};

struct dictEntry {
	char key[MAX_PATH_LEN];
	char value[128];
};

class QPathList : public QStrList
{
private:
	int compareItems(GCI s1, GCI s2);
};

class mfinderapp : public QApplication
{
	Q_OBJECT
public: 
   mfinderapp(int &argc, char** argv, const char *cln=NULL, 
											const unsigned char* icondata=NULL);

	QPixmap iconForFile(char *, char *, int, bool, int, bool);
	QPixmap dragIconForFiles(int);
	void unselectFileObjectsOfOtherWins(char *);
	void unselectAllObjNotDsObj(DeskObj *);
	void selectFileObjectFatherOfWindow(char *, bool);
	void openSelectedObjectsOfWin(MFinderWin *, bool);
	void openSelectedObjectsOfFileListObject(FileListObj *, bool);
	void openAtPath(char*, char*, int);
	MFinderWin* winAtPath(char *);
	void setFocusWin(MFinderWin*);
	MFinderWin* currentFocusWin();
	FileObj* fileObjAtPath(char *);
	FileListObj* fileListObjAtPath(char *);
	void propagatePathChangeInWins(char *, char *);
	void fileopsErr(char *, char *);
	void moveOrCopy(QStrList, char*, QWidget*);
	void newFileObject(MFinderWin*, bool);
	void removeFileObjects(MFinderWin*);	
	void renameFileObjects(MFinderWin*);	
	void duplicateFileObjects(MFinderWin*);	
	void getFileAttributesAtPath(char*);
	void Edit(MFinderWin*);	
	void OpenWith(MFinderWin*);	
	void StartXTerm(MFinderWin*);	
	void InfoObj(MFinderWin*);	
	void findFile(char*);
	void doFind(char*);
	void offixDrop(QWidget*);		
	void makeDeskObjFromDrop(char*, int, int);
	void makeDeskObjFromDictEntry(char*);
	void removeDeskObj(DeskObj*);
	DeskObj* deskObjectAtPath(char *);
	bool x11EventFilter(XEvent *);
		
	QList<MFinderWin> winlist; 
	QList<DeskObj> deskobjlist; 

	MFinderWin *focusWin;
	Dialog *dialog;
	void centreDialogOnScreen(QWidget *);
	ConfBaseIcnWin *baseIconWin;
	ConfFilesIcnWin *filesIconWin;
	
	WMFInfo *wmfinfo;

	bool isdragcopying;
	bool deskobjmoving;
	bool xdnddone;
	bool dndok;
	bool isfinding;
	int xappnum;
	FileObj *fileDragObj;
	FileListObj *fileListDragObj;
	DeskObj *deskDragObj;
	FindWin *findwin;
	
	Atom DndProtocol, DndSelection;
	Display *dspl;	
	int appScreen;			// X11 screen number
	Window appRootWin;	// X11 root window
	// -------------------- Offix DnD -------------------------------
	XButtonEvent	StartEvent;		/* event that started the drag */
	int				DragPrecision;	/* minimum dx,dy to start drag */
	int				Dragging;		/* Drag state flag */
	bool				DataOK;			/* Non-zero if data registered */
	Atom				WM_STATE;		/* Needed for icon stuff */
	Window			Target;			/* Drop window */
	Widget			MainWidget;		/* Main widget of application	*/
	int				DataType;		/* Current drag data type */
	XtEventHandler	OtherDrop;		/* Drop on non-reg app widget */
	XtEventHandler	RootDrop;		/* How to handle root drops */
	XtEventHandler	IconDrop;		/* How to handle icon drops */
	int				RootFlag;		/* Non-zero if dropped on root */
	long           LastDropVersion;/* Last drop protocol version */
	// --------------------------------------------------------------

	QTimer *timer;

	char findstartpath[MAX_PATH_LEN];

	QList<dictEntry> wGeomDict;					// windows geometry
	char *rectOfWinAtPath(char*);
	QList<char> wOpenDict;							// windows to open
	QList<char> deskObjsDict;						// desk objects
	void addDeskObjToDict(DeskObj*);
	void removeDeskObjFromDict(char*);
	QList<dictEntry> icnReqDict;					// required icons
	void setIcnReqDictEntryForKey(char*, char*);
	char *getIcnReqDictEntryForKey(char*);
	QList<dictEntry> icnDict;						// icons
	void setIcnDictEntryForKey(char*, char*);
	char *getIcnDictEntryForKey(char*);
	QList<fileMetaData> filesDict;				// objects custom data
	bool hasCustomIconFileAtPath(char*);
	char *customIconNameOfFileAtPath(char*);
	void setCustomIconOfFileAtPath(char*, char*);
	void unsetCustomIconOfFileAtPath(char*);
	void setIsOpenLocalOfFileAtPath(char*);
	void unsetIsOpenLocalOfFileAtPath(char*);
	bool isOpenLocalOfFileAtPath(char*);

// ---------------- XWorkspace -----------------
	void connectionError(char*);
	void startXWSConnection(long);
	void verifyXWSConnection();
	void sendQueryToXWServer(int, char*);
	void parseServerReply(XEvent *);
// ---------------------------------------------	
		
signals:

public slots:
	void processPendingData();
	void DirOpen(char *);
	void DirOpenFromFindFileWin(char *);
	void openDirAt();
	void DirClose(char *);
	void configureBaseIcons();
	void configureFilesIcons();
	void makeNewReqIconList();
	void makeNewIconList();
	void parseFindQuery();
	void stopFind();
	void showInfo();
	
	void writeDefaults();
	void quit();
	
protected:	
   char *appClassName;
   QPixmap *icon;
	
private: 
	UserInfo user;
	QFont font;
	
	StrtScrn *strscrn, *strscrnft;
	
	char ERR[512];
	ChooseAnIcnDlg *chanicndlg;
	
};

#endif
