#include "mfinderwin.h"
#include "mfinderapp.h"
#include "functions.h"

MFinderWin::MFinderWin(QWidget *parent, const char *path, QRect *rect,
											bool dirfirst, bool dotfiles, int viewtype)
			  : QWidget(0L, 0L)
{
	strcpy(myPath, path);
	this->dirfirst = dirfirst;
	this->dotfiles = dotfiles;
	this->viewtype = viewtype;
	
	setIconText(myPath);
	setCaption(myPath);
	setMinimumWidth(200);
	setGeometry(QRect(rect->topLeft(), rect->bottomRight()));

	QColor x = QColor(230, 230, 230);
	QColorGroup CGnorm = QColorGroup(black, x, x.light(), x.dark(), x.dark(120), black, x);
	setPalette(QPalette(CGnorm, CGnorm, CGnorm));
	
	sBar = new MfindScrBar(this, "barv");
	connect(sBar, SIGNAL(valueChanged(int)), this, SLOT(ScrollV(int)));
	YScrollBar = 0;
	Info = new InfoFrame(this, myPath);
	
	if(viewtype == BYICONS || viewtype == BYSMICONS) {
		objsframe = new ObjsFrame(this, myPath, dirfirst, dotfiles, viewtype);
		connect(objsframe, SIGNAL(resizeDone(int)), this, SLOT(ResizeSBar(int)));
		connect(objsframe, SIGNAL(updateInfo()), Info, SLOT(updateData()));
		objsframe->lower();
	} else {
		objslistframe = new ObjsListFrame(this, myPath, dirfirst, dotfiles);
		connect(objslistframe, SIGNAL(resizeDone(int)), this, SLOT(ResizeSBar(int)));
		connect(objslistframe, SIGNAL(updateInfo()), Info, SLOT(updateData()));
		objslistframe->lower();
	}

	iskeybuff = FALSE;
	buffpos = 0;
	show();
}

MFinderWin::~MFinderWin()
{

}

void MFinderWin::scrollToSeeObjectAtPath(char *objpath)
{
	FileObj *fobj = NULL;
	FileListObj *flstobj = NULL;
	QRect myr, objr;
	
	if(viewtype == BYNAME) {
		flstobj = objslistframe->fileObjInTreeAtPath(objpath);
		if(flstobj) {
			myr = QRect(objslistframe->mapFromParent(rect().topLeft()), 
										objslistframe->mapFromParent(rect().bottomRight()));
			objr = QRect(flstobj->mapToParent(flstobj->icona.rect().topLeft()), 
							 flstobj->mapToParent(flstobj->icona.rect().bottomRight()));
			if(myr.contains(objr)) {
				flstobj->Select(TRUE);
			} else {
				sBar->setValue(objr.topLeft().y());
				flstobj->Select(TRUE);
			}
		}
	} else {
		fobj = objsframe->fileObjAtPath(objpath);
		if(fobj) {
			myr = QRect(objsframe->mapFromParent(rect().topLeft()), 
											objsframe->mapFromParent(rect().bottomRight()));
			objr = QRect(fobj->mapToParent(fobj->rect().topLeft()), 
								 				fobj->mapToParent(fobj->rect().bottomRight()));
			if(myr.contains(objr)) {
				fobj->Select(TRUE);
			} else {
				sBar->setValue(objr.topLeft().y());
				fobj->Select(TRUE);
			}
		}
	}
}

void MFinderWin::changePathAndPropagate(char *newpath)
{
	sprintf(myPath, "%s", newpath);
	setCaption(myPath);
	sprintf(Info->path, "%s", newpath);
	Info->updateData();

// Aggiungere il menu !!!!!
// Aggiungere il menu !!!!!
// Aggiungere il menu !!!!!
// Aggiungere il menu !!!!!

	if(viewtype == BYICONS || viewtype == BYSMICONS) {
		objsframe->~ObjsFrame();
		objsframe = new ObjsFrame(this, myPath, dirfirst, dotfiles, viewtype);
		connect(objsframe, SIGNAL(resizeDone(int)), this, SLOT(ResizeSBar(int)));
		connect(objsframe, SIGNAL(updateInfo()), Info, SLOT(updateData()));
		objsframe->show();
		objsframe->lower();
		QResizeEvent e = QResizeEvent(this->size(), this->size());
		this->event(&e);
	}
	if(viewtype == BYNAME) {
		objslistframe->~ObjsListFrame();
		objslistframe = new ObjsListFrame(this, myPath, dirfirst, dotfiles);
		connect(objslistframe, SIGNAL(resizeDone(int)), this, SLOT(ResizeSBar(int)));
		connect(objslistframe, SIGNAL(updateInfo()), Info, SLOT(updateData()));
		objslistframe->show();
		objslistframe->lower();
		QResizeEvent e = QResizeEvent(this->size(), this->size());
		this->event(&e);		
	}
	((mfinderapp *)qApp)->selectFileObjectFatherOfWindow(myPath, FALSE);	
}

//-------------------------------------------------------------------------
//                               slots
//-------------------------------------------------------------------------
void MFinderWin::ScrollV(int y)
{
	YScrollBar = y;
	
	if(viewtype == BYNAME)
		objslistframe->setGeometry(24, YScrollBar*(-1)+25, width()-26, height()-27+YScrollBar);

	if(viewtype == BYSMICONS)
		objsframe->setGeometry(24, YScrollBar*(-1)+25, width()-26, height()-27+YScrollBar);
	
	if(viewtype == BYICONS)
		objsframe->setGeometry(24, YScrollBar*(-1)+25, width()-26, height()-27+YScrollBar);
}

void MFinderWin::ResizeSBar(int y)
{
	if((y - height()+30) > 0)
		sBar->setRange(0, y - height()+30);
	else
		sBar->setRange(0, 1);
}

void MFinderWin::closeme()
{
	this->close();
}

void MFinderWin::changeViewByIcons()
{
	if(viewtype != BYICONS) {
		if(viewtype == BYNAME) 
			objslistframe->~ObjsListFrame();
		if(viewtype == BYSMICONS) 
			objsframe->~ObjsFrame();
		viewtype = BYICONS;
		objsframe = new ObjsFrame(this, myPath, dirfirst, dotfiles, viewtype);
		connect(objsframe, SIGNAL(resizeDone(int)), this, SLOT(ResizeSBar(int)));
		connect(objsframe, SIGNAL(updateInfo()), Info, SLOT(updateData()));
		objsframe->show();
		objsframe->lower();
		QResizeEvent e = QResizeEvent(this->size(), this->size());
		this->event(&e);
	}
}

void MFinderWin::changeViewBySmallIcons()
{
	if(viewtype != BYSMICONS) {
		if(viewtype == BYNAME) 
			objslistframe->~ObjsListFrame();
		if(viewtype == BYICONS) 
			objsframe->~ObjsFrame();
		viewtype = BYSMICONS;
		objsframe = new ObjsFrame(this, myPath, dirfirst, dotfiles, viewtype);
		connect(objsframe, SIGNAL(resizeDone(int)), this, SLOT(ResizeSBar(int)));
		connect(objsframe, SIGNAL(updateInfo()), Info, SLOT(updateData()));
		objsframe->show();
		objsframe->lower();
		QResizeEvent e = QResizeEvent(this->size(), this->size());
		this->event(&e);
	}
}

void MFinderWin::changeViewByName()
{
	if(viewtype != BYNAME) {
		if(viewtype == BYSMICONS) 
			objsframe->~ObjsFrame();
		if(viewtype == BYICONS) 
			objsframe->~ObjsFrame();
		viewtype = BYNAME;
		objslistframe = new ObjsListFrame(this, myPath, dirfirst, dotfiles);
		connect(objslistframe, SIGNAL(resizeDone(int)), this, SLOT(ResizeSBar(int)));
		connect(objslistframe, SIGNAL(updateInfo()), Info, SLOT(updateData()));
		objslistframe->show();
		objslistframe->lower();
		QResizeEvent e = QResizeEvent(this->size(), this->size());
		this->event(&e);		
	}
}

void MFinderWin::changeHideShowDotFiles()
{
	dotfiles = (!dotfiles) ? TRUE : FALSE;

	if(viewtype == BYNAME) {
		objslistframe->~ObjsListFrame();
		objslistframe = new ObjsListFrame(this, myPath, dirfirst, dotfiles);
		connect(objslistframe, SIGNAL(resizeDone(int)), this, SLOT(ResizeSBar(int)));
		connect(objslistframe, SIGNAL(updateInfo()), Info, SLOT(updateData()));
		objslistframe->show();
		objslistframe->lower();
	}
	if(viewtype == BYSMICONS) {
		objsframe->~ObjsFrame();
		objsframe = new ObjsFrame(this, myPath, dirfirst, dotfiles, viewtype);
		connect(objsframe, SIGNAL(resizeDone(int)), this, SLOT(ResizeSBar(int)));
		connect(objsframe, SIGNAL(updateInfo()), Info, SLOT(updateData()));
		objsframe->show();
		objsframe->lower();
	}
	if(viewtype == BYICONS) {
		objsframe->~ObjsFrame();
		objsframe = new ObjsFrame(this, myPath, dirfirst, dotfiles, viewtype);
		connect(objsframe, SIGNAL(resizeDone(int)), this, SLOT(ResizeSBar(int)));
		connect(objsframe, SIGNAL(updateInfo()), Info, SLOT(updateData()));
		objsframe->show();
		objsframe->lower();
	}
	QResizeEvent e = QResizeEvent(this->size(), this->size());
	this->event(&e);
}

void MFinderWin::addKeyToKeysBuff(QKeyEvent *e)
{
	char fpath[MAX_PATH_LEN] = "";
	char c;
	
	c = e->ascii();
	if(isalpha(c)) {
		if(!iskeybuff) {
			iskeybuff = TRUE;
			QTimer *timer = new QTimer(this);
    		connect(timer, SIGNAL(timeout()), this, SLOT(timerDone()));
    		timer->start(2000, TRUE);
		}
		keybuff[buffpos] = c;
		buffpos++;
		keybuff[buffpos] = '\0';
		sprintf(fpath, "%s", bestMatchingPath(myPath, keybuff));
		if(strlen(fpath) > 1)
			scrollToSeeObjectAtPath(fpath);
	}
}

void MFinderWin::timerDone()
{
	iskeybuff = FALSE;
	sprintf(keybuff, "");
	buffpos = 0;
}

void MFinderWin::scrollUp(bool apage)
{
	if(apage)
		sBar->subtractPage();
	else
		sBar->subtractLine();
}

void MFinderWin::scrollDown(bool apage)
{
	if(apage)
		sBar->addPage();
	else
		sBar->addLine();
}

//-------------------------------------------------------------------------
//                               eventi
//-------------------------------------------------------------------------

void MFinderWin::resizeEvent(QResizeEvent *)
{
	Info->setGeometry(0, 0, width(), 20);
	
	if(viewtype == BYNAME) {
		objslistframe->setGeometry(24, 25, width()-26, height()-27);
	}
	if(viewtype == BYICONS) {
		objsframe->setGeometry(24, 25, width()-26, height()-27);
	}
	if(viewtype == BYSMICONS) {
		objsframe->setGeometry(24, 25, width()-26, height()-27);
	}
		
	sBar->setGeometry(1, 20, 18, height()-20);
	sBar->setValue(0);
	
	if(viewtype == BYICONS)
		sBar->setSteps(75, height());
	else
		sBar->setSteps(20, height());
}

void MFinderWin::focusInEvent(QFocusEvent *)
{
	((mfinderapp *)qApp)->focusWin = this;
	if(viewtype == BYICONS || viewtype == BYSMICONS)
		objsframe->updateContents();
	else
		objslistframe->updateContents();
}

void MFinderWin::closeEvent(QCloseEvent *e)
{
	((mfinderapp *)qApp)->DirClose(myPath);
}

void MFinderWin::keyPressEvent(QKeyEvent *e)
{
	if(e->state() == ControlButton) {
		switch(e->key()) { 
			case Key_W: 
				close();
				break;	
			case Key_O:
				((mfinderapp *)qApp)->openSelectedObjectsOfWin(this, FALSE);		
				break;
			case Key_Q: 
				((mfinderapp *)qApp)->quit();
				break;	
			case Key_A: 
				((mfinderapp *)qApp)->setOverrideCursor(waitCursor);
				if(viewtype == BYNAME)
					objslistframe->selectAllFileObjects();
				else
					objsframe->selectAllFileObjects();
				((mfinderapp *)qApp)->restoreOverrideCursor();
				break;
			case Key_N: 
				((mfinderapp *)qApp)->newFileObject(this, TRUE);
				break;	
			case Key_M: 
				((mfinderapp *)qApp)->newFileObject(this, FALSE);
				break;	
			case Key_D: 
				((mfinderapp *)qApp)->removeFileObjects(this);
				break;
			case Key_R: 
				((mfinderapp *)qApp)->renameFileObjects(this);
				break;
			case Key_I: 
				((mfinderapp *)qApp)->InfoObj(this);
				break;
			case Key_T: 
				((mfinderapp *)qApp)->StartXTerm(this);
				break;
			case Key_E: 
				((mfinderapp *)qApp)->Edit(this);
				break;
			case Key_F: 
				((mfinderapp *)qApp)->OpenWith(this);
				break;				
			case Key_U: 
				((mfinderapp *)qApp)->duplicateFileObjects(this);
				break;
			default:             
				e->ignore();
				return;
		} 				
	} else {
		if(e->key() == Key_PageUp) {
			scrollUp(TRUE);
			return;
		}
		if(e->key() == Key_Up) {
			scrollUp(FALSE);
			return;
		}
		if(e->key() == Key_PageDown) {
			scrollDown(TRUE);
			return;
		}
		if(e->key() == Key_Down) {
			scrollDown(FALSE);
			return;
		}

		addKeyToKeysBuff(e);
	}
}
