#ifndef MFINDSCRBAR_H
#define MFINDSCRBAR_H

#include "common.h"
#include <qwidget.h>
#include <qrangect.h>
#include <qdrawutl.h>

class MfindScrBar : public QWidget, public QRangeControl
{
	Q_OBJECT
public:
	enum Orientation { Horizontal, Vertical };

	MfindScrBar(QWidget *parent=0, const char *name=0);

	Orientation orientation() const;
	void setTracking(bool enable);
	bool tracking() const;

signals:
	void valueChanged(int value);
	void sliderPressed();
	void sliderMoved(int value);
	void sliderReleased();
	void nextLine();
	void prevLine();
	void nextPage();
	void prevPage();

protected:
	void timerEvent(QTimerEvent *);
	void resizeEvent(QResizeEvent *);
	void paintEvent(QPaintEvent *);

	void mousePressEvent(QMouseEvent *);
	void mouseReleaseEvent(QMouseEvent *);
	void mouseMoveEvent(QMouseEvent *);

	void valueChange();
	void stepChange();
	void rangeChange();

	int sliderStart() const;
	QRect sliderRect() const;

private:
	void init();
	void positionSliderFromValue();
	int calculateValueFromSlider() const;

	uint pressedControl	 : 8;
	uint track		 : 1;
	uint clickedAt	 : 1;
	uint orient		 : 1;
	uint thresholdReached : 1;
	uint isTiming	 : 1;

	int slidePrevVal;
	QCOORD sliderPos;
	QCOORD clickOffset;

private:	// Disabled copy constructor and operator=
	MfindScrBar(const MfindScrBar &) {}
	MfindScrBar &operator=(const MfindScrBar &) { return *this; }
};


inline void MfindScrBar::setTracking(bool t)
{
	track = t;
}

inline bool MfindScrBar::tracking() const
{
	return track;
}

inline MfindScrBar::Orientation MfindScrBar::orientation() const
{
	return (Orientation)orient;
}

inline int MfindScrBar::sliderStart() const
{
	return sliderPos;
}


void qDrawArrow(QPainter *, ArrowType type, GUIStyle style, bool down,
		 int x, int y, int w, int h, const QColorGroup &);


#endif
