#include "objsframe.h"
#include "mfinderapp.h"
#include <qpainter.h> 
#include <qdrawutl.h>
#include <qtooltip.h>

extern bool dfirst;
extern bool iview;

ObjsFrame::ObjsFrame(QWidget *parent, const char *path, 
											bool dirfirst, bool dotfiles, int viewtype)
			  : QWidget(parent, path),
			  	 QDropSite(this)
{
	QColor x = QColor(230, 230, 230);
	QColorGroup CGnorm = QColorGroup(black, x, x.light(), x.dark(), x.dark(120), black, x);
	setPalette(QPalette(CGnorm, CGnorm, CGnorm));
	
	strncpy(myPath, path, MAX_PATH_LEN);
	this->dirfirst = dirfirst;
	this->dotfiles = dotfiles;
	this->viewtype = viewtype;
	dfirst = dirfirst;
	myfwin = (MFinderWin *)parent;
	list.setAutoDelete(TRUE);
	tmplist.setAutoDelete(FALSE);
	isShiftClick = FALSE;
	isDragShiftClick = FALSE;
	isSelDragging = FALSE;
	isDragTarget = FALSE;
	makeFileObjects();
	makePopUp();
}

ObjsFrame::~ObjsFrame()
{
	FileObj *fileobj;
	int i, count;

	count = list.count();
	for(i = 0; i < count; i++) {
		fileobj = list.at(0);
		list.removeRef(fileobj);
	}
	tmplist.clear();
}

void ObjsFrame::makeFileObjects()
{
	struct dirent **namelist;
	struct stat listStat;
	char filepath[MAX_PATH_LEN];
	char confdir[MAX_PATH_LEN];
	int m, a, i, type;
	bool fileok;

	iview = TRUE;
	sprintf(confdir, "%s/GNUstep/Library/xwfinder", getenv("HOME"));

  	m = scandir(myPath, &namelist, 0, alphasort);
	for(a = 0; a <= m - 1; a++) {
		fileok = FALSE;
		if(strcmp(namelist[a]->d_name, ".") != 0 
								&& strcmp(namelist[a]->d_name, "..") != 0)
			fileok = TRUE;
		if(!dotfiles)	
			if(namelist[a]->d_name[0] == '.')
				fileok = FALSE;

		if(fileok) {
			if(strcmp(myPath, "/") != 0)
				sprintf(filepath, "%s/%s", myPath, namelist[a]->d_name);
			else
				sprintf(filepath, "%s%s", myPath, namelist[a]->d_name);

   		lstat(filepath, &listStat);
			if(S_ISREG(listStat.st_mode)) {
				if(listStat.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH))
					type = FILEXE;
				else
					type = FIL;
			}
			if(S_ISDIR(listStat.st_mode)) 
				type = DR;
			if(S_ISLNK(listStat.st_mode)) {
				if(chdir(filepath) != 0) {
					if(listStat.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH)) {
						type = FILEXELNK;
					} else {
						type = FILLNK;
					}
				} else {
					chdir(confdir);
					type = DRLNK;
				}
			}		
			if(S_ISCHR(listStat.st_mode)) 
				type = CHRDV;		
			if(S_ISBLK(listStat.st_mode)) 
				type = BLCKDV;	
			if(S_ISFIFO(listStat.st_mode)) 
				type = FIF;		
			if(S_ISSOCK(listStat.st_mode)) 
				type = SCKT;		

			list.inSort(new FileObj(this, myPath, namelist[a]->d_name, type, viewtype));
		}
	}
	emit updateInfo();
}

void ObjsFrame::updateContents()
{
	struct dirent **namelist;
	int m, count, newcount;
	bool fileok, found;
	FileObj *fileobj;	
	QList<char> newlist; 
	QList<char> difflistp;
	QList<char> difflistm;
	
	newlist.setAutoDelete(TRUE);
	difflistp.setAutoDelete(TRUE);
	difflistm.setAutoDelete(TRUE);
	iview = TRUE;
	
  	m = scandir(myPath, &namelist, 0, alphasort);
	for(int i = 0; i <= m - 1; i++) {
		fileok = FALSE;
		if(strcmp(namelist[i]->d_name, ".") != 0 
								&& strcmp(namelist[i]->d_name, "..") != 0)
			fileok = TRUE;
		if(!dotfiles)	
			if(namelist[i]->d_name[0] == '.')
				fileok = FALSE;

		if(fileok) {
			newlist.append(new char[strlen(namelist[i]->d_name)+1]);
			strcpy(newlist.current(), namelist[i]->d_name);
		}
	}
	
	for(int i = 0; i < newlist.count(); i++) {
		found = FALSE;
		for(int j = 0; j < list.count(); j++)
			if(strcmp(newlist.at(i), list.at(j)->myName) == 0)
				found = TRUE;
		if(!found) {
			difflistp.append(new char[strlen(newlist.at(i))+1]);
			strcpy(difflistp.current(), newlist.at(i));
		}
	}
	
	for(int i = 0; i < list.count(); i++) {
		found = FALSE;
		for(int j = 0; j < newlist.count(); j++)
			if(strcmp(list.at(i)->myName, newlist.at(j)) == 0)
				found = TRUE;
		if(!found) {
			difflistm.append(new char[strlen(list.at(i)->myName)+1]);
			strcpy(difflistm.current(), list.at(i)->myName);
		}
	}
	
	if(difflistp.count() > 0)
		for(int i = 0; i < difflistp.count(); i++)
			addAnObject(difflistp.at(i));
	
	if(difflistm.count() > 0) {
		count = list.count();
		for(int i = 0; i < count; i++) {
			fileobj = list.at(i);
			for(int j = 0; j < difflistm.count(); j++) {
				if(strcmp(fileobj->myName, difflistm.at(j)) == 0) {
					list.removeRef(fileobj);
					count--;
					i--;
				}
			}
		}
	}
	
	newlist.clear();	
	difflistp.clear();	
	difflistm.clear();
	QResizeEvent e = QResizeEvent(this->size(), this->size());
	this->event(&e);
	emit updateInfo();
}

void ObjsFrame::removeAnObject(FileObj *obj)
{
	list.removeRef(obj);
	QResizeEvent e = QResizeEvent(this->size(), this->size());
	this->event(&e);
}

void ObjsFrame::addAnObject(char *nameobj)
{
	char filepath[MAX_PATH_LEN];
	char confdir[MAX_PATH_LEN];
	struct stat listStat;
	int type;

	iview = TRUE;
	dfirst = dirfirst;	
	sprintf(confdir, "%s/GNUstep/Library/xwfinder", getenv("HOME"));
	if(strcmp(myPath, "/") != 0)
		sprintf(filepath, "%s/%s", myPath, nameobj);
	else
		sprintf(filepath, "%s%s", myPath, nameobj);

   lstat(filepath, &listStat);
	if(S_ISREG(listStat.st_mode)) {
		if(listStat.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH))
			type = FILEXE;
		else
			type = FIL;
	}
	if(S_ISDIR(listStat.st_mode)) 
		type = DR;
	if(S_ISLNK(listStat.st_mode)) {
		if(chdir(filepath) != 0) {
			if(listStat.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH)) {
				type = FILEXELNK;
			} else {
				type = FILLNK;
			}
		} else {
			chdir(confdir);
			type = DRLNK;
		}
	}		
	if(S_ISCHR(listStat.st_mode)) 
		type = CHRDV;		
	if(S_ISBLK(listStat.st_mode)) 
		type = BLCKDV;	
	if(S_ISFIFO(listStat.st_mode)) 
		type = FIF;		
	if(S_ISSOCK(listStat.st_mode)) 
		type = SCKT;		

	list.inSort(new FileObj(this, myPath, nameobj, type, viewtype));
}

void ObjsFrame::changeNormalDirFirstSort()
{
	FileObj *fileobj;
	int i, count;

	iview = TRUE;

	dirfirst = (!dirfirst) ? TRUE : FALSE;
	dfirst = dirfirst;
	myfwin->dirfirst = dirfirst;
	count = list.count();
	tmplist.clear();
	for(i = 0; i < count; i++) {
		fileobj = list.at(i);
		tmplist.inSort(fileobj);
	}
	list.setAutoDelete(FALSE);
	list = tmplist;
	list.setAutoDelete(TRUE);
	
	viewPopUp->removeItemAt(0);
	if(dirfirst)
		viewPopUp->insertItem("Normal Sort", this, SLOT(changeNormalDirFirstSort()), 0, 0, 0);
	else
		viewPopUp->insertItem("Dir First Sort", this, SLOT(changeNormalDirFirstSort()), 0, 0, 0);
	QResizeEvent e = QResizeEvent(this->size(), this->size());
	this->event(&e);
}

void ObjsFrame::newFolder()
{
	((mfinderapp *)qApp)->newFileObject(myfwin, TRUE);
}

void ObjsFrame::newFile()
{
	((mfinderapp *)qApp)->newFileObject(myfwin, FALSE);
}

void ObjsFrame::findFile()
{
	((mfinderapp *)qApp)->findFile(myPath);		
}

void ObjsFrame::openSelectedObjects()
{
	((mfinderapp *)qApp)->openSelectedObjectsOfWin(myfwin, FALSE);		
}

void ObjsFrame::newXTermAtMyPath()
{
	((mfinderapp *)qApp)->StartXTerm(myfwin);
}

void ObjsFrame::makePopUp()
{
	char buff[MAX_PATH_LEN] = "";
	char apath[MAX_PATH_LEN] = "";
	char *token, *delims = "/";
	int count = 0;
	
	filePopUp = new Popup();
	filePopUp->insertItem("Open Selected", this, SLOT(openSelectedObjects()));
	filePopUp->insertItem("Open At...", (mfinderapp *)qApp, SLOT(openDirAt()));
	filePopUp->insertItem("Close", myfwin, SLOT(closeme()));
	filePopUp->insertItem("New Folder", this, SLOT(newFolder()));
	filePopUp->insertItem("New File", this, SLOT(newFile()));
	filePopUp->insertItem("Find ...", this, SLOT(findFile()));

	editPopUp = new Popup();
	editPopUp->insertItem("Select All", this, SLOT(selectAllFileObjects()));
	editPopUp->insertItem("Start xterm", this, SLOT(newXTermAtMyPath()));
	
	viewPopUp = new Popup();
	if(dirfirst)
		viewPopUp->insertItem("Normal Sort", this, SLOT(changeNormalDirFirstSort()));
	else
		viewPopUp->insertItem("Dir First Sort", this, SLOT(changeNormalDirFirstSort()));
	if(dotfiles)
		viewPopUp->insertItem("Hide Dot Files", myfwin, SLOT(changeHideShowDotFiles()));
	else
		viewPopUp->insertItem("Show Dot Files", myfwin, SLOT(changeHideShowDotFiles()));

	viewPopUp->insertItem("by Icons", myfwin, SLOT(changeViewByIcons()));
	viewPopUp->insertItem("by Small Icons", myfwin, SLOT(changeViewBySmallIcons()));
	viewPopUp->insertItem("by Name", myfwin, SLOT(changeViewByName()));

	configPopUp = new Popup();
	configPopUp->insertItem("Base Icons", (mfinderapp *)qApp, SLOT(configureBaseIcons()));
	configPopUp->insertItem("Files Icons", (mfinderapp *)qApp, SLOT(configureFilesIcons()));

	specialPopUp = new Popup();
	specialPopUp->insertItem("Configure", configPopUp);
	specialPopUp->insertItem("WMFinder Info", (mfinderapp *)qApp, SLOT(showInfo()));
 
	globalPopUp = new Popup();
	globalPopUp->insertItem("File", filePopUp);
	globalPopUp->insertItem("Edit", editPopUp);
	globalPopUp->insertItem("View", viewPopUp);
	globalPopUp->insertItem("Special", specialPopUp);
	globalPopUp->insertItem("Quit", (mfinderapp *)qApp, SLOT(quit()));
	
}

FileObj* ObjsFrame::fileObjAtPath(char *apath)
{
	FileObj *fileobj = NULL;
	bool found = FALSE;

	for(int i = 0; i < list.count(); i++) {
		fileobj = list.at(i);
		if(strcmp(fileobj->myFullPath, apath) == 0) {
			found = TRUE;
			break;
		}
	}
	if(!found)
		fileobj = NULL;
	return fileobj;
}

void ObjsFrame::unselectOtherFileObjects(char *objname)
{
	int i, count;
	FileObj *fileobj;

	count = list.count();
	if(!isShiftClick && !isDragShiftClick) {
		for(i = 0; i < count; i++) {
			fileobj = list.at(i);
			if(strcmp(fileobj->myName, objname) != 0) {
				if(fileobj->isSelect && !fileobj->isOpen)
					fileobj->Unselect(true);
				if(fileobj->isSelect && fileobj->isOpen)
					fileobj->UnselOpen(true);
			}
		}
	}
	((mfinderapp *)qApp)->unselectFileObjectsOfOtherWins(myPath);
}

void ObjsFrame::selectAllFileObjects()
{
	int i, count;
	FileObj *fileobj;

	isShiftClick = TRUE;
	count = list.count();
	for(i = 0; i < count; i++) {
		fileobj = list.at(i);
		if(fileobj->isOpen)
			fileobj->SelOpen(true);
		else
			fileobj->Select(true);
	}
	isShiftClick = FALSE;
}

void ObjsFrame::unselectAllFileObjects()
{
	int i, count;
	FileObj *fileobj;
	
	count = list.count();
	for(i = 0; i < count; i++) {
		fileobj = list.at(i);
		if(fileobj->isSelect && !fileobj->isOpen)
			fileobj->Unselect(true);
		if(fileobj->isSelect && fileobj->isOpen)
			fileobj->UnselOpen(true);
	}
}

QList<FileObj> ObjsFrame::selectedObjects()
{
	QList<FileObj> sellist;
	int i, count;
	FileObj *fileobj;

	sellist.setAutoDelete(FALSE);
	count = list.count();
	for(i = 0; i < count; i++) {
		fileobj = list.at(i);
		if(fileobj->isSelect)
			sellist.append(fileobj);
	}	
	return sellist;
}

QStrList ObjsFrame::makeUrlList()
{
	QStrList urlist;
	int i, count;
	FileObj *fileobj;

	urlist = QStrList(TRUE);
	count = list.count();
	for(i = 0; i < count; i++) {
		fileobj = list.at(i);
		if(fileobj->isSelect)
			urlist.append(fileobj->myFullPath);
	}	
	return urlist;
}

void ObjsFrame::resizeEvent(QResizeEvent *e)
{
	FileObj *fileobj;
	int PosCount, objsSpace, objswidth, HObjs, VObjs, i, FileCount;

	if(viewtype == BYICONS) {
 		objsSpace = 20;
		objswidth = 60;
		i = 0;
		HObjs = objsSpace;
		VObjs = 4;
		PosCount = 0;
		FileCount = list.count();

		while(i < FileCount) {
			while(1) {
				fileobj = list.at(i);
				if(PosCount > 0)
					HObjs = HObjs + objswidth + objsSpace;
				if(HObjs > width() - objswidth) 
					break;

				fileobj->move(HObjs, VObjs);

				i++;						 	
				PosCount++;
				if(i == FileCount) 
			 		break;
			}
			HObjs = objsSpace;
			PosCount = 0;
			VObjs = VObjs + 75;
		}
	} else {
 		objsSpace = 10;
		objswidth = 90;
		i = 0;
		HObjs = objsSpace;
		VObjs = 4;
		PosCount = 0;
		FileCount = list.count();

		while(i < FileCount) {
			while(1) {
				fileobj = list.at(i);
				if(PosCount > 0)
					HObjs = HObjs + objswidth + objsSpace;
				if(HObjs > width() - objswidth) 
					break;

				fileobj->move(HObjs, VObjs);

				if((HObjs + fileobj->width() + 5 + fileobj->label->width()) 
															> (HObjs + objswidth + objsSpace))
					HObjs = HObjs + objswidth + objsSpace;
				i++;						 	
				PosCount++;
				if(i == FileCount) 
			 		break;
			}
			HObjs = objsSpace;
			PosCount = 0;
			VObjs = VObjs + 20;
		}
	}
	emit resizeDone(VObjs);	
}

void ObjsFrame::mousePressEvent(QMouseEvent *e)
{
	if(e->button() & RightButton) {
		globalPopUp->popup(mapToGlobal(e->pos()));
		isShiftClick = FALSE;
	}
	
	if(e->button() & LeftButton) {
		if(e->state() & ShiftButton) {
			isShiftClick = TRUE;
		} else {
			isShiftClick = FALSE;
			((mfinderapp *)qApp)->selectFileObjectFatherOfWindow(myPath, FALSE);
		}	
	}
}

void ObjsFrame::mouseMoveEvent(QMouseEvent *)
{
	QRect r;

	if(!isSelDragging) {
		pBegSel = mapFromGlobal(QCursor::pos());
		isSelDragging = TRUE;
		isDragShiftClick = TRUE;
	} else {
		QPoint cpnt = mapFromGlobal(QCursor::pos());
		if(pBegSel.x() < cpnt.x()) {
			SelRect.setLeft(pBegSel.x());
			SelRect.setRight(cpnt.x());
		} else {
			SelRect.setLeft(cpnt.x());
			SelRect.setRight(pBegSel.x());
		}
		if(pBegSel.y() < cpnt.y()) {
			SelRect.setTop(pBegSel.y());
			SelRect.setBottom(cpnt.y());
		} else {
			SelRect.setTop(cpnt.y());
			SelRect.setBottom(pBegSel.y());
		}
		
		for(int i = 0; i < list.count(); i++) {
			FileObj *filobj = list.at(i);
			QPoint cp(filobj->x()+filobj->width()/2,
									filobj->y()+filobj->height()/2);
			if(SelRect.contains(cp)) {
				if(!filobj->isSelect) {
					if(filobj->isOpen)
						filobj->SelOpen(TRUE);
					else
						filobj->Select(TRUE);
				}
			} else {
				if(!isShiftClick) {
					if(filobj->isSelect) {
						if(filobj->isOpen)
							filobj->UnselOpen(TRUE);
						else
							filobj->Unselect(TRUE);
					}	
				}
			}

			r = QRect(filobj->mapToParent(filobj->rect().topLeft()), 
								 filobj->mapToParent(filobj->rect().bottomRight()));
			if(SelRect.intersects(r)) {
				if(!SelRect.contains(r)) {
	 				QRect localrect = QRect(filobj->mapFromParent(SelRect.intersect(r).topLeft()), 
													filobj->mapFromParent(SelRect.intersect(r).bottomRight()));
					filobj->drawSelRect(localrect, TRUE);
				} else {
					filobj->drawSelRect(QRect(0,0,0,0), FALSE);
				}
			} else {
				filobj->drawSelRect(QRect(0,0,0,0), FALSE);
			}

		}
		update();
	}
}

void ObjsFrame::mouseReleaseEvent(QMouseEvent *)
{
	if(isSelDragging) {
		isSelDragging = FALSE;
		isDragShiftClick = FALSE;
		isShiftClick = FALSE;
		update();
		for(int i = 0; i < list.count(); i++) {
			FileObj *filobj = list.at(i);
			filobj->drawSelRect(QRect(0,0,0,0), FALSE);
		}
	}
}

void ObjsFrame::dragEnterEvent(QDragEnterEvent *e)
{
	if(QUrlDrag::canDecode(e)) {
		e->accept();
		isDragTarget = TRUE;
		update();	  
	}
}

void ObjsFrame::dragLeaveEvent(QDragLeaveEvent *e)
{
	isDragTarget = FALSE;
	update();	  
}

void ObjsFrame::dropEvent(QDropEvent *e)
{
	QStrList droplist;

	e->accept();
	QUrlDrag::decode(e, droplist);
	((mfinderapp *)qApp)->moveOrCopy(droplist, myPath, this);
}

void ObjsFrame::paintEvent(QPaintEvent *)
{
	QPainter p;
	
	if(isSelDragging) {
		p.begin(this);
		p.drawRect(SelRect);
		p.end();
	}
	
	if(isDragTarget) {
		p.begin(this);
		p.setPen(QPen(QColor(150, 150, 255), 2));
		p.drawRect(1, 1, width()-2, height()-2);
		p.end();
	}
}
