#include "objslistframe.h"
#include "mfinderapp.h"
#include <qpainter.h> 
#include <qdrawutl.h>

extern bool dfirst;
extern bool iview;

ObjsListFrame::ObjsListFrame(QWidget *parent, const char *path, 
													bool dirfirst, bool dotfiles)
			  	  : QWidget(parent, path),
			  	    QDropSite(this)
{
	QColor x = QColor(230, 230, 230);
	QColorGroup CGnorm = QColorGroup(black, x, x.light(), x.dark(), x.dark(120), black, x);
	setPalette(QPalette(CGnorm, CGnorm, CGnorm));
	
	strncpy(myPath, path, MAX_PATH_LEN);
	this->dirfirst = dirfirst;
	this->dotfiles = dotfiles;
	dfirst = dirfirst;
	myfwin = (MFinderWin *)parent;
	list.setAutoDelete(TRUE);
	allobjslist.setAutoDelete(FALSE);
	isShiftClick = FALSE;
	isDragShiftClick = FALSE;
	isSelDragging = FALSE;
	isDragTarget = FALSE;
	labelsshift = 0;
	
	makeFileObjects();
	makePopUp();
}

ObjsListFrame::~ObjsListFrame()
{
	FileListObj *fileobj;

	allobjslist.clear();
	for(int i = 0; i < list.count(); i++) {
		fileobj = list.at(0);
		list.removeRef(fileobj);
	}
}

void ObjsListFrame::makeFileObjects()
{
	struct dirent **namelist;
	struct stat listStat;
	char filepath[MAX_PATH_LEN];
	char confdir[MAX_PATH_LEN];
	int m, a, i, type;
	bool fileok;

	iview = FALSE;

	sprintf(confdir, "%s/GNUstep/Library/xwfinder", getenv("HOME"));

  	m = scandir(myPath, &namelist, 0, alphasort);
	for(a = 0; a <= m - 1; a++) {
		fileok = FALSE;
		if(strcmp(namelist[a]->d_name, ".") != 0 
								&& strcmp(namelist[a]->d_name, "..") != 0)
			fileok = TRUE;
		if(!dotfiles)	
			if(namelist[a]->d_name[0] == '.')
				fileok = FALSE;

		if(fileok) {
			if(strcmp(myPath, "/") != 0)
				sprintf(filepath, "%s/%s", myPath, namelist[a]->d_name);
			else
				sprintf(filepath, "%s%s", myPath, namelist[a]->d_name);

   		lstat(filepath, &listStat);
			if(S_ISREG(listStat.st_mode)) {
				if(listStat.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH))
					type = FILEXE;
				else
					type = FIL;
			}
			if(S_ISDIR(listStat.st_mode)) 
				type = DR;
			if(S_ISLNK(listStat.st_mode)) {
				if(chdir(filepath) != 0) {
					if(listStat.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH)) {
						type = FILEXELNK;
					} else {
						type = FILLNK;
					}
				} else {
					chdir(confdir);
					type = DRLNK;
				}
			}		
			if(S_ISCHR(listStat.st_mode)) 
				type = CHRDV;		
			if(S_ISBLK(listStat.st_mode)) 
				type = BLCKDV;	
			if(S_ISFIFO(listStat.st_mode)) 
				type = FIF;		
			if(S_ISSOCK(listStat.st_mode)) 
				type = SCKT;		

			list.inSort(new FileListObj(this, myPath, namelist[a]->d_name, type, 0, FALSE, 0L));
			allobjslist.append(list.current());
		}
	}
	emit updateInfo();
}

void ObjsListFrame::updateContents()
{
	struct dirent **namelist;
	int m, count, newcount;
	bool fileok, found;
	FileListObj *fileobj;	
	QList<char> newlist; 
	QList<char> difflistp;
	QList<char> difflistm;
	
	newlist.setAutoDelete(TRUE);
	difflistp.setAutoDelete(TRUE);
	difflistm.setAutoDelete(TRUE);
	iview = FALSE;
	
	for(int i = 0; i < allobjslist.count(); i++) {
		fileobj = allobjslist.at(i);
		if(fileobj->isOpenLocal)
			fileobj->updateContents();
	}
	
  	m = scandir(myPath, &namelist, 0, alphasort);
	for(int i = 0; i <= m - 1; i++) {
		fileok = FALSE;
		if(strcmp(namelist[i]->d_name, ".") != 0 
								&& strcmp(namelist[i]->d_name, "..") != 0)
			fileok = TRUE;
		if(!dotfiles)	
			if(namelist[i]->d_name[0] == '.')
				fileok = FALSE;

		if(fileok) {
			newlist.append(new char[strlen(namelist[i]->d_name)+1]);
			strcpy(newlist.current(), namelist[i]->d_name);
		}
	}
	
	for(int i = 0; i < newlist.count(); i++) {
		found = FALSE;
		for(int j = 0; j < list.count(); j++)
			if(strcmp(newlist.at(i), list.at(j)->myName) == 0)
				found = TRUE;
		if(!found) {
			difflistp.append(new char[strlen(newlist.at(i))+1]);
			strcpy(difflistp.current(), newlist.at(i));
		}
	}
	
	for(int i = 0; i < list.count(); i++) {
		found = FALSE;
		for(int j = 0; j < newlist.count(); j++)
			if(strcmp(list.at(i)->myName, newlist.at(j)) == 0)
				found = TRUE;
		if(!found) {
			difflistm.append(new char[strlen(list.at(i)->myName)+1]);
			strcpy(difflistm.current(), list.at(i)->myName);
		}
	}
	
	if(difflistp.count() > 0)
		for(int i = 0; i < difflistp.count(); i++)
			addAnObject(difflistp.at(i));
	
	if(difflistm.count() > 0) {
		count = list.count();
		for(int i = 0; i < count; i++) {
			fileobj = list.at(i);
			for(int j = 0; j < difflistm.count(); j++) {
				if(strcmp(fileobj->myName, difflistm.at(j)) == 0) {
					allobjslist.removeRef(fileobj);
					list.removeRef(fileobj);
					count--;
					i--;
				}
			}
		}
	}
	
	newlist.clear();	
	difflistp.clear();	
	difflistm.clear();
		
	QResizeEvent e = QResizeEvent(this->size(), this->size());
	this->event(&e);
	emit updateInfo();
}

void ObjsListFrame::removeAnObject(FileListObj *obj)
{
	if(obj->isOpenLocal) {
		for(int i = 0; i < obj->list.count(); i++)
			allobjslist.removeRef(obj->list.at(i));
	}
	allobjslist.removeRef(obj);
	for(int i = 0; i < list.count(); i++)
		if(list.at(i)->isOpenLocal)
			list.at(i)->removeAnObject(obj);
	list.removeRef(obj);
	QResizeEvent e = QResizeEvent(this->size(), this->size());
	this->event(&e);
}

void ObjsListFrame::addAnObject(char *nameobj)
{
	char filepath[MAX_PATH_LEN];
	char confdir[MAX_PATH_LEN];
	struct stat listStat;
	int type;
	
	iview = FALSE;
	dfirst = dirfirst;
	sprintf(confdir, "%s/GNUstep/Library/xwfinder", getenv("HOME"));
	if(strcmp(myPath, "/") != 0)
		sprintf(filepath, "%s/%s", myPath, nameobj);
	else
		sprintf(filepath, "%s%s", myPath, nameobj);

   lstat(filepath, &listStat);
	if(S_ISREG(listStat.st_mode)) {
		if(listStat.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH))
			type = FILEXE;
		else
			type = FIL;
	}
	if(S_ISDIR(listStat.st_mode)) 
		type = DR;
	if(S_ISLNK(listStat.st_mode)) {
		if(chdir(filepath) != 0) {
			if(listStat.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH)) {
				type = FILEXELNK;
			} else {
				type = FILLNK;
			}
		} else {
			chdir(confdir);
			type = DRLNK;
		}
	}		
	if(S_ISCHR(listStat.st_mode)) 
		type = CHRDV;		
	if(S_ISBLK(listStat.st_mode)) 
		type = BLCKDV;	
	if(S_ISFIFO(listStat.st_mode)) 
		type = FIF;		
	if(S_ISSOCK(listStat.st_mode)) 
		type = SCKT;		

	list.inSort(new FileListObj(this, myPath, nameobj, type, 0, FALSE, 0L));
	allobjslist.append(list.current());
}

void ObjsListFrame::newFolder()
{
	((mfinderapp *)qApp)->newFileObject(myfwin, TRUE);
}

void ObjsListFrame::newFile()
{
	((mfinderapp *)qApp)->newFileObject(myfwin, FALSE);
}

void ObjsListFrame::findFile()
{
	((mfinderapp *)qApp)->findFile(myPath);		
}

void ObjsListFrame::openSelectedObjects()
{
	((mfinderapp *)qApp)->openSelectedObjectsOfWin(myfwin, FALSE);		
}

void ObjsListFrame::newXTermAtMyPath()
{
	((mfinderapp *)qApp)->StartXTerm(myfwin);
}

void ObjsListFrame::makePopUp()
{
	char buff[MAX_PATH_LEN] = "";
	char apath[MAX_PATH_LEN] = "";
	char *token, *delims = "/";
	int count = 0;

	filePopUp = new Popup();
	filePopUp->insertItem("Open Selected", this, SLOT(openSelectedObjects()));
	filePopUp->insertItem("Open At...", (mfinderapp *)qApp, SLOT(openDirAt()));
	filePopUp->insertItem("Close", myfwin, SLOT(closeme()));
	filePopUp->insertItem("New Folder", this, SLOT(newFolder()));
	filePopUp->insertItem("New File", this, SLOT(newFile()));
	filePopUp->insertItem("Find ...", this, SLOT(findFile()));
	
	editPopUp = new Popup();
	editPopUp->insertItem("Select All", this, SLOT(selectAllFileObjects()));
	editPopUp->insertItem("Start xterm", this, SLOT(newXTermAtMyPath()));
	
	viewPopUp = new Popup();
	if(dirfirst)
		viewPopUp->insertItem("Normal Sort", this, SLOT(changeNormalDirFirstSort()));
	else
		viewPopUp->insertItem("Dir First Sort", this, SLOT(changeNormalDirFirstSort()));
	if(dotfiles)
		viewPopUp->insertItem("Hide Dot Files", myfwin, SLOT(changeHideShowDotFiles()));
	else
		viewPopUp->insertItem("Show Dot Files", myfwin, SLOT(changeHideShowDotFiles()));

	viewPopUp->insertItem("by Icons", myfwin, SLOT(changeViewByIcons()));
	viewPopUp->insertItem("by Small Icons", myfwin, SLOT(changeViewBySmallIcons()));
	viewPopUp->insertItem("by Name", myfwin, SLOT(changeViewByName()));

	configPopUp = new Popup();
	configPopUp->insertItem("Base Icons", (mfinderapp *)qApp, SLOT(configureBaseIcons()));
	configPopUp->insertItem("Files Icons", (mfinderapp *)qApp, SLOT(configureFilesIcons()));

	specialPopUp = new Popup();
	specialPopUp->insertItem("Configure", configPopUp);
#ifdef __Linux__
	specialPopUp->insertItem("Process Manager", (mfinderapp *)qApp, SLOT(openProcessManager()));
#endif
	specialPopUp->insertItem("WMFinder Info", (mfinderapp *)qApp, SLOT(showInfo()));
 
	globalPopUp = new Popup();
	globalPopUp->insertItem("File", filePopUp);
	globalPopUp->insertItem("Edit", editPopUp);
	globalPopUp->insertItem("View", viewPopUp);
	globalPopUp->insertItem("Special", specialPopUp);
	globalPopUp->insertItem("Quit", (mfinderapp *)qApp, SLOT(quit()));
	
}

FileListObj* ObjsListFrame::fileObjAtPath(char *apath)
{
	FileListObj *fileobj;
	int i, count;
	bool found = FALSE;
	
	count = list.count();
	for(i = 0; i < count; i++) {
		fileobj = list.at(i);
		if(strcmp(fileobj->myFullPath, apath) == 0) {
			found = TRUE;
			break;
		}
	}
	if(!found)
		fileobj = NULL;
	return fileobj;
}

FileListObj* ObjsListFrame::fileObjInTreeAtPath(char *apath)
{
	FileListObj *fileobj = NULL, *tpmobj = NULL;
	int i, count;
	
	count = list.count();
	for(i = 0; i < count; i++) {
		tpmobj = list.at(i);
		if(strcmp(tpmobj->myFullPath, apath) == 0) {
			fileobj = tpmobj;
			return fileobj;
		} else {
			if(tpmobj->isOpenLocal)
				fileobj = tpmobj->discendentFileObjectAtPath(apath);
		}
	}
	return fileobj;
}

void ObjsListFrame::unselectOtherFileObjects(char *objfullpath, bool inotherwins)
{
	int i, count;
	FileListObj *fileobj;

	if(!isShiftClick && !isDragShiftClick) {
		count = list.count();
		for(i = 0; i < count; i++) {
			fileobj = list.at(i);
			if(strcmp(fileobj->myFullPath, objfullpath) != 0) {
				if(fileobj->isSelect && !fileobj->isOpen)
					fileobj->Unselect(true);
				if(fileobj->isSelect && fileobj->isOpen)
					fileobj->UnselOpen(true);
			}
			if(fileobj->isOpenLocal)
				fileobj->unselectDiscendentFileObjects(objfullpath);
		}
	}
	if(inotherwins)
		((mfinderapp *)qApp)->unselectFileObjectsOfOtherWins(myPath);
}

void ObjsListFrame::selectAllFileObjects()
{
	int i, count;
	FileListObj *fileobj;

	isShiftClick = TRUE;
	count = list.count();
	for(i = 0; i < count; i++) {
		fileobj = list.at(i);
		if(fileobj->isOpen)
			fileobj->SelOpen(true);
		else
			fileobj->Select(true);
	}
	isShiftClick = FALSE;
}

void ObjsListFrame::unselectAllFileObjects()
{
	int i, count;
	FileListObj *fileobj;
	
	count = list.count();
	for(i = 0; i < count; i++) {
		fileobj = list.at(i);
		if(fileobj->isSelect && !fileobj->isOpen)
			fileobj->Unselect(true);
		if(fileobj->isSelect && fileobj->isOpen)
			fileobj->UnselOpen(true);
		if(fileobj->isOpenLocal)
			fileobj->unselectDiscendentFileObjects("/");		
	}
}

QList<FileListObj> ObjsListFrame::selectedObjects()
{
	QList<FileListObj> sellist;
	int i, count;
	FileListObj *fileobj;

	sellist.setAutoDelete(FALSE);
	count = list.count();
	for(i = 0; i < count; i++) {
		fileobj = list.at(i);
		if(fileobj->isSelect)
			sellist.append(fileobj);
		if(fileobj->isOpenLocal)
			addToSelectedList(&sellist, fileobj);
	}	
	return sellist;
}

void ObjsListFrame::addToSelectedList(QList<FileListObj> *alist, FileListObj *anobj)
{
	int i, count;
	FileListObj *fileobj;
	
	count = anobj->list.count();
	for(i = 0; i < count; i++) {
		fileobj = anobj->list.at(i);
		if(fileobj->isSelect) {
			alist->append(fileobj);
		}
		if(fileobj->isOpenLocal)
			addToSelectedList(alist, fileobj);	
	}
}

QStrList ObjsListFrame::makeUrlList()
{
	QStrList urlist;
	int i, count;
	FileListObj *fileobj;

	urlist = QStrList(TRUE);
	count = list.count();
	for(i = 0; i < count; i++) {
		fileobj = list.at(i);
		if(fileobj->isSelect)
			urlist.append(fileobj->myFullPath);
		if(!fileobj->isSelect && fileobj->isOpenLocal)
			addToUrlList(&urlist, fileobj);
	}	
	return urlist;
}

void ObjsListFrame::addToUrlList(QStrList *alist, FileListObj *anobj)
{
	int i, count;
	FileListObj *fileobj;
	
	count = anobj->list.count();
	for(i = 0; i < count; i++) {
		fileobj = anobj->list.at(i);
		if(fileobj->isSelect) {
			alist->append(fileobj->myFullPath);
		}
		if(!fileobj->isSelect && fileobj->isOpenLocal)
			addToUrlList(alist, fileobj);	
	}
}

void ObjsListFrame::changeNormalDirFirstSort()
{
	FileListObj *fileobj;
	QList<FileListObj> tmplist; 	

	iview = FALSE;

	dirfirst = (!dirfirst) ? TRUE : FALSE;
	dfirst = dirfirst;
	myfwin->dirfirst = dirfirst;
	tmplist.clear();
	for(int i = 0; i < list.count(); i++) {
		fileobj = list.at(i);
		tmplist.inSort(fileobj);
		if(fileobj->isOpenLocal)
			fileobj->changeNormalDirFirstSort();		
	}
	list.setAutoDelete(FALSE);
	list = tmplist;
	list.setAutoDelete(TRUE);
	
	viewPopUp->removeItemAt(0);
	if(dirfirst)
		viewPopUp->insertItem("Normal Sort", this, SLOT(changeNormalDirFirstSort()), 0, 0, 0);
	else
		viewPopUp->insertItem("Dir First Sort", this, SLOT(changeNormalDirFirstSort()), 0, 0, 0);
	QResizeEvent e = QResizeEvent(this->size(), this->size());
	this->event(&e);
}

void ObjsListFrame::resizeEvent(QResizeEvent *e)
{
	FileListObj *fileobj;

	VObjs = 4;
	for(int i = 0; i < list.count(); i++) 
		objectPosition(list.at(i));

	labelsshift = 0;
	for(int i = 0; i < allobjslist.count(); i++) {
		int sp = allobjslist.at(i)->leftspace 
				+ (allobjslist.at(i)->sep * allobjslist.at(i)->movx) 
				+ allobjslist.at(i)->arrowlabel->width() 
				+ allobjslist.at(i)->icona.width() + 10 
				+ allobjslist.at(i)->label->width() + allobjslist.at(i)->sep;
		if(labelsshift < sp)
			labelsshift = sp;
	}

	for(int i = 0; i < allobjslist.count(); i++)
		allobjslist.at(i)->shiftLabels();

	emit resizeDone(VObjs);	
}

void ObjsListFrame::objectPosition(FileListObj *anobj)
{
	int i, localcount;

	anobj->move(2, VObjs);
	VObjs = VObjs + anobj->hspace;

	if(anobj->isOpenLocal) {
		localcount = anobj->list.count();
		for(i = 0; i < localcount; i++) 
			objectPosition(anobj->list.at(i));
	}
}

void ObjsListFrame::mousePressEvent(QMouseEvent *e)
{
	if(e->button() & RightButton) {
		globalPopUp->popup(mapToGlobal(e->pos()));
		isShiftClick = FALSE;
	}
	
	if(e->button() & LeftButton) {
		if(e->state() & ShiftButton) {
			isShiftClick = TRUE;
		} else {
			isShiftClick = FALSE;
			((mfinderapp *)qApp)->selectFileObjectFatherOfWindow(myPath, FALSE);
		}	
	}
}

void ObjsListFrame::mouseMoveEvent(QMouseEvent *)
{
	QRect r;
	
	if(!isSelDragging) {
		pBegSel = mapFromGlobal(QCursor::pos());
		isSelDragging = TRUE;
		isDragShiftClick = TRUE;
	} else {
		QPoint cpnt = mapFromGlobal(QCursor::pos());
		if(pBegSel.x() < cpnt.x()) {
			SelRect.setLeft(pBegSel.x());
			SelRect.setRight(cpnt.x());
		} else {
			SelRect.setLeft(cpnt.x());
			SelRect.setRight(pBegSel.x());
		}
		if(pBegSel.y() < cpnt.y()) {
			SelRect.setTop(pBegSel.y());
			SelRect.setBottom(cpnt.y());
		} else {
			SelRect.setTop(cpnt.y());
			SelRect.setBottom(pBegSel.y());
		}
		
		for(int i = 0; i < allobjslist.count(); i++) {
			FileListObj *filobj = allobjslist.at(i);
			QPoint cp(filobj->x()+100,
									filobj->y()+filobj->height()/2);		
			if(SelRect.contains(cp)) {
				if(!filobj->isSelect) {
					if(filobj->isOpen)
						filobj->SelOpen(TRUE);
					else
						filobj->Select(TRUE);
				}
			} else {
				if(!isShiftClick) {
					if(filobj->isSelect) {
						if(filobj->isOpen)
							filobj->UnselOpen(TRUE);
						else
							filobj->Unselect(TRUE);
					}	
				}
			}

			r = QRect(filobj->mapToParent(filobj->rect().topLeft()), 
								 filobj->mapToParent(filobj->rect().bottomRight()));
			if(SelRect.intersects(r)) {
				if(!SelRect.contains(r)) {
	 				QRect localrect = QRect(filobj->mapFromParent(SelRect.intersect(r).topLeft()), 
													filobj->mapFromParent(SelRect.intersect(r).bottomRight()));
					filobj->drawSelRect(localrect, TRUE);
				} else {
					filobj->drawSelRect(QRect(0,0,0,0), FALSE);
				}
			} else {
				filobj->drawSelRect(QRect(0,0,0,0), FALSE);
			}

		}
		update();
	}
}

void ObjsListFrame::mouseReleaseEvent(QMouseEvent *)
{
	if(isSelDragging) {
		isSelDragging = FALSE;
		isDragShiftClick = FALSE;
		isShiftClick = FALSE;
		update();
		for(int i = 0; i < allobjslist.count(); i++) {
			FileListObj *filobj = allobjslist.at(i);
			filobj->drawSelRect(QRect(0,0,0,0), FALSE);
		}
	}
}

void ObjsListFrame::dragEnterEvent(QDragEnterEvent *e)
{
	if(QUrlDrag::canDecode(e)) {
		e->accept();
		isDragTarget = TRUE;
		update();	
	}
}

void ObjsListFrame::dragLeaveEvent(QDragLeaveEvent *e)
{
	isDragTarget = FALSE;
	update();	  
}

void ObjsListFrame::dropEvent(QDropEvent *e)
{
	QStrList droplist;

	e->accept();
	QUrlDrag::decode(e, droplist);
	((mfinderapp *)qApp)->moveOrCopy(droplist, myPath, this);
}

void ObjsListFrame::paintEvent(QPaintEvent *)
{
	QPainter p;
	
	p.begin(this);
	if(isSelDragging) {
		p.drawRect(SelRect);
	}
	if(isDragTarget) {
		p.setPen(QPen(QColor(150, 150, 255), 2));
		p.drawRect(1, 1, width()-1, height()-1);
	}
	p.end();
}
