#include "mfinderapp.h"
#include "functions.h"
#include "xworkspace.h"

#define MAX_PROPERTY_SIZE 32*1024

char connerr[256] = "";

void mfinderapp::startXWSConnection(long win)
{
	if(!XWopenXDisplay()) {
		sprintf(connerr, "libXWorkspace: error opening display\n");
		connectionError(connerr);
	}
	if(!XWconnectToTheServer("xwfinder", getpid(), win)) {
		sprintf(connerr, "error connecting to the server\n");
		connectionError(connerr);
	}
	if(XWverifyConnection()) {
		XWsendQueryToTheServer(NOTIFICATIONREQ, XWDidLaunchApplicationNotification);
		XWsendQueryToTheServer(NOTIFICATIONREQ, XWDidPerformFileOperationNotification);
		XWsendQueryToTheServer(NOTIFICATIONREQ, XWFileSystemDidChangeNotification);
		XWsendQueryToTheServer(NOTIFICATIONREQ, XWDidSetFileAttributesNotification);
	}
	printf("CONNECTED\nservername: %s\nserverpid: %i\nserverwin: %li\n", 
													servername, serverpid, serverwin);
}

void mfinderapp::verifyXWSConnection()
{
	if(!XWverifyConnection()) {
		sprintf(connerr, "connection to the server closed\n");
		connectionError(connerr);
	}
}

void mfinderapp::sendQueryToXWServer(int key, char *data)
{
	XWsendQueryToTheServer(key, data);
}

void mfinderapp::parseServerReply(XEvent *e)
{
	Atom atom, type_ret;
	int fmt_ret, i, qref;
	unsigned long nitems_ret;
	unsigned long bytes_after_ret;
	char *serverdata;
	char *token, *delims = "\n";
	char srcpath[MAX_PATH_LEN] = "", destpath[MAX_PATH_LEN] = "";
	char operation[128] = "";
	FileObj* fobj = NULL;
	FileListObj *flobj = NULL;
	MFinderWin *winToUpdate;
	FileListObj *listObjToUpdate;	
	DeskObj *deskObjToUpdate;
	
	atom = XInternAtom(dspl, XGetAtomName(dspl, e->xclient.data.l[0]), 0);

	XGetWindowProperty(dspl, appRootWin, 
					  atom, 0L, MAX_PROPERTY_SIZE, 0, XA_STRING,
					  &type_ret, &fmt_ret, &nitems_ret, &bytes_after_ret,
					  (unsigned char**)&serverdata);

	token = strtok(serverdata, delims);
	
	if(strcmp(token, XWFileSystemDidChangeNotification) == 0) {
		token = strtok(NULL, delims);
		strcpy(operation, token);

// ------------------------ Create ------------------------
		if(strcmp(operation, XWCreateFileOperation) == 0
					|| strcmp(operation, XWCreateDirOperation) == 0) {
			token = strtok(NULL, delims);
			strcpy(destpath, token);
			
			winToUpdate = winAtPath(destpath);
			if(winToUpdate) {
				if(winToUpdate->viewtype == BYICONS || winToUpdate->viewtype == BYSMICONS)
					winToUpdate->objsframe->updateContents();
				else
					winToUpdate->objslistframe->updateContents();
			}

			listObjToUpdate = fileListObjAtPath(destpath);
			if(listObjToUpdate)
				listObjToUpdate->myobjsframe->updateContents();
		}
		
// ------------------------- Move -------------------------
		if(strcmp(operation, XWMoveOperation) == 0) {
			token = strtok(NULL, delims);
			strcpy(srcpath, token);
			token = strtok(NULL, delims);
			strcpy(destpath, token);

			winToUpdate = winAtPath(srcpath);
			if(winToUpdate) {
				if(winToUpdate->viewtype == BYICONS || winToUpdate->viewtype == BYSMICONS)
					winToUpdate->objsframe->updateContents();
				else
					winToUpdate->objslistframe->updateContents();
			}
			winToUpdate = winAtPath(destpath);
			if(winToUpdate) {
				if(winToUpdate->viewtype == BYICONS || winToUpdate->viewtype == BYSMICONS)
					winToUpdate->objsframe->updateContents();
				else
					winToUpdate->objslistframe->updateContents();
			}

			listObjToUpdate = fileListObjAtPath(srcpath);
			if(listObjToUpdate) 
				listObjToUpdate->myobjsframe->updateContents();
			
			listObjToUpdate = fileListObjAtPath(destpath);
			if(listObjToUpdate)
				listObjToUpdate->myobjsframe->updateContents();

			propagatePathChangeInWins(srcpath, destpath);

			
			// Aggiungere le custom icons, l'editor ecc...		
		}
		
// ----------------------- Copy -----------------------		
		if(strcmp(operation, XWCopyOperation) == 0) {
			token = strtok(NULL, delims);
			token = strtok(NULL, delims);
			strcpy(destpath, token);

			winToUpdate = winAtPath(destpath);
			if(winToUpdate) {
				if(winToUpdate->viewtype == BYICONS || winToUpdate->viewtype == BYSMICONS)
					winToUpdate->objsframe->updateContents();
				else
					winToUpdate->objslistframe->updateContents();
			}

			listObjToUpdate = fileListObjAtPath(destpath);
			if(listObjToUpdate)
				listObjToUpdate->myobjsframe->updateContents();

			// Aggiungere le custom icons, l'editor ecc...				
		}

// ----------------------- Link -----------------------		
		if(strcmp(operation, XWLinkOperation) == 0) {
			token = strtok(NULL, delims);
			token = strtok(NULL, delims);
			strcpy(destpath, token);

			winToUpdate = winAtPath(destpath);
			if(winToUpdate) {
				if(winToUpdate->viewtype == BYICONS || winToUpdate->viewtype == BYSMICONS)
					winToUpdate->objsframe->updateContents();
				else
					winToUpdate->objslistframe->updateContents();
			}

			listObjToUpdate = fileListObjAtPath(destpath);
			if(listObjToUpdate)
				listObjToUpdate->myobjsframe->updateContents();
		}

// ---------------------- Delete ----------------------				
		if(strcmp(operation, XWDestroyOperation) == 0) {
			token = strtok(NULL, delims);
			strcpy(destpath, token);

			winToUpdate = winAtPath(destpath);
			if(winToUpdate) {
				if(winToUpdate->viewtype == BYICONS || winToUpdate->viewtype == BYSMICONS)
					winToUpdate->objsframe->updateContents();
				else
					winToUpdate->objslistframe->updateContents();
			}

			listObjToUpdate = fileListObjAtPath(destpath);
			if(listObjToUpdate) 
				listObjToUpdate->myobjsframe->updateContents();
			
			propagatePathChangeInWins(destpath, NULL);

			// Aggiungere le custom icons, l'editor ecc...	
		}			

// ---------------------- Rename ----------------------				
		if(strcmp(operation, XWRenameOperation) == 0) {
			token = strtok(NULL, delims);
			strcpy(srcpath, token);
			token = strtok(NULL, delims);
			strcpy(destpath, token);

			winToUpdate = winAtPath(deleteLastPathComponent(srcpath));
			if(winToUpdate) {
				if(winToUpdate->viewtype == BYICONS || winToUpdate->viewtype == BYSMICONS)
					winToUpdate->objsframe->updateContents();
				else
					winToUpdate->objslistframe->updateContents();
			}

			listObjToUpdate = fileListObjAtPath(deleteLastPathComponent(srcpath));
			if(listObjToUpdate) 
				listObjToUpdate->myobjsframe->updateContents();
			
			propagatePathChangeInWins(srcpath, destpath);
			
			// Aggiungere le custom icons, l'editor ecc...	
		}
		
// -------------------- Duplicate ---------------------				
		if(strcmp(operation, XWDuplicateOperation) == 0) {
			token = strtok(NULL, delims);
			strcpy(srcpath, token);
			token = strtok(NULL, delims);
			strcpy(destpath, token);

			winToUpdate = winAtPath(destpath);
			if(winToUpdate) {
				if(winToUpdate->viewtype == BYICONS || winToUpdate->viewtype == BYSMICONS)
					winToUpdate->objsframe->updateContents();
				else
					winToUpdate->objslistframe->updateContents();
			}

			listObjToUpdate = fileListObjAtPath(destpath);
			if(listObjToUpdate) 
				listObjToUpdate->myobjsframe->updateContents();

			// Aggiungere le custom icons, l'editor ecc...	
		}

	}	
	
	if(strcmp(token, XWDidSetFileAttributesNotification) == 0) {
		token = strtok(NULL, delims);
		strcpy(destpath, token);

		fobj = fileObjAtPath(destpath);
		if(fobj) {
			fobj->updateMyData();
		} else {
			flobj = fileListObjAtPath(destpath);
			if(flobj)
				flobj->updateMyData();
		}
	}
		
	XFree(serverdata);
	XDeleteProperty(dspl, appRootWin, atom);
}

void mfinderapp::connectionError(char *err)
{
	dialog = new Dialog(0L, "Error", TRUE, 0, TRUE, TRUE, FALSE, FALSE, err);
	int m = dialog->exec();
	exit(0);
}
